/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef MBOXGRAB_H
#define MBOXGRAB_H MMBOXGRAB_H

// the app
#include"../../main/application.hpp"

// stored Interfaces
#include "nmm/interfaces/general/progress/IProgress.hpp"
#include "nmm/interfaces/video/osd/manager/ITextView.hpp"
#include "nmm/interfaces/video/osd/manager/IPlaneDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IScrollDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IBorderDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IHeaderDecorator.hpp"

namespace NMM { 

  class GrabState 
    : public MMBoxState {
  public:
    
    GrabState(string, MMBoxApplication*);
    ~GrabState();
    
    // methods from the MMBoxState interface
    Result doCreate(string);
    Result doKill();
    
    Result doInitialize(string);
    Result doUninitialize();
    
    Result Back();
    Result receiveMessage(string) ;
    
    // EventListener methods
    Result finished();
    Result startTrack(string&);
    Result setProgress(u_int64_t&, u_int64_t&);
    
  private:
    ///! the total count of tracks on the AudioCD being grabbed
    unsigned int total_tracks;
    ///! the currently grabbed track
    string actual_track_name;
    ///! the last values received as progress
    Rational current_progress;

    // EventListener objects
    TEDObject1<GrabState, string>               startTrack_listener;
    TEDObject2<GrabState, u_int64_t, u_int64_t> setProgress_listener;
    TEDObject0<GrabState>                       finished_listener;

    // own Nodes
    INode* cdda;
    INode* enc;
    INode* write;
    
    // Interfaces
    IProgress* iprogress;

    // widgets
    ITextView* text;
    IPlaneDecorator* plane;
    IBorderDecorator* border;
    IScrollDecorator* scroll;
    IHeaderDecorator* header;
    
    // window
    IWindow* win;
    
    // private helpers
    ///! this method cleans up by releasing and deleting all allocated
    //   stuff
    void cleanup();

    ///! these to methods allocate and deallocate the OSD window
    void createWindow();
    void deleteWindow();
  };
}
  
#endif //MMBOXGRAB_H
