/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Florian Winter <fw@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef __GRABDVDAPPLICATION__
#define __GRABDVDAPPLICATION__

#include "../../main/application.hpp"
//#include "../../main/mmboxstate.hpp"
#include "nmm/utils/DirReader.hpp"

#include "nmm/interfaces/base/INode.hpp"
#include "nmm/interfaces/device/dvd/IDVD.hpp"
#include "nmm/interfaces/video/osd/manager/IOSDManager.hpp"
#include "nmm/interfaces/video/osd/manager/IPlaneDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IScrollDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IHeaderDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IBorderDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/ITextView.hpp"
#include "nmm/interfaces/video/osd/manager/IWindow.hpp"

#include "nmm/plugins/video/osd/manager/OSDElements/doublelistunit.hpp"

#include <fstream>

namespace NMM {

  class DVDGrabState : public MMBoxState {
  public:
  
    enum { MAX_DVD_TITLES = 100 };
  
    enum State { TITLE_SELECT=1, CHAPTER_SELECT=2, OPTIONS=4, GRABBING=8 };
  
    DVDGrabState( string id,  MMBoxApplication* _app );
    ~DVDGrabState();
  
    Result doCreate(string);
    Result doKill();
  
    Result doInitialize(string);
    Result doUninitialize();
  
    Result Back();
    Result Stop();
    Result Up();
    Result Down();
    Result Left();
    Result Right();
    Result Return();
    Result receiveMessage( string message ) ;
  
    Result startTrack(string&);
    Result endTrack();
    Result setFrameCount( unsigned int&, float& );
    Result setProgress( u_int64_t&, u_int64_t& );
    
    /**
     * show option in options list
     * \param id ID of teh option in MMBoxConfig
     */
    Result showOption( string id, string shortform );
  private:
  
    list< string > registry_list;
    DirReader	     dirreader;
  
    /* nodes */
    INode* dvd;
    INode* demux;
    INode* video_dec;
    INode* spu_dec;
    INode* overlay;
    INode* deinterlacer;
    INode* scaler;
    INode* video_enc;
    INode* ac3_dec;
    INode* audio_enc;
    INode* av_mux;
    INode* writer;
    
    /* interfaces */
    IDVD* dvd_interface;
    
    /* handlers */
    TEDObject1<DVDGrabState, string> startTrack_listener;
    TEDObject0<DVDGrabState> endTrack_listener;
    TEDObject2<DVDGrabState, unsigned int, float> setFrameCount_listener;
    TEDObject2<DVDGrabState, u_int64_t, u_int64_t> setProgress_listener;
    
    /* options */
    string device;
    string filename;
    string root_path;
    string dest_dir;
    int audio_track;
    int title;
    int chapter;
    int subtitle_stream;
    int video_bitrate;
    int audio_bitrate;
    int audio_quality;
    bool ac3_writethrough;
    bool no_subtitles;
    bool video_deinterlace;
    bool video_scale;
    bool auto_aspect;
    bool use_ffmpeg;
  
    /* dvd information */
    int num_titles;
    int num_chapters[MAX_DVD_TITLES];
  
    /* state */
    State state;
  
    /* visual parts */
    INode*             osd_node;
    IOSDManager*       manager;
    ITextView*	       text;
    IPlaneDecorator*   plane;
    IBorderDecorator*  border;
    IScrollDecorator*  scroll;
    IHeaderDecorator*  header;
    IWindow*           win;
    DoubleListUnit*    dlunit;
    
    Rational current_progress;
    
    /* configuration */
    MMBoxConfig*       config;
    
    map<string, string> shown_options;
    
    void setState(State state);
  
    void createWindow();
    void deleteWindow();
  
    void registerProgressListener();
    void deRegisterProgressListener();
  
    Result startGrabbing();
    Result startTitleMenu();
    Result startChapterMenu();
    Result startOptionsMenu();
    
    Result initDVD();
    void deinitDVD();
  
    void createNodes();
    void deleteNodes();
  
    void initNodes();
    void deinitNodes();
  
    void connectNodes();
    void disconnectNodes();
  
    void startNodes();
    void stopNodes();
  
    void flushNodes();
    void lowerNodesPriority();
    
    void initOptionsList();
    void readConfig();
  };
}


#endif //__GRABDVDAPPLICATION__
