/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef AUTOMENU_HPP
#define AUTOMENU_HPP

#include "../../main/mmboxstate.hpp"

// include used interfaces
#include "nmm/interfaces/video/osd/manager/IWindow.hpp"
#include "nmm/interfaces/video/osd/manager/IOSDManager.hpp"
#include "nmm/interfaces/video/osd/manager/IWidgetFactory.hpp"
#include "nmm/interfaces/video/osd/manager/IRadioButtons.hpp"

namespace NMM {

  class AutoMenu : 
    public MMBoxState {
  protected:
    IRadioButtons* buttons;
    IOSDManager* manager;
    IWidgetFactory* factory;
    IWindow* win;
  
    bool first_init;
  
  public:
  
    /**
     * constructor needs a string id that identifies the class
     * @param id the menu id string
     */
    AutoMenu( string, MMBoxApplication* );
    ~AutoMenu();
  
    /**
     * These functions are passed to the RadioButtons object.
     */
    void addButton( int buttonid, string label, int x, int y, string inactive_id, string active_id, string select_id );
    void setStep( unsigned int s );
    void setDimension( unsigned int w, unsigned int h );
  
    Result Up();
    Result Down();
    Result Left();
    Result Right();
    Result Return();
    Result Back();
    Result doCreate( string );
    Result doKill();
    Result doInitialize( string );
    Result doUninitialize();
  
  };

}

#endif //AUTOMENU_HPP
