/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002, 2003
 *                    Network-Multimedia Workgroup,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        David Maass (dama@grahics.cs.uni-sb.de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef __TRANSCODE__
#define __TRANSCODE__

#include <vector>
#include <string>

#include "../../main/application.hpp"
#include "../../main/mmboxstate.hpp"
#include "../../main/PlaylistTemplateFunctions.hpp"
#include "../../main/TranscodeTemplateFunctions.hpp"
#include "nmm/plugins/video/osd/manager/OSDElements/OSDElementComposit.hpp"
#include "nmm/plugins/video/osd/manager/OSDElements/tableunit.hpp"
#include "nmm/base/registry/GraphDescription.hpp"

#include "../../gui/WindowSelectorTranscode.hpp"
#include "../../gui/TranscodeConfig.hpp"

//#include "JobManager.hpp"
#include "TranscodingGroup.hpp"
#include "TranscoderWebInterface.hpp"

namespace NMM {
  class JobManager;
  class TranscodingGroup;
  class Transcode : public MMBoxState,
                    public PlaylistTemplateFunctions,
                    public TranscoderWebInterface,
                    public TranscodeTemplateFunctions {

  public: 
    /**
     * constructor needs a string id that identifies the class
     * \param id the menu id string
     */
    Transcode( string, MMBoxApplication* );
    ~Transcode();

    Result Up();
    Result Down();
    Result Left();
    Result Right();
    Result Return();
    Result Back();
    Result doSelectTrackWindow(string url);
    Result doSelectPlaylistWindow(string url);
    Result doSelectGDWindow(GraphDescription* gd, list<NodeConfiguration>* nodeConfig, const string& des);
    string mapFormat(const string&);

    void   setConfiguration(const string&,const string&);
    void   doChangeState(string state);
    void   createStatusWindow();
    void   updateColorBar();

    Result colorKeyRed();
    Result colorKeyGreen();
    Result colorKeyYellow();
    Result colorKeyBlue();

    Result doCreate( string );
    Result doKill();
    Result doInitialize( string );
    Result doUninitialize();
    Result receiveMessage( string );

    Result showOption(string,string);

    void setWinProperties( int , OSDElementComposit::win_properties_t*);
    void showJobs(const int& i);
    void showGroups();
    void updateInfo();
    void showJobInfo(const int& i,const int& j);
    void showGroupInfo(const int&i);

    /* TranscoderWebinterface */
    list<pair<int,string> > getTranscodingGroups();
    list<string>            getTranscodingJobs(const int& group_number);
    list<string>            getGroupInformation(const int& group_number);
    list<string>            getJobInformation(const int& group_number, const int& job_number);

  protected:
    WindowSelectorTranscode* playlist_gui;
    TranscodeConfig* config_gui;
    ListUnit* group_list;
    TableUnit* job_list;
    ListUnit* info_window;

    //for updating the windows
    ThreadMutex updatemutex;
    /** mmbox rootpath */
    string       rootpath,gdpath,transpath,cdrompath;
    enum mode{JOBS=0, FILES, CONFIG} _mode;
    /** OSD manager interface */
    IOSDManager* manager;
    vector<TranscodingGroup*> _transcoding_groups;
    string _url;
    bool distribute,priori,use_localhost;
    int parallel;
    map<string, string> shown_options;
    list< pair<string,int> > reg_list;
    JobManager* jobm;
    string transcode_format;
    enum spymode{NONE=0, RAW, TRANS} _spymode;
    enum focus{GROUPLIST=0,JOBLIST} _focus;
  };
}


#endif
