#include "RecordHandler.hpp"

#include <fstream>
#include <math.h>
#include <stdio.h>

#include "nmm/utils/utils.hpp"
#include "nmm/interfaces/misc/tvguide/ITVGuide.hpp"

namespace NMM{
  RecordHandler::RecordHandler(ITVTimer *itvt)
    : WebRequestHandler("record"),
      itvtimer(itvt)
  {}
  RecordHandler::~RecordHandler(){
  }
  
  void RecordHandler::doGet (WebServerRequest *pRequest, WebServerResponse *pResponse){
    //cerr << "Method  : " << pRequest->getMethod() << endl;
    //cerr << "Protocol: " << pRequest->getProtocol() << endl;
    //cerr << "URI     : " << pRequest->getRequestURI() << endl;
    //cerr << "Query   : " << pRequest->getQueryString() << endl;
    //cerr << "RemAddr : " << pRequest->getRemoteAddress() << endl;

    if(!itvtimer){
      pResponse->write("<html><head><title>MMBox - TVTimer</title></head><body>no TVTimer available<br></body></html>");
      return;
    }

    string requestOption = pRequest->getQueryString();
    requestOption = requestOption.substr(0,requestOption.find_first_of('='));

    if(requestOption.empty()){

      showTimerList(pResponse);

    }else if(requestOption == "show"){
        pResponse->write("<html><head><title>Show</title></head><body>Test Successful <br></body></html>");

    }else if(requestOption == "play"){
      string requestParams = pRequest->getQueryString().substr(pRequest->getQueryString().find_first_of('=')+1);
      size_t pos = requestParams.find_first_of(':');
      string startTime = requestParams.substr(0,pos);
      string stopTime  = requestParams.substr(pos+1, pos);
      
      pos =  requestParams.find_last_of(':');
      string channel   = requestParams.substr(pos+1);
      
      list<TVTimerEntry> timers = itvtimer->getTimers(startTime.substr(0, 8), channel);
      
      for(list<TVTimerEntry>::iterator i = timers.begin(); i != timers.end(); i++){
        if(stringToTime(startTime) == i->getStartTime() && stringToTime(stopTime) == i->getEndTime()){
          //what I plan: Set up a graph to play the timer. source is specified in TVTimerEntry, Sink-host comes with request.
          //Problems: relasing of graph, when and how to detect? controlling of the graph
          //For first testing, we can just make a page with a link to release the graph
          
          break;
        }
      }
    }else if(requestOption == "delete"){
      string requestParams = pRequest->getQueryString().substr(pRequest->getQueryString().find_first_of('=')+1);
      size_t pos = requestParams.find_first_of(':');
      string startTime = requestParams.substr(0,pos);
      string stopTime  = requestParams.substr(pos+1, pos);
      
      pos =  requestParams.find_last_of(':');
      string channel   = requestParams.substr(pos+1);
      
      pos = channel.find_first_of("%20");
      if(pos != string::npos){
        channel.replace(pos, 3, " ");
      }
      
      list<TVTimerEntry> timers = itvtimer->getTimers(startTime.substr(0, 8), channel);
      
      for(list<TVTimerEntry>::iterator i = timers.begin(); i != timers.end(); i++){
        if(stringToTime(startTime) == i->getStartTime() && stringToTime(stopTime) == i->getEndTime()){
          try{
            itvtimer->deleteTimer(i->getID(), i->getUserName());
          }catch(const Exception &e){}
          
          break;
        }
      }
      showTimerList(pResponse);

    }else if(requestOption == "program"){
      string requestParams = pRequest->getQueryString().substr(pRequest->getQueryString().find_first_of('=')+1);
      size_t pos = requestParams.find_first_of(':');
      string startTime = requestParams.substr(0,pos);
      string stopTime  = requestParams.substr(pos+1, pos);
      
      pos =  requestParams.find_last_of(':');
      string channel   = requestParams.substr(pos+1);
      
      pos = channel.find_first_of("%20");
      if(pos != string::npos){
        channel.replace(pos, 3, " ");
      }


      TVTimerEntry *entry = new TVTimerEntry(UserInfo().getUsername());
      entry->setStartTime(stringToTime(startTime));
      entry->setEndTime(stringToTime(stopTime));
      entry->setChannelName(channel);
      
      Result res = FAILURE;
      try{
        res = itvtimer->programTimer(*entry);
      }catch(const Exception &e){}

      if(res == SUCCESS){
        pResponse->write("<html><head>");
        ifstream f((getNMMResourceDir() + "/mmbox/webhandler/record.css").c_str());
        char line[1000];
        
        if(f.fail()){
        }else{
          while(!f.eof()){
            f.getline(line, 1000);
            pResponse->write(string(line));
          }
          f.close(); 
        }
        
        pResponse->write("<body>Timer successfully programmed <br><br>");
        //pResponse->write("<a href='"+ pRequest->getRequestURI() + "?" + pRequest->getQueryString() + "'>Back</a>");
        pResponse->write("</body></html>");
      }else{
        pResponse->write("<html><head>");
        ifstream f((getNMMResourceDir() + "/mmbox/webhandler/record.css").c_str());
        char line[1000];
        
        if(f.fail()){
        }else{
          while(!f.eof()){
            f.getline(line, 1000);
            pResponse->write(string(line));
          }
          f.close(); 
        }
        pResponse->write("<body>Error programming timer <br><br>");
        //pResponse->write("<a href='"+ pRequest->getRequestURI() +"?" + pRequest->getQueryString() + "'>Back</a>");
        pResponse->write("</body></html>");

      }
    }else if(requestOption == "newTimer"){
      pResponse->write("<html>");
      pResponse->write("<head>");
      pResponse->write("<title> New Timer </title>");

      ifstream f((getNMMResourceDir() + "/mmbox/webhandler/record.css").c_str());
      char line[1000];
      
      if(f.fail()){
      }else{
        while(!f.eof()){
          f.getline(line, 1000);
          pResponse->write(string(line));
        }
        f.close(); 
      }
      
      pResponse->write("<script type=\"text/javascript\">");
      pResponse->write("<!--");
      pResponse->write("function verify(){");
      pResponse->write("if(document.Formular.day.value == \"\"){");
      pResponse->write("alert(\"Please enter Day\");");
      pResponse->write("document.Formular.end.focus();");
      pResponse->write("return false;");
      pResponse->write("}");
      pResponse->write("if(document.Formular.moth.value == \"\"){");
      pResponse->write("alert(\"Please enter Month\");");
      pResponse->write("document.Formular.end.focus();");
      pResponse->write("return false;");
      pResponse->write("}");
      pResponse->write("if(document.Formular.year.value == \"\"){");
      pResponse->write("alert(\"Please enter Year\");");
      pResponse->write("document.Formular.end.focus();");
      pResponse->write("return false;");
      pResponse->write("}");
      pResponse->write("if(document.Formular.start.value == \"\"){");
      pResponse->write("alert(\"Please enter Start-Time\");");
      pResponse->write("document.Formular.start.focus();");
      pResponse->write("return false;");
      pResponse->write("}");
      pResponse->write("if(document.Formular.end.value == \"\"){");
      pResponse->write("alert(\"Please enter Stop-Time\");");
      pResponse->write("document.Formular.end.focus();");
      pResponse->write("return false;");
      pResponse->write("}");
      pResponse->write("}");
      pResponse->write("//-->");
      pResponse->write("</script>");

      pResponse->write("<body>");
      pResponse->write("<h1>MMBox - TVTimer : New Timer</h1>");

      pResponse->write("<form name=\"Formular\" action=\"/record\" method=\"get\" onSubmit=\"return verify()\">");
      pResponse->write("<table name='newTimer'>");
      pResponse->write("<tr>");
      pResponse->write("<td>Day<br><input name=\"day\" type=\"text\" size=\"2\" maxlength=\"2\"></td>");
      pResponse->write("<td>Month<br><input name=\"month\" type=\"text\" size=\"2\" maxlength=\"2\"></td>");
      pResponse->write("<td>Year<br><input name=\"year\" type=\"text\" size=\"4\" maxlength=\"4\"></td>");
      pResponse->write("</tr><tr>");
      pResponse->write("<td>Start<br><input name=\"start\" type=\"text\" size=\"4\" maxlength=\"4\"></td>");
      pResponse->write("<td>End<br><input name=\"end\" type=\"text\" size=\"4\" maxlength=\"4\"></td>");

      if(itvtimer){
        TVChannelList tvchannellist = itvtimer->getChannelConfig();
        pResponse->write("<tr><td colspan='2'>");
        pResponse->write("<select name=\"channel\" size=\"5\">");
        list<TVChannel> tvchannels = tvchannellist.getChannels();
        for(list<TVChannel>::iterator  i = tvchannels.begin(); i != tvchannels.end(); i++){
          string optionString = "<option";
          if(i == tvchannels.begin()) optionString += " selected";
          optionString += ">";
          optionString += i->getChannelName();
          optionString += "</option>";
          pResponse->write(optionString);
        }
        
        pResponse->write("</select>");
        pResponse->write("</td>");
      }

      pResponse->write("<td>");
      pResponse->write("<select name=\"period\" size=\"5\">");
      pResponse->write("<option value='1'>Daily</option>");      
      pResponse->write("<option value='2'>Weekly</option>");      
      pResponse->write("<option value='3'>Weekdays</option>");      
      pResponse->write("<option value='4'>Weekends</option>");      
      pResponse->write("<option value='5'>Monthly</option>");      
      pResponse->write("<option value='6'>Random</option>");      
      pResponse->write("<option value='7'>Random on channel</option>");      
      pResponse->write("<option value='8'>Random on channel and time</option>");      
      pResponse->write("</select>");
      pResponse->write("</td></tr>");
      
      pResponse->write("<tr><td>");
      pResponse->write("<input type=\"submit\">");
      pResponse->write("</td><td>");
      pResponse->write("<input type=\"reset\">");
      pResponse->write("</td></tr>");

      pResponse->write("</table>");
      pResponse->write("</form>");

    }else{
      //program a new timer
      string queryParameters = pRequest->getQueryString();
      size_t pos = string::npos;
      size_t oldPos = 0;
      int len = 0;
      TVTimerEntry *entry = new TVTimerEntry(UserInfo().getUsername());
      string date; 
      do{
        pos = queryParameters.find_first_of('&', oldPos);
        len = pos - oldPos;
        string parameter = queryParameters.substr(oldPos, len);
        oldPos = pos + 1;

        size_t paramPos = parameter.find_first_of('=');
        string param = parameter.substr(0,paramPos);
        string value = parameter.substr(paramPos+1);

        if(param == "day"){
          if(value.length() == 1) value = "0" + value;
          date += value + ".";
        }else if(param == "month"){
          if(value.length() == 1) value = "0" + value;
          date += value + ".";
        }else if(param == "year"){
          if(value.length() == 2) value = "20" + value;
          date += value + " ";
        }else if(param == "start"){
          string tmptime = date + value.substr(0,2) + ":" + value.substr(2,2);
          time_t startTime = transformTime(tmptime);
          if(startTime >= 0){
            entry->setStartTime(startTime);
          }else{
            pResponse->write("<html><head><title>Show</title></head><body>Programming failed <br> Wrong Time-Format</body></html>");
            return;
          }
        }else if(param == "end"){
          string tmptime = date + value.substr(0,2) + ":" + value.substr(2,2);
          time_t endTime = transformTime(tmptime);
          if(endTime >= 0){
            entry->setEndTime(endTime);
          }else{
            pResponse->write("<html><head><title>Show</title></head><body>Programming failed <br> Wrong Time-Format</body></html>");
            return;
          }
        }else if(param == "channel"){
          transform(value);
          entry->setChannelName(value);
        }else if(param == "period"){
          entry->setPeriod((TVTimerEntry::repmode_e)atoi(value.c_str()));
        }
      }while(pos != string::npos);

      try{
        if(itvtimer->programTimer(*entry) == SUCCESS){
          showTimerList(pResponse);
        }else{
          pResponse->write("<html><head><title>Show</title></head><body>Programming failed <br></body></html>");
        }
      }catch(const Exception &e){
          pResponse->write("<html><head><title>Show</title></head><body>Programming failed <br></body></html>");
      }
    }
  }

  void RecordHandler::doPost(WebServerRequest *, WebServerResponse *pResponse){
    showTimerList(pResponse);
  }



  /* ----------------- Private Functions ------------------------- */
  void RecordHandler::showTimerList(WebServerResponse *pResponse){
    pResponse->write("<html>");
    pResponse->write("<head>");
    pResponse->write("<title> Record-Site </title>");

    ifstream f((getNMMResourceDir() + "/mmbox/webhandler/record.css").c_str());
    char line[1000];
    
    if(f.fail()){
    }else{
      while(!f.eof()){
        f.getline(line, 1000);
        pResponse->write(string(line));
      }
      f.close(); 
    }

    pResponse->write("</head>");
    
    pResponse->write("<body>");
    
    pResponse->write("<h1>MMBox - TVTimer : Timerlist</h1>");
    pResponse->write("<table>");
    
    list<TVTimerEntry> timers = itvtimer->getTimers();
    for(list<TVTimerEntry>::iterator i = timers.begin(); i != timers.end(); i++){
      pResponse->write("<tr>");
      
      char chtime[20];
      time_t tmptime = i->getStartTime();
      strftime(chtime, sizeof(chtime), "%d.%m.%Y %H:%M", localtime(&tmptime));
      pResponse->write("<td>" + string(chtime) + "</td>");
      
      tmptime = i->getEndTime();
      strftime(chtime, sizeof(chtime), "%d.%m.%Y %H:%M", localtime(&tmptime));
      pResponse->write("<td>" + string(chtime) + "</td>");
      
      pResponse->write("<td>" + i->getChannelName() + "</td>");
      
      switch(i->getStatus()){
      case TVTimerEntry::RECORDED:
        pResponse->write("<td>Recorded</td>");
        break;
      case TVTimerEntry::RECORDING:
        pResponse->write("<td>Recording</td>");
        break;
      case TVTimerEntry::RECORDED_WITH_ERROR:
        pResponse->write("<td>Error</td>");
        break;
      default:
        pResponse->write("<td>Sleeping</td>");
      }
      
      string playUrl = "<td><a href='/record?play=";
      tmptime = i->getStartTime();
      strftime(chtime, sizeof(chtime), "%Y%m%d%H%M", localtime(&tmptime));
      playUrl += string(chtime) + ":";
      tmptime = i->getEndTime();
      strftime(chtime, sizeof(chtime), "%Y%m%d%H%M", localtime(&tmptime));
      playUrl += string(chtime) + ":";
      playUrl += i->getChannelName() + "'>View</a></td>";
      //pResponse->write(playUrl);
      
      string deleteUrl = "<td><a href='/record?delete=";
      tmptime = i->getStartTime();
      strftime(chtime, sizeof(chtime), "%Y%m%d%H%M", localtime(&tmptime));
      deleteUrl += string(chtime) + ":";
      tmptime = i->getEndTime();
      strftime(chtime, sizeof(chtime), "%Y%m%d%H%M", localtime(&tmptime));
      deleteUrl += string(chtime) + ":";
      deleteUrl += i->getChannelName() + "'>Delete</a></td>";
      pResponse->write(deleteUrl);
      pResponse->write("</tr>");
    }
    
    pResponse->write("</table>");
    
    pResponse->write("</body>");
    
    pResponse->write("</html>");
  }

  void RecordHandler::transform(string &s){
    for (unsigned int i = 0; i <= s.length(); i++){
      if(s[i] == '+') s.replace(i, 1, " ");
      if(s[i] == '%'){
        string hexvalue = s.substr(i+1, 2);
        unsigned int hexint;
        sscanf(hexvalue.c_str(), "%X", &hexint);
        char c[2] = {(char)hexint, '\0'};
        string tmp((char*)&c);
        s.replace(i, 3, tmp);
        
      }
    }
  }


  time_t RecordHandler::transformTime(string s){
    transform(s);
    time_t retval = time(NULL);
    struct tm *ret_tm = localtime(&retval);
    int y, mon, d, hour, min;
    try{
      d = atoi(s.substr(0,2).c_str());
      mon = atoi(s.substr(3,2).c_str());
      y = atoi(s.substr(6,4).c_str());
      hour = atoi(s.substr(11,2).c_str());
      min  = atoi(s.substr(14,2).c_str());

      ret_tm->tm_mday = d;
      ret_tm->tm_mon  = mon-1;
      ret_tm->tm_year = y-1900;
      ret_tm->tm_hour = hour;
      ret_tm->tm_min  = min;
      ret_tm->tm_sec  = 0;
      
      return mktime(ret_tm);
    }catch(...){
      return 0;
    }
  }  
}
