/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#include "mycamknob.h"
#include <iostream>
#include "back.xpm"
#include "forward.xpm"

MyCamKnob::MyCamKnob(QWidget *parent, const char *name )
  : QWidget(parent,name)
{
  actual_value = 0;
  gridLayout = new QGridLayout(this,3,3);

  init();

  gridLayout->activate();
}

MyCamKnob::~MyCamKnob(){
}

void MyCamKnob::init()
{
  // Init
  QPixmap *pright  = new QPixmap(forward);
  QPixmap *pleft  = new QPixmap(back);

  myKnob  = new CamKnob(this);
  myLine  = new QSpinBox(0,255,1,this);
  myLine->setFixedHeight(20);

  myLabel  = new QLabel("NoName",this);
  myLabel->setFixedHeight(20);

  right    = new QToolButton(this);
  right->setAutoRepeat(true);

  left    = new QToolButton(this);
  left->setAutoRepeat(true);

  //connect
  connect(left, SIGNAL(clicked()), this, SLOT(decValue()));
  connect(right, SIGNAL(clicked()), this, SLOT(incValue()));
  connect(myLine, SIGNAL(valueChanged(int)), this, SLOT(setValue(int)));
  connect(myKnob, SIGNAL(valueChanged(int)), this, SLOT(setValue(int)));
 

  // set some default Values
  myKnob->setMinimumSize(myKnob->sizeHint());
  myKnob->setRange(0,255,1);

  myLine->setMinimumWidth(40);
  myLine->setFixedHeight(20);

  myLabel->setMinimumSize(myLabel->sizeHint());
  myLabel->setAlignment(AlignCenter);
  myLabel->setMinimumWidth(50);

  right->setFixedSize(right->sizeHint());
  right->setPixmap(*pright);

  left->setFixedSize(left->sizeHint());
  left->setPixmap(*pleft);

  //add to Layout
  gridLayout->addMultiCellWidget(myLabel, 0,0,0,2);
  gridLayout->addMultiCellWidget(myKnob,1,1,0,2);
  gridLayout->addWidget(left,2,0);
  gridLayout->addWidget(myLine,2,1);
  gridLayout->addWidget(right,2,2);
}


void MyCamKnob::setLabel(const char *newLabel)
{
  myLabel->setText( newLabel );
}


void MyCamKnob::setRange(int i, int j, int k)
{
  myKnob->setRange((double)i,(double)j,(double)k);

  myLine->setRange(i,j);
  myLine->setValue(actual_value);
}

void MyCamKnob::incValue()
{
  setValue(actual_value+1);
}

void MyCamKnob::decValue()
{
  setValue(actual_value-1);
}

void MyCamKnob::setValue(int new_value)
{
 // cout << "Set Value to " << new_value << endl;
  if (new_value != actual_value) {
    actual_value = new_value;
    myLine->setValue(actual_value);
    myKnob->setValue(actual_value);
    if(actual_value == myLine->minValue())
       left->setEnabled(false);
    else left->setEnabled(true);

    if(actual_value == myLine->maxValue())
       right->setEnabled(false);
    else
      right->setEnabled(true);

    emit valueChanged(actual_value);
  }

}

