/*
 * Qwt Widget Library (http://qwt.sf.net)
 * Copyright (C) 1997   Josef Wilgen
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, 
 * USA.
 */


#ifndef QWT_KNOB_H
#define QWT_KNOB_H
#include "qwt_sldbase.h"
#include "qwt_sclif.h"
#include <qcolor.h>

class QwtKnob 
	: public QwtSliderBase , 
	  public QwtScaleIf
{
public:
    enum Symbol { Line, Dot };
	
private:

    int d_borderWidth;
    int d_borderDist;
    int d_hasScale;
    int d_scaleDist;
    int d_maxScaleTicks;
    int d_newVal;
    int d_knobWidth;
    int d_dotWidth;

    Symbol d_symbol;
    double d_angle;
    double d_oldAngle; 
    double d_totalAngle;
    double d_nTurns;
   
    QRect d_kRect;
    QColor d_faceColor;
    QColor d_markerColor;

    void recalcAngle();
	
public:

    QwtKnob(QWidget* parent = 0, const char *name = 0);
    ~QwtKnob();

    void setKnobWidth(int w);
    void setTotalAngle (double angle);
    void setBorderWidth(int bw);
	
		QPoint *mypoint;
private:
	
    void valueChange();
    void rangeChange();
	
    void drawKnob(QPainter *p, const QRect &r);
    void drawMarker(QPainter *p, double arc, const QColor &c);

    void paintEvent(QPaintEvent *e);
    void resizeEvent(QResizeEvent *e);
public:
    double getValue(const QPoint &p);
private:
    void getScrollMode( const QPoint &p, int &scrollMode, int &direction );
    void scaleChange();
    void fontChange();

};


#endif



