/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christian Gerstner <gerstner@graphics.cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include <qvalidator.h>
#include "GuiWidget.hpp"

GuiWidget::GuiWidget(NMM::NMMApplication& app_) : //, const char* name) :
  app(app_),
  labelOutFileName(NULL),
  labelVNCHostAndPort(NULL),
  labelVNCPassword(NULL),
  labelVideoBitrate(NULL),
  labelAudioBitrate(NULL),
  labelWidth(NULL),
  labelHeight(NULL),
  editOutFileName(NULL),
  editVNCHostAndPort(NULL),
  editVNCPassword(NULL),
  lcdVBR(NULL),
  outFileName(NULL),
  start(NULL),
  stop(NULL),
  preview(NULL),
  quit(NULL),
  checkBoxDisplay(NULL),
  checkBoxVisca(NULL),
  checkBoxDeinterlacer(NULL),
  widthBox(NULL),
  heightBox(NULL),
  sliderVBR(NULL),
  sliderABR(NULL)
{
  // Output File
  labelOutFileName = new QLabel("Output file", this, "lofn");
  labelOutFileName->setGeometry(10,10, 90,20);
  editOutFileName = new QLineEdit(this, "lofn");
  editOutFileName->setGeometry(110,10, 200,20);
  outFileName = new QPushButton ("...", this, "ofn");
  outFileName->setFont( QFont( "Times", 12, QFont::Normal ) );
  outFileName->setGeometry(330,10, 20,20);

  // VNC Host and Port
  labelVNCHostAndPort = new QLabel("VNC Host/Port", this, "lvhap");
  labelVNCHostAndPort->setGeometry(10,40, 90,20);
  editVNCHostAndPort = new QLineEdit(this, "lvhap");
  editVNCHostAndPort->setGeometry(110,40, 200,20);

  // VNC Password
  labelVNCPassword = new QLabel("VNC Password", this, "lvp");
  labelVNCPassword->setGeometry(10,70, 90,20);
  editVNCPassword = new QLineEdit(this, "lvp");
  editVNCPassword->setGeometry(110,70, 200,20);
  editVNCPassword->setEchoMode(QLineEdit::Password);

  // start button
  start = new QPushButton( "Start", this, "start" );
  start->setFont( QFont( "Times", 12, QFont::Bold ) );
  start->setGeometry(110,250,65,27);

  // stop button
  stop = new QPushButton( "Stop", this, "stop" );
  stop->setFont( QFont( "Times", 12, QFont::Bold ) );
  stop->setGeometry(110,250,65,27);
  stop->hide();

  // preview button
  preview = new QPushButton( "Preview", this, "preview" );
  preview->setFont( QFont( "Times", 12, QFont::Bold ) );
  preview->setGeometry(178,250,64,27);
  
  // Video-Bitrate
  labelVideoBitrate = new QLabel("Video-Bitrate", this, "videobitrate");
  labelVideoBitrate->setGeometry(10,105, 80,10);
  lcdVBR  = new MyLineEdit( this, "lcdVBR" );
  QIntValidator* intValid = new QIntValidator (lcdVBR, "intvalidlcdvbr");
  intValid->setRange(1,2000);
  lcdVBR->setValidator(intValid);
  lcdVBR->setGeometry(320,100, 50,30);
  sliderVBR = new QScrollBar( Horizontal, this, "sliderVBR" );
  sliderVBR->setGeometry(110,105, 200,15);
  connect( sliderVBR, SIGNAL(valueChanged(int)), lcdVBR, SLOT(setMyText(int)) );
  sliderVBR->setRange( 1, 2000 );
  sliderVBR->setValue( 400 );

  // Audio-Bitrate
  labelAudioBitrate = new QLabel("Audio-Bitrate", this, "audiobitrate");
  labelAudioBitrate->setGeometry(10,140, 80,10);
  QLCDNumber* lcdABR  = new QLCDNumber( 4, this, "lcdABR" );
  lcdABR->setGeometry(320,135, 50,30);
  sliderABR = new QScrollBar( Horizontal, this, "sliderABR" );
  sliderABR->setGeometry(110,140, 200,15);
  connect( sliderABR, SIGNAL(valueChanged(int)), lcdABR, SLOT(display(int)) );
  sliderABR->setRange( 1, 320 );
  sliderABR->setValue( 160 );

  checkBoxDisplay = new QCheckBox("Use Display", this, "usedisplay");
  checkBoxDisplay->setGeometry(10,170, 150,15);
  checkBoxVisca = new QCheckBox("Use VISCA-Cam", this, "usevisca");
  checkBoxVisca->setGeometry(10,190, 150,15);
  checkBoxDeinterlacer = new QCheckBox("Deinterlace Video", this, "deinterlace");
  checkBoxDeinterlacer->setGeometry(10,210, 150,15);

  labelWidth = new QLabel("Width", this, "width");
  labelWidth->setGeometry(260,170, 50,20);
  widthBox = new QSpinBox(64, 768, 16, this, "widthBox");
  widthBox->setValue(384);
  widthBox->setGeometry(320,170, 50,20);
  labelHeight = new QLabel("Height", this, "height");
  labelHeight->setGeometry(260,200, 50,20);
  heightBox = new QSpinBox(64, 576, 16, this, "widthBox");
  heightBox->setValue(288);
  heightBox->setGeometry(320,200, 50,20);

  // quit button
  quit = new QPushButton( "Quit", this, "quit" );
  quit->setFont( QFont( "Times", 12, QFont::Bold ) );
  quit->setGeometry(245,250,65,27);
}

GuiWidget::~GuiWidget() {
  if (labelOutFileName) delete labelOutFileName;
  if (labelVNCHostAndPort) delete labelVNCHostAndPort;
  if (labelVNCPassword) delete labelVNCPassword;
  if (labelVideoBitrate) delete labelVideoBitrate;
  if (labelAudioBitrate) delete labelAudioBitrate;
  if (labelWidth) delete labelWidth;
  if (labelHeight) delete labelHeight;
  if (editOutFileName) delete editOutFileName;
  if (editVNCHostAndPort) delete editVNCHostAndPort;
  if (editVNCPassword) delete editVNCPassword;
  if (lcdVBR) delete lcdVBR;
  if (outFileName) delete outFileName;
  if (start) delete start;
  if (quit) delete quit;
  if (checkBoxDisplay) delete checkBoxDisplay;
  if (checkBoxVisca) delete checkBoxVisca;
  if (checkBoxDeinterlacer) delete checkBoxDeinterlacer;
  if (widthBox) delete widthBox;
  if (heightBox) delete heightBox;
}

void GuiWidget::setDefaultValues(QString outputfilename_, QString host_and_port_){
  editOutFileName->insert(outputfilename_);
  editVNCHostAndPort->setText(host_and_port_);
}

QPushButton* GuiWidget::getButtonStart() {
  return start;
}

QPushButton* GuiWidget::getButtonStop() {
  return stop;
}

QPushButton* GuiWidget::getButtonPreview() {
  return preview;
}

QCheckBox* GuiWidget::getCheckBoxDisplay() {
  return checkBoxDisplay;
}

QCheckBox* GuiWidget::getCheckBoxVisca() {
  return checkBoxVisca;
}

QCheckBox* GuiWidget::getCheckBoxDeinterlacer() {
  return checkBoxDeinterlacer;
}

QLineEdit* GuiWidget::getEditOutFileName () {
  return editOutFileName;
}

QLineEdit* GuiWidget::getEditVNCHostAndPort() {
  return editVNCHostAndPort;
}

QLineEdit* GuiWidget::getEditVNCPassword() {
  return editVNCPassword;
}

QPushButton* GuiWidget::getButtonOutFileName() {
  return outFileName;
}

QPushButton* GuiWidget::getButtonQuit() {
  return quit;
}

QSpinBox* GuiWidget::getWidthBox() {
  return widthBox;
}

QSpinBox* GuiWidget::getHeightBox() {
  return heightBox;
}

QScrollBar* GuiWidget::getSliderVBR() {
  return sliderVBR;
}

QScrollBar* GuiWidget::getSliderABR() {
  return sliderABR;
}
