/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Andrs Otn Urbano
 *                    email: andres.oton@gmail.com                    
 *                     
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_ADVANCEDREADNODE_HPP
#define NMM_ADVANCEDREADNODE_HPP NMM_ADVANCEDREADNODE_HPP

#include "nmm/plugins/file/GenericReadNode.hpp"

#include "nmm/interfaces/file/IFileSizeImpl.hpp"

#include "contrib/nmm/interfaces/file/INavFileImpl.hpp"
#include "contrib/nmm/interfaces/file/IFileMetaInformationImpl.hpp"

#include <time.h>
#include <sys/stat.h>

namespace NMM {

  class AdvancedReadNode :
    public GenericReadNode,
    public IFileMetaInformationImpl,
    public INavFileImpl,
    public IFileSizeImpl
  {

  public:
   
    AdvancedReadNode(const char *name = "AdvancedReadNode",
		    StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
		    int = StreamQueue::DEFAULT_SIZE);

    virtual ~AdvancedReadNode();

  protected:
  /** INavFile interface ***/
    virtual list<string> getFileList(const string& dir);

    virtual list<string> getDirectoryList(const string& dir);

  /** IFileMetaInformation interface ***/
    virtual bool fillMetaInformation(const string& name);

    virtual string getAccessTime();
    virtual long   getNumAccessTime();

    virtual string getCreationTime();
    virtual long   getNumCreationTime();

    virtual string getModifyTime();
    virtual long   getNumModifyTime();

    virtual string getTitle();

    virtual string getAlbum();

    virtual string getArtist();

    virtual string getGenre();

    virtual string getDate();

    virtual string getMimeType();

    virtual string getAudioFormat();

    virtual string getVideoFormat();

    virtual int getMode();

    virtual bool getIsDir();

    /** IFileSize */
    virtual bool hasFileSize();

    virtual void getFileSize(u_int64_t& bytes);

  private:
    /** Convert a time_t structure to string. */
    string timeString(time_t t);

    string getExtractInformation(string keyword);

    bool fillAVInfo();

    struct stat statbuf;
    bool filled;
    string file;
    string audiof;
    string videof;

  };

}

#endif //NMM_ADVANCEDREADNODE_HPP
