/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "CommunicationChannel.hpp"
#include "ConnectionManager.hpp"

namespace NMM {
  
  void CommunicationChannel::finishStrategyList(TSAddress::BindingType binding_type) {
    /* Determine default strategy */
    string default_strategy(SConnectionManager::getInstance().getDefaultStrategy(getTypeInfo(), binding_type));
  
    /* Check if default strategy is in the list */
    list<string>::iterator found(find(getStrategyList().begin(), getStrategyList().end(), default_strategy));
    if (found != getStrategyList().end()) {
      /* Move default strategy to the beginning of the list */
      getStrategyList().erase(found);
      getStrategyList().push_front(default_strategy);
    }
  }
  
  TypeInfo CommunicationChannel::getStaticTypeInfo() {
    return TypeInfo("CommunicationChannel", "NMM");
  }
  
}
