/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_SERVICEPROXY_HPP
#define NMM_SERVICEPROXY_HPP

#include "ProxyObject.hpp"
#include "ObjectAcceptor.hpp"
namespace NMM {

class ServiceProxy {
protected:
  ProxyObject* proxy_object;
  IObjectAcceptor* object_connector;
public:
  ServiceProxy()
  {
    proxy_object = 0;
    object_connector = 0;
  }

  ~ServiceProxy()
  {
    if(object_connector) {
      delete object_connector;
    }
  }

  template <typename T> T* getInterface() const
  { 
    return proxy_object->getInterface<T>();
  }

  template <typename T> T* getCheckedInterface() const
    throw (const InterfaceNotFoundException&) 
  { 
    return proxy_object->getCheckedInterface<T>();
  }
  
  void connectTo(const TSAddress& address, const string& service)
    throw (const BindingException&)
  {
    proxy_object = new ProxyObject(true);
    proxy_object->connectToObject(address);
    
    if(object_connector != 0) {
      delete object_connector;
      object_connector = 0;
    }

    object_connector = proxy_object->getInterface<IObjectAcceptor>();
    if(object_connector == 0) {
      throw BindingException();
    }
    object_connector->connectToService(service);
  }
};

}
#endif
