/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "StdNamedObject.hpp"
#include <fstream>

namespace NMM {
  StdNamedObject::StdNamedObject(const string& name)
    : NamedObject(name.c_str()),
      __error_stream(4), __warning_stream(4), __debug_stream(4), __message_stream(4)
  {
    //Initialize the stream vector
    for (unsigned int i = 0; i< __error_stream.size(); ++i)
      __error_stream[i] = __debug_stream[i] = __warning_stream[i] = __message_stream[i] = 0;
  }

  StdNamedObject::~StdNamedObject() {
    for (unsigned int i = 0; i < __error_stream.size(); ++i) {
      delete __error_stream[i];
      delete __debug_stream[i];
      delete __warning_stream[i];
      delete __message_stream[i];
    }
  }

  void StdNamedObject::setErrorStream(const string& filename, const int& level) {
    if (__error_stream[level])
      delete __error_stream[level];

    __error_stream[level] = new fstream(filename.c_str(), ios::out);
    MessageLevel m_level = static_cast<const MessageLevel>(level);
    NamedObject::setErrorStream(__error_stream[level], m_level);
  }

  void StdNamedObject::setDebugStream(const string& filename, const int& level) {
    if (__debug_stream[level])
      delete __debug_stream[level];

    __debug_stream[level] = new fstream(filename.c_str(), ios::out);
    MessageLevel m_level = static_cast<const MessageLevel>(level);
    NamedObject::setDebugStream(__debug_stream[level], m_level);
  }

  void StdNamedObject::setWarningStream(const string& filename, const int& level) {
    if (__warning_stream[level])
      delete __warning_stream[level];

    __warning_stream[level] = new fstream(filename.c_str(), ios::out);
    MessageLevel m_level = static_cast<const MessageLevel>(level);
    NamedObject::setWarningStream(__warning_stream[level], m_level);
  }

  void StdNamedObject::setMessageStream(const string& filename, const int& level) {
    if (__message_stream[level])
      delete __message_stream[level];

    __message_stream[level] = new fstream(filename.c_str(), ios::out);
    MessageLevel m_level = static_cast<const MessageLevel>(level);
    NamedObject::setMessageStream(__message_stream[level], m_level);
  }

}

