/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_CONNECT_HPP_
#define _NMM_CONNECT_HPP_

#include "nmm/interfaces/base/INode.hpp"

namespace NMM {

  /********************** INode connect *********************/
  void connect(INode* n1, const string& out_jack,
	       INode* n2, const string& in_jack,
	       const Format* format, Format* connection_format)
    throw(const BindingException&);

  void connect(INode* n1, const string& out_jack, 
	       INode* n2, const string& in_jack, 
	       const Format* format)
    throw(const BindingException&);

  void connect(INode* n1,  
	       INode* n2,  
	       const Format* format)
    throw(const BindingException&);

  void connect(INode* n1, const string& out_jack, 
	       INode* n2,  
	       const Format* format)
    throw(const BindingException&);

  void connect(INode* n1,  
	       INode* n2, const string& in_jack, 
	       const Format* format)
    throw(const BindingException&);

  void connect(INode* n1, const string& out_jack, 
	       INode* n2, const string& in_jack)
    throw(const BindingException&);

  void connect(INode* n1,  
	       INode* n2)
    throw(const BindingException&);

  void connect(INode* n1, const string& out_jack, 
	       INode* n2)
    throw(const BindingException&);

  void connect(INode* n1,  
	       INode* n2, const string& in_jack)
    throw(const BindingException&);

  /********************** INode_var connect *********************/
  void connect(INode_var& n1, const string& out_jack,
	       INode_var& n2, const string& in_jack,
	       const Format* format, Format* conection_format)
    throw(const BindingException&);

  void connect(INode_var& n1, const string& out_jack, 
	       INode_var& n2, const string& in_jack, 
	       const Format* format)
    throw(const BindingException&);

  void connect(INode_var& n1,  
	       INode_var& n2,  
	       const Format* format)
    throw(const BindingException&);

  void connect(INode_var& n1, const string& out_jack, 
	       INode_var& n2,  
	       const Format* format)
    throw(const BindingException&);

  void connect(INode_var& n1,  
	       INode_var& n2, const string& in_jack, 
	       const Format* format)
    throw(const BindingException&);

  void connect(INode_var& n1, const string& out_jack, 
	       INode_var& n2, const string& in_jack)
    throw(const BindingException&);

  void connect(INode_var& n1,  
	       INode_var& n2)
    throw(const BindingException&);

  void connect(INode_var& n1, const string& out_jack, 
	       INode_var& n2)
    throw(const BindingException&);

  void connect(INode_var& n1,  
	       INode_var& n2, const string& in_jack)
    throw(const BindingException&);

  /********************** INode c_connect *********************/

  CommunicationChannel* c_connect(INode* n1, const string& out_jack,
				  INode* n2, const string& in_jack,
				  const Format* format, Format* conection_format)
    throw(const BindingException&);


  CommunicationChannel* c_connect(INode* n1, const string& out_jack, 
				  INode* n2, const string& in_jack, 
				  const Format* format)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode* n1,  
				  INode* n2,  
				  const Format* format)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode* n1, const string& out_jack, 
				  INode* n2,  
				  const Format* format)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode* n1,  
				  INode* n2, const string& in_jack, 
				  const Format* format)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode* n1, const string& out_jack, 
				  INode* n2, const string& in_jack)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode* n1,  
				  INode* n2)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode* n1, const string& out_jack, 
				  INode* n2)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode* n1,  
				  INode* n2, const string& in_jack)
    throw(const BindingException&);

  /********************** INode_var c_connect *********************/
  CommunicationChannel* c_connect(INode_var& n1, const string& out_jack,
				  INode_var& n2, const string& in_jack,
				  const Format* format, Format* conection_format)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode_var& n1, const string& out_jack, 
				  INode_var& n2, const string& in_jack, 
				  const Format* format)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode_var& n1,  
				  INode_var& n2,  
				  const Format* format)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode_var& n1, const string& out_jack, 
				  INode_var& n2,  
				  const Format* format)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode_var& n1,  
				  INode_var& n2, const string& in_jack, 
				  const Format* format)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode_var& n1, const string& out_jack, 
				  INode_var& n2, const string& in_jack)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode_var& n1,  
				  INode_var& n2)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode_var& n1, const string& out_jack, 
				  INode_var& n2)
    throw(const BindingException&);

  CommunicationChannel* c_connect(INode_var& n1,  
				  INode_var& n2, const string& in_jack)
    throw(const BindingException&);

  /********************** disconnect *********************/

  void quick_disconnect_output(INode_var& n, const string& out_jack = "default")
    throw(const UnbindingException&);

  void quick_disconnect_output(INode* n, const string& out_jack = "default" )
    throw(const UnbindingException&);

  void disconnect_output(INode_var& n, const string& out_jack = "default")
    throw(const UnbindingException&);

  void disconnect_output(INode* n, const string& out_jack = "default" )
    throw(const UnbindingException&);

  void disconnect_input(INode_var& n, const string& in_jack = "default")
    throw(const UnbindingException&);

  void disconnect_input(INode* n, const string& in_jack = "default" )
    throw(const UnbindingException&);

  /********************** s_connect ***********************/
  void s_connect(INode* n1, const string& out_jack, 
		 INode* n2, const string& in_jack, Format* conection_format = 0)
    throw(const BindingException&);

  CommunicationChannel* c_s_connect(INode* n1, const string& out_jack,
		 INode* n2, const string& in_jack, Format* conection_format = 0)
    throw(const BindingException&);

  /********************** unchecked_connect ***********************/

  void unchecked_connect(INode* n1, const string& out_jack, 
			 INode* n2, const string& in_jack, 
			 const Format* given_format)
    throw(const BindingException&);

  CommunicationChannel* unchecked_c_connect(INode* n1, const string& out_jack, 
					    INode* n2, const string& in_jack, 
					    const Format* given_format)
    throw(const BindingException&);
}

#endif
