/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_FPARAMETER_HPP
#define _NMM_FPARAMETER_HPP

#include "Parameter.hpp"

namespace NMM {

/**
   This Class stores FLOAT Parameter-Values.
**/

  class FParameter : public Parameter 
  {
    
  protected:
    //! The STL-List to store the FLOATs
    map<float,float> m_value_list;
    //! Private Variable for browsing the m_value_list
    mutable map<float,float>::const_iterator m_value_counter;
    //! Private Variable for browsing the pairs of m_value_list
    mutable map<float,float>::const_iterator m_pair_counter;

    //! The default value
    float m_default_value;
	
  public:
    /**
     The default Constructor.
     \param string parameter_name: the Name of the Parameter.
    **/
    FParameter(const string & parameter_name ="");
    
    /**
       Constructor with a parameter pValue that is directly added to the Value_List
       \param string parameter_name: the name of the parameter
       \param float pValue: the value of the parameter
    **/
    FParameter(const string & parameter_name, float pValue, float pWeight =  DEFAULT_WEIGHT);	
    
    /**
       The default destructor
    **/
    ~FParameter() {};
    
    virtual void addValue(void *pValue);

    virtual void addValue(void *pValue, float pWeight = DEFAULT_WEIGHT);
    
    virtual void addValue(float value, float weight = DEFAULT_WEIGHT);

    virtual void showParameter() const;
    
    virtual ostream& showParameter(ostream&) const;
    
    virtual ostream& showParameterVerbose(ostream&) const;

    virtual bool isValueIncluded(void *pValue) const;
    
    virtual bool isValueIncluded(float value) const;

    virtual void *nextValue() const;

    /**
       Returns the next pair which consists of
       two floats, the value and the weighting factor 
       for this value.
       \return FLOAT_Iterator: the next pair.
    **/
    virtual map<float,float>::const_iterator nextPair() const;

    virtual Parameter * duplicate() const;

    virtual float getWeight(void *pValue) const;

    virtual uint getNumberOfValues() const;

    virtual bool isParameterIdentical(const Parameter *reference) const;

    virtual bool isParameterMatching(const Parameter *reference) const;

    virtual Parameter * getIntersection(const Parameter *reference) const;

    virtual void * getFirstValue() const;

    virtual float getStandardizedWeight() const;

    virtual float getDefaultValue() const;

    virtual void setDefaultValue(float param);

    virtual float getLowestValue() const;

    virtual float getHighestValue() const;

    virtual const float_list getFloatValueList() const;

    virtual void fromStream(IValueStream& );
    virtual void toStream(OValueStream&) const  ;
  };
}
#endif
