/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_B_ALLOCATOR_HPP_
#define _NMM_B_ALLOCATOR_HPP_

#include "BufferManager.hpp"

namespace NMM {
  /**
   * This is a simple BufferManager implementation using STL maps.
   */
  class B_Allocator : public BufferManager , public NamedObject {

  public:
    
    /**
     * Constructor.
     * \param size the size of the buffers belonging to this buffer manager
     * \param start_num the number of pre-allocated buffers
     * \param max_num the maximum number of buffer managed by this buffer manager
     */
    B_Allocator(size_t size = 0, size_t start_num = 0, size_t max_num = 0, const char* name = "B_Allocator");

    virtual Buffer* requestBuffer (const size_t size) const;
    virtual void releaseBuffer (Buffer* buffer) const;

    /**
     * Sets the maximum number of buffers which can be allocated by this manager
     */
    Result setMaxBufferCount(const size_t max);

    /**
     * Gets the maximum number of buffers which can be allocated by this manager
     */
    const size_t getMaxBufferCount() const;

  protected:
    virtual ~B_Allocator();
    virtual void erase() const;

  private:
    typedef multimap<size_t, Buffer*> bufset_t;

    mutable bufset_t bufs;
    mutable int num_of_buffers;     /* how many buffers are given away */
    mutable bool erase_flag;       

    ThreadMutex mutex;

    size_t max_buffer_count;

  };

};

#endif //_NMM_B_ALLOCATOR_HPP_
