/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "PrivateBuffer.hpp"
#include "Buffer.hpp"
namespace NMM {

  PrivateBuffer::PrivateBuffer(Buffer *my_buffer, const char *name) : NamedObject(name) {
    this->my_buffer = my_buffer;
    busy = 0;
  }

  PrivateBuffer::~PrivateBuffer()
  {
    delete my_buffer;
  }

  Buffer* PrivateBuffer::requestBuffer (const size_t size) const {
    if (busy || (size > my_buffer->getSize())) {
      // in use or not large enough
      return 0;
    } else {
      busy = -1;
      return my_buffer;
    }
  }

  void PrivateBuffer::erase() const 
  {
    delete this;
  }

  void PrivateBuffer::releaseBuffer (Buffer*) const {
    busy = 0;
  }

  void PrivateBuffer::setBuffer(Buffer *buffer) {
    my_buffer = buffer;
    busy = 0;
  }

}
