/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Florian Winter <fw@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_LIVETHREAD_HPP
#define NMM_LIVETHREAD_HPP NMM_LIVETHREAD_HPP

#include <live/liveMedia.hh>
#include <live/GroupsockHelper.hh>
#include <live/BasicUsageEnvironment.hh>

#include "nmm/utils/thread/ThreadedObject.hpp"

namespace NMM {
  
  /** This class encapsulates the event handling thread of the LIVE library.
   */
  class LiveThread :
    public ThreadedObject 
  {
  public:
    /** This method */
    LiveThread(UsageEnvironment& env, MediaSource& source, MediaSink& sink);
    
    /** This method starts the event handling thread.
     */
    void start();
    
    /** This method stops the event handling thread.
     */
    void stop();
    
  protected:
    virtual void run();
    
  private:
    /** Stop handler for LIVE.
     * This method is passed as a handler to LIVE which is called when the thread is stopped.
     */
    static void afterPlaying(void* obj);
    
  private:
    UsageEnvironment& __env;       ///< The LIVE usage environment
    MediaSource&      __source;    ///< The LIVE media source
    MediaSink&        __sink;      ///< The LIVE media sink
    char              __live_stop; ///< Stop flag for LIVE
  };
  
} // namespace NMM

#endif // NMM_LIVETHREAD_HPP

