/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_QUALITYPARAMETER_HPP_
#define _NMM_QUALITYPARAMETER_HPP_
#include "nmm/NMMTypes.hpp"
#include <string>
#include <iosfwd>

namespace NMM {

class QualityParameter : public NMM::SerializedValue {
  
  public:
    
    enum QType {
      NIL = 1,    // default type for empty parameter 
      INT32 = 2, 
      FLOAT = 4, 
      STRING = 8
    };
    
    QualityParameter();
    QualityParameter(int);
    QualityParameter(float);
    QualityParameter(string);
    QualityParameter(const QualityParameter&);
    
    const QType getParameterType() const {
      return parameter_type.getRValue();
    }
    
    int    getIntValue();
    float  getFloatValue();
    string getStringValue();
  
    QualityParameter* copy() const { return new QualityParameter(*this); } //virtual CopyConstructor

/*    QualityParameter&    operator=(const QualityParameter& qp) {
      parameter_type = qp.parameter_type.getRValue();
      i_val = qp.i_val.getRValue();
      f_val = qp.f_val.getRValue();
      s_val = qp.s_val.getRValue();

      return *this;
    }
*/
  private:

    void registerValues();
    
    //The attributes
    NMM::TValue<int>            i_val;
    NMM::TValue<float>          f_val;
    NMM::TValue<string>         s_val;
    NMM::TValue<QType>          parameter_type;

  public:
    operator float() const {
      switch (parameter_type.getRValue()) {
        case NIL:
          return 0.0F; 
        case INT32:
          return static_cast<float>(i_val.getRValue());
        case FLOAT:
          return f_val.getRValue();
        case STRING:
          return 0.0F;
      }
      return 0.0F;
    }
    
    
  
    friend bool operator== (const QualityParameter& A, const QualityParameter& B) {
      if (A.parameter_type.getRValue() == B.parameter_type.getRValue()) {
        switch (A.parameter_type.getRValue()) {
          case NIL:
            return true;
          case INT32:
            return (A.i_val.getRValue() == B.i_val.getRValue());
          case FLOAT:
            return (A.f_val.getRValue() == B.f_val.getRValue());
          case STRING:
            return (A.s_val.getRValue() == B.s_val.getRValue());
        }
      }
      
      return false;
    }
    
    bool operator< (const QualityParameter& B);
    
    QualityParameter& operator-= (const QualityParameter& A);
    
    friend ostream& operator<< (ostream& stream, const QualityParameter& qp);
};
    
OValueStream& operator<<(NMM::OValueStream& ov, const QualityParameter::QType& t);
IValueStream& operator>>(NMM::IValueStream& iv, QualityParameter::QType& t);
 
}

#endif
