/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "ResourceContainer.hpp"
#include <sys/time.h>
#include <sstream>
namespace NMM {

ResourceContainer::ResourceContainer() {
    // get actual time on creation
    gettimeofday(&start_time, NULL);
    containers.clear();
}

Result ResourceContainer::addThread(pid_t thread_id) {
    // check if the thread exists
    stringstream name;
    name << "/proc/" << thread_id;
    if (access(name.str().c_str(), R_OK) != 0)
      return FAILURE;
    // if so add it
    CPUMonitor* cpu_monitor = new CPUMonitor(thread_id);
    used_threads.push_back(cpu_monitor);
    return SUCCESS;
}

Result ResourceContainer::removeThread(pid_t thread_id) {
    // remove the thread from the used list
    list<CPUMonitor*>::iterator ut_i = used_threads.begin();
    while( (ut_i != used_threads.end()) && ((*ut_i)->getPID() != thread_id)) {
      ut_i++;
    }
    if (ut_i == used_threads.end())
      return FAILURE;
    else {
      delete *ut_i;
      used_threads.erase(ut_i);
    }
    return SUCCESS;
}

Result ResourceContainer::addContainer(ResourceContainer *rc) {
  containers.push_back(rc);
  return SUCCESS;
}

Result ResourceContainer::removeContainer(ResourceContainer *rc) {
  containers.remove(rc);
  return SUCCESS;
}


unsigned int ResourceContainer::getCPUUsage() {
    unsigned int busy_time = 0;
    list<CPUMonitor*>::iterator ut_i;

    // read used cpu time from proc for every thread in the list
    for (ut_i = used_threads.begin(); ut_i != used_threads.end(); ut_i++) {
        busy_time += (*ut_i)->getBusyTime();
    }

    // add the cpu times of all containers added to this one
    for (list<ResourceContainer*>::iterator i = containers.begin(); i != containers.end(); ++i) {
      busy_time += (*i)->getCPUUsage();
    }

    return busy_time;
}

unsigned int ResourceContainer::getBandwidth() {
        return 0;
}

}
