/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_RESOURCE_CONTAINER_HPP_
#define _NMM_RESOURCE_CONTAINER_HPP_

#include "nmm/NMMTypes.hpp"
#include "nmm/utils/proc/CPUMonitor.hpp"
#include <sys/time.h>
#include <list>

namespace NMM {


  /**
   * \class ResourceContainer
   * \brief This class implements a container for resources which belong together
   */

  class ResourceContainer {
      list<CPUMonitor*> used_threads;
      list<ResourceContainer*> containers; // the list of managed containers

    struct timeval start_time;

//      CPUMonitor* cpu_monitor;

    public:

      ResourceContainer();
//      ~ResourceContainer();

      Result addThread(pid_t);
      Result removeThread(pid_t);

      Result addContainer(ResourceContainer*);
      Result removeContainer(ResourceContainer*);

      //! returns amount of cpu-time used (measured in jiffies (1/100ths of a sec))
      unsigned int getCPUUsage();

      //! returns used bandwidth (in bytes/sec)
      unsigned int getBandwidth();

  }; //end class
} // end namespace
#endif
