/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include <iostream>
#include "StdOValueStream.hpp"

namespace NMM {

/* StdOValueStream*/

StdOValueStream::~StdOValueStream() {
}


StdOValueStream::StdOValueStream()
  : __std_stream(&cerr), __tab_counter(0)
{

}

StdOValueStream::StdOValueStream(std::ostream& stream)
  : __std_stream(&stream), __tab_counter(0)
{

}


void StdOValueStream::beginType(const string& type, const string& ns) {
  writeBlanks();
  (*__std_stream) << type << "  "  << ns << "\n";
  ++__tab_counter;
}

void StdOValueStream::endType() {
  --__tab_counter;
}


void StdOValueStream::writeBuffer(const char* buffer, unsigned int size) {
  (*__std_stream) << "char_ptr : ";
  __std_stream->write(buffer, size);
}

void StdOValueStream::writeData()  {
  getOPartnerStream()->writeData();
}

void StdOValueStream::writeBuffer(const unsigned char* buffer, unsigned int size) {
  writeBlanks();
  (*__std_stream) << "u_char_ptr : ";
  getOPartnerStream()->writeBuffer(buffer, size);
}


StdOValueStream& StdOValueStream::operator<<(const u_int16_t &i) {
  writeBlanks();
  (*__std_stream) << "u_int16 : " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const int16_t&  i) {
  writeBlanks();
  (*__std_stream) << "int16 : " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const u_int32_t& i) {
  writeBlanks();
  (*__std_stream) << "u_int32 : " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const int32_t&  i) {
  writeBlanks();
  (*__std_stream) << "int32 : " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const u_int64_t& i) {
  writeBlanks();
  (*__std_stream) << "u_int64 : " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const int64_t& i) {
  writeBlanks();
  (*__std_stream) << "int64 : " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const unsigned long& i) {
  writeBlanks();
  (*__std_stream) << "u_long : " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const long& i) {
  writeBlanks();
  (*__std_stream) << "long : " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const double& i) {
  writeBlanks();
  (*__std_stream) << "double : " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const float& i) {
  writeBlanks();
  (*__std_stream) << "float: " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const char& i) {
  writeBlanks();
  (*__std_stream) << "char : " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const unsigned char& i) {
  writeBlanks();
  (*__std_stream) << "u_char : " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const string& i) {
  writeBlanks();
  (*__std_stream) << "string : " << i << endl;
  return *this;
}

StdOValueStream& StdOValueStream::operator<<(const bool& i) {
  writeBlanks();
  (*__std_stream) << "bool : " << i << endl;
  return *this;
}

void StdOValueStream::writeBlanks() {
  for ( unsigned int i = 0; i < __tab_counter; ++i)
    (*__std_stream) << "  ";
}



const string StdOValueStream::__o_value_stream_type = "StdOValueStream";




} //end namespace NMM                                                                        


