/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#ifndef _NMM_GENERIC_SYNC_SOURCE_NODE_HPP_
#define _NMM_GENERIC_SYNC_SOURCE_NODE_HPP_ _NMM_GENERIC_SYNC_SOURCE_NODE_HPP_

#include "nmm/base/GenericSourceNode.hpp"
#include "nmm/interfaces/base/sync/ISynchronizedSourceImpl.hpp"

namespace NMM {

  class SourceController;

  //! This class is a superclass for synchronized source nodes.
  /*!
   *  Synchronization can be switched on and off by the setSynchronized -
   *  method. By default the switch is turned off, so an application 
   *  that don't uses synchronization should not care about it.
   *  In synchronized mode, be careful that incoming buffers are 
   *  timestamped: The GenericSyncSourceNode tries to present them according
   *  to their sync_time in the timestamps. 
   */
  class GenericSyncSourceNode : public GenericSourceNode,
				public ISynchronizedSourceImpl {
  private:
    bool synchronized;           //!< Internal flag.
                                 //!< (True if the node should be synchronized.)
    bool sync_master;            //!< Internal flag.
                                 //!< (True if the node is the master for synchronization)
    bool const_rate_producer;    //!< Internale flag.
                                 //!< (True if the node produces its data at a constant rate)
    Buffer* last_buffer;          //!< A pointer to the last buffer (needed to double a buffer)

    u_int64_t valid_bytes_send;   ///< The number of bytes ever send
    int buf_count;                ///< The number of buffers produced
    Interval time_skew;           ///< The difference between the sound card's clock
                                  ///< and the system clock
    ThreadMutex reset_mutex;
  protected:

    SourceController* controller; //!< The controller belonging to this node

    virtual Message* produceBuffer(Buffer*) = 0;
    
    virtual Time getLocalTime(u_int64_t /*valid_read_bytes*/) = 0;

  public:
    //! The main constructor.
    GenericSyncSourceNode(const char* name,
			const StreamQueue::Mode queue_mode 
			 = StreamQueue::DEFAULT_MODE,
			const int size = StreamQueue::DEFAULT_SIZE);

    //! The main destructor.
    virtual ~GenericSyncSourceNode();

    Message* processBuffer(Buffer *);

    /**
     * Implementation of ISynchronizedSource interface.
     */
    Result setSynchronized(const bool&);
    bool isSynchronized();
    void setSyncMaster(const bool&);
    bool isSyncMaster();
    void setConstantRateProducer(const bool&);
    bool isConstantRateProducer();
    ISourceController* getController();

    /**
     * Implementation of ISynchronized interface.
     */
    Result syncReset();
    Result syncEnable();
    Result syncDisable();

    ////////////////////////////////////////////////////////
    //////////// Internal Methods, used to  ////////////////
    /////// communicate between Node and Controller ////////
    ////////////////////////////////////////////////////////

    void resetTime();
  };
}

#endif // _NMM_GENERIC_SYNC_SOURCE_NODE_HPP_
