/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_GRAPHURLOBJECT_HPP
#define NMM_GRAPHURLOBJECT_HPP NMM_GRAPHURLOBJECT_HPP

#include <list>
#include "GraphURL.hpp"

using namespace std;

namespace NMM {

  class GraphURLObject {
  protected:
    GraphURL* __url;
    string    __id;

    GraphURLObject(const string& id) : __url(0), __id(id) {};

  public:

    GraphURLObject() : __url(0), __id("unknown") {};

    virtual ~GraphURLObject() {};

    virtual void setGraphURL(const GraphURL* url) { __url = const_cast<GraphURL*>(url); };

    virtual const string& getID() const { return __id; };

    virtual Format* getFormatFromProtocol() const { return 0; };
    virtual Format* getFormatFromExtension() const;
    virtual Format* getFormatFromContent() const { return 0; };

    virtual string getNodeNameFromProtocol() const { return string(); };
    virtual string getNodeNameFromExtension() const { return string(); };
    virtual string getNodeNameFromContent() const { return string(); };

    virtual list<string> getInterfaces() const { list<string> l; return l; };
 
    virtual void configureConstructed(INode* ) {};
    virtual void configureInitialized(INode* ) {};
    virtual void configureOutputInitialized(INode* ) {};
  };

  class FileURLObject : public GraphURLObject {
  public:
    FileURLObject() : GraphURLObject("file") { };

    virtual ~FileURLObject() {};

    virtual Format* getFormatFromContent() const;

    virtual string getNodeNameFromExtension() const;
    virtual string getNodeNameFromContent() const;

    virtual list<string> getInterfaces() const;

    virtual void configureConstructed(INode* node);
  };

  class AudioCDURLObject : public GraphURLObject {
  public:
    AudioCDURLObject() : GraphURLObject("audiocd") { };

    virtual ~AudioCDURLObject() {};

    virtual Format* getFormatFromProtocol() const;

    virtual void configureConstructed(INode* node);
    virtual void configureOutputInitialized(INode* node);
    list<string> getInterfaces() const;
  };

  class DVDURLObject : public GraphURLObject {
  public:
    DVDURLObject() : GraphURLObject("dvd") { };

    virtual ~DVDURLObject() {};

    virtual Format* getFormatFromProtocol() const;

    virtual void configureConstructed(INode* node);
    virtual void configureOutputInitialized(INode* node);
    list<string> getInterfaces() const;
  };

  class GenericTVURLObject : public GraphURLObject {
  public:
    GenericTVURLObject(const string& id) : GraphURLObject(id) { };

    virtual ~GenericTVURLObject() {};

    virtual void configureConstructed(INode* node);
    virtual void configureInitialized(INode* node);
    list<string> getInterfaces() const;
  };

  class TVURLObject : public GenericTVURLObject {
  public:
    TVURLObject() : GenericTVURLObject("tv") {};

    virtual string getNodeNameFromProtocol() const;
  };

  class DVBTVURLObject : public GenericTVURLObject {
  public:
    DVBTVURLObject() : GenericTVURLObject("dvbtv") {};

    virtual string getNodeNameFromProtocol() const;
  };

  class IVTVURLObject : public GenericTVURLObject {
  public:
    IVTVURLObject() : GenericTVURLObject("ivtv") {};

    virtual string getNodeNameFromProtocol() const;
    list<string> getInterfaces() const;
    virtual void configureInitialized(INode* node);

  };

  class MPEGTVURLObject : public GraphURLObject {
  public:
    MPEGTVURLObject() : GraphURLObject("mpegtv") { };

    virtual ~MPEGTVURLObject() {};

    virtual string getNodeNameFromProtocol() const;

    virtual void configureConstructed(INode* node);
  };
}
#endif //NMM_GRAPHURLOBJECT_HPP
