
#include "IMMBoxState.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result IMMBoxState::initialize(const string& message /* = "" */)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > message_value(message, false);
    Event event("IMMBoxState::initialize", &message_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMMBoxState::uninitialize()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IMMBoxState::uninitialize");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMMBoxState::create(const string& message /* = "" */)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > message_value(message, false);
    Event event("IMMBoxState::create", &message_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMMBoxState::kill()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IMMBoxState::kill");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IMMBoxState::IMMBoxState_type = "IMMBoxState";
  TIFactoryObject<IMMBoxState > __IMMBoxState_prototype("IMMBoxState");


  const char* IMMBoxState::initialize_event = "IMMBoxState::initialize";
  const char* IMMBoxState::uninitialize_event = "IMMBoxState::uninitialize";
  const char* IMMBoxState::create_event = "IMMBoxState::create";
  const char* IMMBoxState::kill_event = "IMMBoxState::kill";
  
  
  INode* IDistMMBoxState::createNode(const NodeDescription& ndescr)
      throw(const Exception&)
  {
    TOutValue<INode* > return_value;
    TInValue<NodeDescription > ndescr_value(ndescr, false);
    Event event("IDistMMBoxState::createNode", &ndescr_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode* IDistMMBoxState::getAudioNode()
      throw(const Exception&)
  {
    TOutValue<INode* > return_value;
    Event event("IDistMMBoxState::getAudioNode");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode* IDistMMBoxState::getOsdNode()
      throw(const Exception&)
  {
    TOutValue<INode* > return_value;
    Event event("IDistMMBoxState::getOsdNode");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode* IDistMMBoxState::getDisplayNode()
      throw(const Exception&)
  {
    TOutValue<INode* > return_value;
    Event event("IDistMMBoxState::getDisplayNode");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IDistMMBoxState::setNeedVideo(const bool& b)
      throw(const Exception&)
  {
    TInValue<bool > b_value(b, false);
    Event event("IDistMMBoxState::setNeedVideo", &b_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IDistMMBoxState::setNeedAudio(const bool& b)
      throw(const Exception&)
  {
    TInValue<bool > b_value(b, false);
    Event event("IDistMMBoxState::setNeedAudio", &b_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IDistMMBoxState::IDistMMBoxState_type = "IDistMMBoxState";
  TIFactoryObject<IDistMMBoxState > __IDistMMBoxState_prototype("IDistMMBoxState");


  const char* IDistMMBoxState::createNode_event = "IDistMMBoxState::createNode";
  const char* IDistMMBoxState::getAudioNode_event = "IDistMMBoxState::getAudioNode";
  const char* IDistMMBoxState::getOsdNode_event = "IDistMMBoxState::getOsdNode";
  const char* IDistMMBoxState::getDisplayNode_event = "IDistMMBoxState::getDisplayNode";
  const char* IDistMMBoxState::setNeedVideo_event = "IDistMMBoxState::setNeedVideo";
  const char* IDistMMBoxState::setNeedAudio_event = "IDistMMBoxState::setNeedAudio";
  
  
  INode* IDistMP3State::getMPEGAudioDecodeNode()
      throw(const Exception&)
  {
    TOutValue<INode* > return_value;
    Event event("IDistMP3State::getMPEGAudioDecodeNode");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IInputJack* IDistMP3State::getMPEGAudioInputJack()
      throw(const Exception&)
  {
    TOutValue<IInputJack* > return_value;
    Event event("IDistMP3State::getMPEGAudioInputJack");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode* IDistMP3State::getPlaybackNode()
      throw(const Exception&)
  {
    TOutValue<INode* > return_value;
    Event event("IDistMP3State::getPlaybackNode");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IDistMP3State::addTextLine(const string& line)
      throw(const Exception&)
  {
    TInValue<string > line_value(line, false);
    Event event("IDistMP3State::addTextLine", &line_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IDistMP3State::selectTextLine(const unsigned int& num)
      throw(const Exception&)
  {
    TInValue<unsigned int > num_value(num, false);
    Event event("IDistMP3State::selectTextLine", &num_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IDistMP3State::IDistMP3State_type = "IDistMP3State";
  TIFactoryObject<IDistMP3State > __IDistMP3State_prototype("IDistMP3State");


  const char* IDistMP3State::getMPEGAudioDecodeNode_event = "IDistMP3State::getMPEGAudioDecodeNode";
  const char* IDistMP3State::getMPEGAudioInputJack_event = "IDistMP3State::getMPEGAudioInputJack";
  const char* IDistMP3State::getPlaybackNode_event = "IDistMP3State::getPlaybackNode";
  const char* IDistMP3State::addTextLine_event = "IDistMP3State::addTextLine";
  const char* IDistMP3State::selectTextLine_event = "IDistMP3State::selectTextLine";


}


