
#include "IEchoCancellation.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IEchoCancellation::setMin_xf(const float& min_xf)
      throw(const Exception&)
  {
    TInValue<float > min_xf_value(min_xf, false);
    Event event("IEchoCancellation::setMin_xf", &min_xf_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEchoCancellation::setGeigelThreshold(const float& geigelthreshold)
      throw(const Exception&)
  {
    TInValue<float > geigelthreshold_value(geigelthreshold, false);
    Event event("IEchoCancellation::setGeigelThreshold", &geigelthreshold_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEchoCancellation::setNLPAttenuation(const float& nlpattenuation)
      throw(const Exception&)
  {
    TInValue<float > nlpattenuation_value(nlpattenuation, false);
    Event event("IEchoCancellation::setNLPAttenuation", &nlpattenuation_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEchoCancellation::setBufferShifting(const int& shifting_const)
      throw(const Exception&)
  {
    TInValue<int > shifting_const_value(shifting_const, false);
    Event event("IEchoCancellation::setBufferShifting", &shifting_const_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEchoCancellation::setSampleShifting(const int& shifting_const)
      throw(const Exception&)
  {
    TInValue<int > shifting_const_value(shifting_const, false);
    Event event("IEchoCancellation::setSampleShifting", &shifting_const_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEchoCancellation::setECEnabled(const bool& ece)
      throw(const Exception&)
  {
    TInValue<bool > ece_value(ece, false);
    Event event("IEchoCancellation::setECEnabled", &ece_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEchoCancellation::setESEnabled(const bool& ese)
      throw(const Exception&)
  {
    TInValue<bool > ese_value(ese, false);
    Event event("IEchoCancellation::setESEnabled", &ese_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEchoCancellation::setForwardEnabled(const bool& fe)
      throw(const Exception&)
  {
    TInValue<bool > fe_value(fe, false);
    Event event("IEchoCancellation::setForwardEnabled", &fe_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEchoCancellation::setDoBeep(const bool& dB)
      throw(const Exception&)
  {
    TInValue<bool > dB_value(dB, false);
    Event event("IEchoCancellation::setDoBeep", &dB_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEchoCancellation::increaseBufferShifting()
      throw(const Exception&)
  {
    Event event("IEchoCancellation::increaseBufferShifting");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEchoCancellation::decreaseBufferShifting()
      throw(const Exception&)
  {
    Event event("IEchoCancellation::decreaseBufferShifting");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int IEchoCancellation::calculateBufferDelay(const int& backwards, const int& forwards)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInValue<int > backwards_value(backwards, false);
    TInValue<int > forwards_value(forwards, false);
    Event event("IEchoCancellation::calculateBufferDelay", &backwards_value, &forwards_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IEchoCancellation::setDumpFile(const string& filename)
      throw(const Exception&)
  {
    TInValue<string > filename_value(filename, false);
    Event event("IEchoCancellation::setDumpFile", &filename_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEchoCancellation::setCalcDelayMode(const bool& enable)
      throw(const Exception&)
  {
    TInValue<bool > enable_value(enable, false);
    Event event("IEchoCancellation::setCalcDelayMode", &enable_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IEchoCancellation::calcDelayFinished(const int& samples_delay)
      throw(const Exception&)
  {
    TInValue<int > samples_delay_value(samples_delay, false);
    Event event("IEchoCancellation::calcDelayFinished", &samples_delay_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IEchoCancellation::IEchoCancellation_type = "IEchoCancellation";
  TIFactoryObject<IEchoCancellation > __IEchoCancellation_prototype("IEchoCancellation");


  const char* IEchoCancellation::setMin_xf_event = "IEchoCancellation::setMin_xf";
  const char* IEchoCancellation::setGeigelThreshold_event = "IEchoCancellation::setGeigelThreshold";
  const char* IEchoCancellation::setNLPAttenuation_event = "IEchoCancellation::setNLPAttenuation";
  const char* IEchoCancellation::setBufferShifting_event = "IEchoCancellation::setBufferShifting";
  const char* IEchoCancellation::setSampleShifting_event = "IEchoCancellation::setSampleShifting";
  const char* IEchoCancellation::setECEnabled_event = "IEchoCancellation::setECEnabled";
  const char* IEchoCancellation::setESEnabled_event = "IEchoCancellation::setESEnabled";
  const char* IEchoCancellation::setForwardEnabled_event = "IEchoCancellation::setForwardEnabled";
  const char* IEchoCancellation::setDoBeep_event = "IEchoCancellation::setDoBeep";
  const char* IEchoCancellation::increaseBufferShifting_event = "IEchoCancellation::increaseBufferShifting";
  const char* IEchoCancellation::decreaseBufferShifting_event = "IEchoCancellation::decreaseBufferShifting";
  const char* IEchoCancellation::calculateBufferDelay_event = "IEchoCancellation::calculateBufferDelay";
  const char* IEchoCancellation::setDumpFile_event = "IEchoCancellation::setDumpFile";
  const char* IEchoCancellation::setCalcDelayMode_event = "IEchoCancellation::setCalcDelayMode";
  const char* IEchoCancellation::calcDelayFinished_event = "IEchoCancellation::calcDelayFinished";


}


