
#ifndef NMM_IEchoCancellationImpl_HPP
#define NMM_IEchoCancellationImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IEchoCancellation.hpp"


namespace NMM
{

  class IEchoCancellationImpl  : public virtual NMMObject { 
  public:
    IEchoCancellationImpl() { addInterface(new IEchoCancellation(*this)); registerEvents(); };

    virtual ~IEchoCancellationImpl() {};

  protected:
    /**
     * minimum energy in xf. Range: M70dB_PCM to M50dB_PCM. 
     * Should be equal to microphone ambient Noise level 
     */

    virtual void setMin_xf(const float& min_xf) = 0;
    /**
     * Double Talk Detector Speaker/Microphone Threshold. Range <=1
     * Large value (M0dB) is good for Single-Talk Echo cancellation, 
     * small value (M12dB) is good for Doulbe-Talk AEC
     */

    virtual void setGeigelThreshold(const float& geigelthreshold) = 0;
    /**
     * for Non Linear Processor. Range >0 to 1. Large value (M0dB) is good
     * for Double-Talk, small value (M12dB) is good for Single-Talk
     */

    virtual void setNLPAttenuation(const float& nlpattenuation) = 0;
    /**
     * Sets the constant value of buffers by which the audio streams are shifted against each other. 
     */

    virtual void setBufferShifting(const int& shifting_const) = 0;
    /**
     * Sets the constant value of samples by which the audio streams are shifted against each other. 
     */

    virtual void setSampleShifting(const int& shifting_const) = 0;
    /**
     * Enables/Disables Echo Cancellation 
     */

    virtual void setECEnabled(const bool& ece) = 0;
    /**
     * Enables/Disables Echo Surpression
     */

    virtual void setESEnabled(const bool& ese) = 0;
    /**
     * Node just forwards incoming buffer without changing them
     */

    virtual void setForwardEnabled(const bool& fe) = 0;
    /**
     * Lets Node send a beep, which can be used do estimate delay.
     */

    virtual void setDoBeep(const bool& dB) = 0;
    /**
     * Increases constant for buffer shifting in EchoCancellationNode
     */

    virtual void increaseBufferShifting() = 0;
    /**
     * Decrease constant for buffer shifting in EchoCancellationNode
     */

    virtual void decreaseBufferShifting() = 0;
    /**
     * Recalculate Buffer offset between speaker and mic stream in EchoCancellationNode.
     * Test offset for values between start and start plus buffer_range.
     */

    virtual int calculateBufferDelay(const int& backwards, const int& forwards) = 0;
    /**
     * Sets filename to dump audiostreams to.
     */

    virtual void setDumpFile(const string& filename) = 0;
    /**
     * Set EchoCancellationNode to this special mode to calculate echo delay.
     */

    virtual void setCalcDelayMode(const bool& enable) = 0;
    /**
     * Used to notify application of finished delay calculation.
     * Argument is equal to calculated delay.
     */

    virtual void calcDelayFinished(const int& samples_delay) = 0;


  private:
    void registerEvents();
  };

}

#endif
