
#ifndef NMM_IIcecast_HPP
#define NMM_IIcecast_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/interfaces/base/INode.hpp"

namespace NMM
{
  /** Interface for IceCast sink node.
   */

  class IIcecast : virtual public Interface { 
  public:
    IIcecast() {  setTypeInfo(TypeInfo(IIcecast::IIcecast_type, "NMM")); };

    IIcecast(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IIcecast::IIcecast_type, "NMM")); setEventReceiver(parent); };

    virtual ~IIcecast() {};

    static const char* IIcecast_type;

    virtual  string getInterfaceType() const { return IIcecast_type; }

    Interface* copy() const { return new IIcecast(*this); }
    static const char* setServerIP_event;
    static const char* setServerPort_event;
    static const char* setServerPassword_event;
    static const char* setUser_event;
    static const char* setMountpoint_event;
    static const char* setTitle_event;
    static const char* setStreamName_event;
    static const char* setGenre_event;
    static const char* setDescription_event;
    static const char* setURL_event;
    static const char* setPublic_event;
    static const char* setSendTitle_event;
    /** Set the IP address of the IceCast streaming server.
     */

    virtual Result setServerIP(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const Exception&, const IllegalStateException&);
    /** Set the port of the IceCast streaming server.
     */

    virtual Result setServerPort(const int& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const Exception&, const IllegalStateException&);
    /** Set the source password for the IceCast streaming server.
     */

    virtual Result setServerPassword(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const Exception&, const IllegalStateException&);
    /** Set the user name for the IceCast streaming server.
     */

    virtual Result setUser(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const Exception&, const IllegalStateException&);
    /** Set the mount point.
     */

    virtual Result setMountpoint(const string& value) /* limited(INode::INITIALIZED) */ /* required(INode::INITIALIZED, INode::ACTIVATED) */
      throw(const Exception&, const IllegalStateException&);
    /** Set the source title.
     */

    virtual Result setTitle(const string& value) /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** Set the source name.
     */

    virtual Result setStreamName(const string& value) /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** Set the source genre.
     */

    virtual Result setGenre(const string& value) /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** Set the source description.
     */

    virtual Result setDescription(const string& value) /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** Set the source URL.
     */

    virtual Result setURL(const string& value) /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** Set public mode.
     */

    virtual Result setPublic(const int& value) /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** Set whether to send title.
     */

    virtual Result setSendTitle(const bool& value) /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);


  };

  typedef auto_ptr<IIcecast> IIcecast_var;




}

#endif
