
#ifndef NMM_ICompositeStrategyImpl_HPP
#define NMM_ICompositeStrategyImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ICompositeStrategy.hpp"

#include "ITransportStrategyImpl.hpp"

namespace NMM
{
  /**
   *  @brief Interface of CompositeStrategy
   *
   *  This is the interface of the CompositeStrategy class.
   */

  class ICompositeStrategyImpl  : public virtual NMMObject { 
  public:
    ICompositeStrategyImpl() { addInterface(new ICompositeStrategy(*this)); registerEvents(); };

    virtual ~ICompositeStrategyImpl() {};

  protected:
    /**
     *  @name Child strategy access
     */
//@{
    /**
     *  @brief Get buffer child strategy
     */

    virtual ITransportStrategy* getBufferChildStrategy() const = 0;
    /**
     *  @brief Get event child strategy
     */

    virtual ITransportStrategy* getEventChildStrategy() const = 0;
//@}
    /**
     *  @name Internal configuration events
     */
//@{
    /**
     *  @brief Set synchronization mode
     *
     *  Internal method for setting the synchronization mode. TEMPORARY!
     */

    virtual void setMode(const ICompositeStrategy::Mode& mode) = 0;
    /**
     *  @brief Set data loss interval
     *
     *  This method sets the interval for simulating data loss in unreliable mode.
     *  It is only meant for benchmarks and should not be used. An interval of 0 disables data loss simulation.
     */

    virtual void setLossInterval(const unsigned int& loss_interval) = 0;
    /**
     *  @brief Connect buffer sub strategy
     *  @exception BindingException if this event was not sent through a TransportStrategy
     *
     *  This event is sent by the CompositeStrategy that initiates a connection to connect
     *  the partner strategy of its buffer child strategy to its partner strategy.
     *  The receiver assumes that the sender of the event is the partner strategy of the sub strategy
     *  and sets its sub strategy accordingly.
     */

    virtual void connectToCompositeBuffer() 
      throw(const BindingException&) = 0;
    /**
     *  @brief Connect event sub strategy
     *  @exception BindingException if this event was not sent through a TransportStrategy
     *
     *  This event is sent by the CompositeStrategy that initiates a connection to connect
     *  the partner strategy of its buffer child strategy to its partner strategy.
     *  The receiver assumes that the sender of the event is the partner strategy of the sub strategy
     *  and sets its sub strategy accordingly.
     */

    virtual void connectToCompositeEvent() 
      throw(const BindingException&) = 0;
    /**
     *  @brief Event acknowledgement
     *  @param stream_counter Sequence number of the event
     *
     *  This event is sent from the receiver to the sender when a reliable event has been received.
     */

    virtual void eventReceived(const int& stream_counter) /* in-stream */  = 0;

//@}

  private:
    void registerEvents();
  };

}

#endif
