
#ifndef NMM_IOutputJack_HPP
#define NMM_IOutputJack_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "IJack.hpp"
#include "ITransportStrategy.hpp"

namespace NMM
{

  class CommunicationChannel;

  class IInputJack;

  class IOutputJack : virtual public IJack { 
  public:
    IOutputJack() {  setTypeInfo(TypeInfo(IOutputJack::IOutputJack_type, "NMM")); };

    IOutputJack(NMMObject& parent) : IJack(parent) {  setTypeInfo(TypeInfo(IOutputJack::IOutputJack_type, "NMM")); setEventReceiver(parent); };

    virtual ~IOutputJack() {};

    static const char* IOutputJack_type;

    virtual  string getInterfaceType() const { return IOutputJack_type; }

    Interface* copy() const { return new IOutputJack(*this); }
    static const char* connectTo_event;
    static const char* getDataPartner_event;
    static const char* getTransportStrategy_event;
    /**
     * Connect to a partner jack.
     * \param jack partner jack.
     */

    virtual CommunicationChannel* connectTo(const IInputJack& jack)
      throw(const Exception&, const BindingException&);
    /**
     * A method to obtain a pointer to the data partner of this jack.
     * \returns a copy to the data partner jack of this jack, or NULL
     * if this jack has no data partner.
     */

    virtual IInputJack* getDataPartner()
      throw(const Exception&);

    virtual ITransportStrategy* getTransportStrategy()
      throw(const Exception&);


  };

  typedef auto_ptr<IOutputJack> IOutputJack_var;




}

#endif
