
#include "IReferenceCount.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IReferenceCount::increaseCount(const unsigned int& i /* = 1 */)
      throw(const Exception&)
  {
    TInValue<unsigned int > i_value(i, false);
    Event event("IReferenceCount::increaseCount", &i_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IReferenceCount::decreaseCount(const unsigned int& i /* = 1 */)
      throw(const Exception&)
  {
    TInValue<unsigned int > i_value(i, false);
    Event event("IReferenceCount::decreaseCount", &i_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  unsigned int IReferenceCount::getCount()
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IReferenceCount::getCount");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IReferenceCount::IReferenceCount_type = "IReferenceCount";
  TIFactoryObject<IReferenceCount > __IReferenceCount_prototype("IReferenceCount");


  const char* IReferenceCount::increaseCount_event = "IReferenceCount::increaseCount";
  const char* IReferenceCount::decreaseCount_event = "IReferenceCount::decreaseCount";
  const char* IReferenceCount::getCount_event = "IReferenceCount::getCount";


}


