
#include "IUsage.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IUsage::setUsage(const IUsage::usage_t& usage, const IUsageNotify* user)
      throw(const Exception&)
  {
    TInValue<IUsage::usage_t > usage_value(usage, false);
    TInValue<IUsageNotify* > user_value (const_cast<IUsageNotify* >(user), false);
    Event event("IUsage::setUsage", &usage_value, &user_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IUsage::unsetUsage(const IUsage::usage_t& usage, const IUsageNotify* user)
      throw(const Exception&)
  {
    TInValue<IUsage::usage_t > usage_value(usage, false);
    TInValue<IUsageNotify* > user_value (const_cast<IUsageNotify* >(user), false);
    Event event("IUsage::unsetUsage", &usage_value, &user_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  IUsage::usage_t IUsage::getUsage()
      throw(const Exception&)
  {
    TOutValue<IUsage::usage_t > return_value;
    Event event("IUsage::getUsage");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int IUsage::getRank(const IUsage::usage_t& usage, const IUsageNotify* user)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInValue<IUsage::usage_t > usage_value(usage, false);
    TInValue<IUsageNotify* > user_value (const_cast<IUsageNotify* >(user), false);
    Event event("IUsage::getRank", &usage_value, &user_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IUsage::isMaster(const IUsage::usage_t& mode, const IUsageNotify* user)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<IUsage::usage_t > mode_value(mode, false);
    TInValue<IUsageNotify* > user_value (const_cast<IUsageNotify* >(user), false);
    Event event("IUsage::isMaster", &mode_value, &user_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IUsage::IUsage_type = "IUsage";
  TIFactoryObject<IUsage > __IUsage_prototype("IUsage");
  TValueConstructor< IUsage::usage_t > __IUsage_usage_t_prototype(TypeInfo("IUsage::usage_t", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const IUsage::usage_t& t)
  {
    ov.beginType(TypeInfo("IUsage::usage_t", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, IUsage::usage_t& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IUsage::usage_t", "NMM")) {
      throw InvalidTypeException(TypeInfo("IUsage::usage_t", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }



  const char* IUsage::setUsage_event = "IUsage::setUsage";
  const char* IUsage::unsetUsage_event = "IUsage::unsetUsage";
  const char* IUsage::getUsage_event = "IUsage::getUsage";
  const char* IUsage::getRank_event = "IUsage::getRank";
  const char* IUsage::isMaster_event = "IUsage::isMaster";


}


