
#include "IServerRegistry.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  unsigned int IServerRegistry::getTransactionTimeout()
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IServerRegistry::getTransactionTimeout");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IServerRegistry::setTransactionTimeout(const unsigned int& time_out)
      throw(const Exception&)
  {
    TInValue<unsigned int > time_out_value(time_out, false);
    Event event("IServerRegistry::setTransactionTimeout", &time_out_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  unsigned int IServerRegistry::getNodeLeasingTime()
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IServerRegistry::getNodeLeasingTime");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IServerRegistry::setNodeLeasingTime(const unsigned int& time_out)
      throw(const Exception&)
  {
    TInValue<unsigned int > time_out_value(time_out, false);
    Event event("IServerRegistry::setNodeLeasingTime", &time_out_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  unsigned int IServerRegistry::refreshNodeLeasing(const RegistryID& reg_id, const ObjectID& node_id, const UserInfo& user_info)
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    TInValue<ObjectID > node_id_value(node_id, false);
    TInValue<UserInfo > user_info_value(user_info, false);
    Event event("IServerRegistry::refreshNodeLeasing", &reg_id_value, &node_id_value, &user_info_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IServerRegistry::refreshTransaction(const UserInfo& user_info)
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<UserInfo > user_info_value(user_info, false);
    Event event("IServerRegistry::refreshTransaction", &user_info_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IServerRegistry::startResourceManager()
      throw(const Exception&)
  {
    Event event("IServerRegistry::startResourceManager");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  IResourceManager* IServerRegistry::getResourceManager()
      throw(const Exception&)
  {
    TOutValue<IResourceManager* > return_value;
    Event event("IServerRegistry::getResourceManager");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IServerRegistry::releaseResourceManager()
      throw(const Exception&)
  {
    Event event("IServerRegistry::releaseResourceManager");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IServerRegistry::startIPerfServer()
      throw(const Exception&)
  {
    Event event("IServerRegistry::startIPerfServer");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IServerRegistry::stopIPerfServer()
      throw(const Exception&)
  {
    Event event("IServerRegistry::stopIPerfServer");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  Interface* IServerRegistry::requestNMMObject(const ObjectDescription& od)
      throw(const Exception&)
  {
    TOutValue<Interface* > return_value;
    TInValue<ObjectDescription > od_value(od, false);
    Event event("IServerRegistry::requestNMMObject", &od_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IServerRegistry::releaseNMMObject(const Interface& i_face)
      throw(const Exception&)
  {
    TInValue<Interface > i_face_value(i_face, false);
    Event event("IServerRegistry::releaseNMMObject", &i_face_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  ISession* IServerRegistry::requestSharedGraph(GraphDescription& request, const UserInfo& u_info) 
      throw(const Exception&, const RegistryException&)
  {
    TOutValue<ISession* > return_value;
    TInOutValue<GraphDescription > request_value;
    request_value.setValue(request);
    TInValue<UserInfo > u_info_value(u_info, false);
    Event event("IServerRegistry::requestSharedGraph", &request_value, &u_info_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(RegistryException* ex = dynamic_cast<RegistryException*>(event.getException())) {
          throw RegistryException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    request = request_value.getRValue();
    return return_value.getRValue();
  }
  
  void IServerRegistry::releaseSession(const ISession& session, const UserInfo& u_info)
      throw(const Exception&)
  {
    TInValue<ISession > session_value(session, false);
    TInValue<UserInfo > u_info_value(u_info, false);
    Event event("IServerRegistry::releaseSession", &session_value, &u_info_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IServerRegistry::releaseSharedNode(const INode& node)
      throw(const Exception&)
  {
    TInValue<INode > node_value(node, false);
    Event event("IServerRegistry::releaseSharedNode", &node_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  ISession* IServerRegistry::initSession(GraphDescription& gd, const UserInfo& u_info, const ISession* given_session)
      throw(const Exception&)
  {
    TOutValue<ISession* > return_value;
    TInOutValue<GraphDescription > gd_value;
    gd_value.setValue(gd);
    TInValue<UserInfo > u_info_value(u_info, false);
    TInValue<ISession* > given_session_value (const_cast<ISession* >(given_session), false);
    Event event("IServerRegistry::initSession", &gd_value, &u_info_value, &given_session_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    gd = gd_value.getRValue();
    return return_value.getRValue();
  }
  
  void IServerRegistry::requestGraph(GraphDescription& gd, const UserInfo& user_info)
      throw(const Exception&)
  {
    TInOutValue<GraphDescription > gd_value;
    gd_value.setValue(gd);
    TInValue<UserInfo > user_info_value(user_info, false);
    Event event("IServerRegistry::requestGraph", &gd_value, &user_info_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    gd = gd_value.getRValue();
  }

  const char* IServerRegistry::IServerRegistry_type = "IServerRegistry";
  TIFactoryObject<IServerRegistry > __IServerRegistry_prototype("IServerRegistry");


  const char* IServerRegistry::getTransactionTimeout_event = "IServerRegistry::getTransactionTimeout";
  const char* IServerRegistry::setTransactionTimeout_event = "IServerRegistry::setTransactionTimeout";
  const char* IServerRegistry::getNodeLeasingTime_event = "IServerRegistry::getNodeLeasingTime";
  const char* IServerRegistry::setNodeLeasingTime_event = "IServerRegistry::setNodeLeasingTime";
  const char* IServerRegistry::refreshNodeLeasing_event = "IServerRegistry::refreshNodeLeasing";
  const char* IServerRegistry::refreshTransaction_event = "IServerRegistry::refreshTransaction";
  const char* IServerRegistry::startResourceManager_event = "IServerRegistry::startResourceManager";
  const char* IServerRegistry::getResourceManager_event = "IServerRegistry::getResourceManager";
  const char* IServerRegistry::releaseResourceManager_event = "IServerRegistry::releaseResourceManager";
  const char* IServerRegistry::startIPerfServer_event = "IServerRegistry::startIPerfServer";
  const char* IServerRegistry::stopIPerfServer_event = "IServerRegistry::stopIPerfServer";
  const char* IServerRegistry::requestNMMObject_event = "IServerRegistry::requestNMMObject";
  const char* IServerRegistry::releaseNMMObject_event = "IServerRegistry::releaseNMMObject";
  const char* IServerRegistry::requestSharedGraph_event = "IServerRegistry::requestSharedGraph";
  const char* IServerRegistry::releaseSession_event = "IServerRegistry::releaseSession";
  const char* IServerRegistry::releaseSharedNode_event = "IServerRegistry::releaseSharedNode";
  const char* IServerRegistry::initSession_event = "IServerRegistry::initSession";
  const char* IServerRegistry::requestGraph_event = "IServerRegistry::requestGraph";


}


