
#ifndef NMM_IResourceManagerImpl_HPP
#define NMM_IResourceManagerImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IResourceManager.hpp"


namespace NMM
{

  class IResourceManagerImpl  : public virtual NMMObject { 
  public:
    IResourceManagerImpl() { addInterface(new IResourceManager(*this)); registerEvents(); };

    virtual ~IResourceManagerImpl() {};

  protected:
    /**
     * set the number of usable network interfaces
     * this clears all previous reservations
     */

    virtual void setNetworkInterfaces(const unsigned int& num) = 0;
    /**
     * add a connected Node to reserve the needed cpu resources
     */

    virtual Result addNode(const INode* node, unsigned int& id, const unsigned int& interf = 0) = 0;
    /**
     * release a previously added node (and free its reserved resources)
     */

    virtual Result releaseNode(const INode* node, const unsigned int& id) = 0;
    /** This method returns the name, number, speed (MHz and BogoMIPS) and % of free computation
      * time of the cpu on the machine the registry runs on
      */

    virtual void getCPUInfo(string& name, uint& bogo_mips, uint& num, float& free) = 0;
    /** This method returns the speed (BogoMIPS) and % of free computation time of the cpu on the machine the registry runs on
      */

    virtual void getCPUSpeed(uint& bogo_mips, float& free) = 0;
   /**
    * \returns the amount of free cpu-time within one second, accumulated on all available cpus
    */

    virtual void getCPUFree(float& free) = 0;
   /**
    * \set the update intervall of cpuInfo-polling
    */

    virtual void setCPUUpdateIntervall(const unsigned int& seconds) = 0;
   /**
    * \returns the amount of free cpu-time within one second, accumulated on all available cpus
    */

    virtual void getCPULeft(float& left) = 0;
   /**
    *
    */

    virtual void setReservationPending() = 0;
    /**
     *
     */

    virtual void commitReservation() = 0;
   /**
     * start an iperf-server
     */

    virtual void startServer() = 0;
    /**
     * stop the iperf-server
     */

    virtual void stopServer() = 0;
    /**
     * test the speed of a connection to hostname
     */

    virtual float testConnection(const string& hostname) = 0;
    /**
     * test the speeds of the simultaneous connections to all specified hosts
     * \param hosts a list of hosts to test
     * \param speeds an empty list that will be filled with the measured speeds
     */

    virtual void testConnections(const list<string > & hosts, list<float > & speeds) = 0;

    virtual Result setHosts(const list<string > & hosts) = 0;

    virtual void getCPUDescription(string& description) = 0;

    virtual Result updateNodeInfo(const INode* node) = 0;


  private:
    void registerEvents();
  };

}

#endif
