
#include "IAudioHandoverSynchronizer.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result IAudioHandoverSynchronizer::setFirstSinkController(const ISinkController* controller)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<ISinkController* > controller_value (const_cast<ISinkController* >(controller), false);
    Event event("IAudioHandoverSynchronizer::setFirstSinkController", &controller_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IAudioHandoverSynchronizer::setSecondSinkController(const ISinkController* controller)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<ISinkController* > controller_value (const_cast<ISinkController* >(controller), false);
    Event event("IAudioHandoverSynchronizer::setSecondSinkController", &controller_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IAudioHandoverSynchronizer::switchControllers()
      throw(const Exception&)
  {
    Event event("IAudioHandoverSynchronizer::switchControllers");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  Result IAudioHandoverSynchronizer::setSkew(const Interval& skew)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Interval > skew_value(skew, false);
    Event event("IAudioHandoverSynchronizer::setSkew", &skew_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IAudioHandoverSynchronizer::setSwitch(const ISwitch* s)
      throw(const Exception&)
  {
    TInValue<ISwitch* > s_value (const_cast<ISwitch* >(s), false);
    Event event("IAudioHandoverSynchronizer::setSwitch", &s_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IAudioHandoverSynchronizer::setMode(const IAudioHandoverSynchronizer::Mode& mode)
      throw(const Exception&)
  {
    TInValue<IAudioHandoverSynchronizer::Mode > mode_value(mode, false);
    Event event("IAudioHandoverSynchronizer::setMode", &mode_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IAudioHandoverSynchronizer::setOverlap(const Interval& overlap)
      throw(const Exception&)
  {
    TInValue<Interval > overlap_value(overlap, false);
    Event event("IAudioHandoverSynchronizer::setOverlap", &overlap_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IAudioHandoverSynchronizer::waitForSwitchControllers()
      throw(const Exception&)
  {
    Event event("IAudioHandoverSynchronizer::waitForSwitchControllers");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IAudioHandoverSynchronizer::IAudioHandoverSynchronizer_type = "IAudioHandoverSynchronizer";
  TIFactoryObject<IAudioHandoverSynchronizer > __IAudioHandoverSynchronizer_prototype("IAudioHandoverSynchronizer");
  TValueConstructor< IAudioHandoverSynchronizer::Mode > __IAudioHandoverSynchronizer_Mode_prototype(TypeInfo("IAudioHandoverSynchronizer::Mode", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const IAudioHandoverSynchronizer::Mode& t)
  {
    ov.beginType(TypeInfo("IAudioHandoverSynchronizer::Mode", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, IAudioHandoverSynchronizer::Mode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IAudioHandoverSynchronizer::Mode", "NMM")) {
      throw InvalidTypeException(TypeInfo("IAudioHandoverSynchronizer::Mode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }

  TValueConstructor< IAudioHandoverSynchronizer::SinkId > __IAudioHandoverSynchronizer_SinkId_prototype(TypeInfo("IAudioHandoverSynchronizer::SinkId", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const IAudioHandoverSynchronizer::SinkId& t)
  {
    ov.beginType(TypeInfo("IAudioHandoverSynchronizer::SinkId", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, IAudioHandoverSynchronizer::SinkId& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IAudioHandoverSynchronizer::SinkId", "NMM")) {
      throw InvalidTypeException(TypeInfo("IAudioHandoverSynchronizer::SinkId", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }



  const char* IAudioHandoverSynchronizer::setFirstSinkController_event = "IAudioHandoverSynchronizer::setFirstSinkController";
  const char* IAudioHandoverSynchronizer::setSecondSinkController_event = "IAudioHandoverSynchronizer::setSecondSinkController";
  const char* IAudioHandoverSynchronizer::switchControllers_event = "IAudioHandoverSynchronizer::switchControllers";
  const char* IAudioHandoverSynchronizer::setSkew_event = "IAudioHandoverSynchronizer::setSkew";
  const char* IAudioHandoverSynchronizer::setSwitch_event = "IAudioHandoverSynchronizer::setSwitch";
  const char* IAudioHandoverSynchronizer::setMode_event = "IAudioHandoverSynchronizer::setMode";
  const char* IAudioHandoverSynchronizer::setOverlap_event = "IAudioHandoverSynchronizer::setOverlap";
  const char* IAudioHandoverSynchronizer::waitForSwitchControllers_event = "IAudioHandoverSynchronizer::waitForSwitchControllers";


}


