
#ifndef NMM_IMultiAudioVideoSynchronizer_HPP
#define NMM_IMultiAudioVideoSynchronizer_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "IMultiAudioSinkSynchronizer.hpp"
#include "IMultiVideoSinkSynchronizer.hpp"

namespace NMM
{
  /**
   * Audio-video sink synchronizer.
   * Interface for a synchronizer which performs inter-stream synchronization between
   * multiple audio and video sinks relative to a master audio sink.
   */

  class IMultiAudioVideoSynchronizer : virtual public ISinkSynchronizer, virtual public IMultiAudioSinkSynchronizer, virtual public IMultiVideoSinkSynchronizer { 
  public:
    IMultiAudioVideoSynchronizer() {  setTypeInfo(TypeInfo(IMultiAudioVideoSynchronizer::IMultiAudioVideoSynchronizer_type, "NMM")); };

    IMultiAudioVideoSynchronizer(NMMObject& parent) : ISinkSynchronizer(parent), IMultiAudioSinkSynchronizer(parent), IMultiVideoSinkSynchronizer(parent) {  setTypeInfo(TypeInfo(IMultiAudioVideoSynchronizer::IMultiAudioVideoSynchronizer_type, "NMM")); setEventReceiver(parent); };

    virtual ~IMultiAudioVideoSynchronizer() {};

    static const char* IMultiAudioVideoSynchronizer_type;

    virtual  string getInterfaceType() const { return IMultiAudioVideoSynchronizer_type; }

    Interface* copy() const { return new IMultiAudioVideoSynchronizer(*this); }
    static const char* setMasterAudioSink_event;
    static const char* setAudioSkew_event;
    static const char* setVideoSkew_event;
    /**
     * Set master audio sink.
     * Selects an audio sink as master audio sink. If this method is not called, the first
     * added audio sink becomes the master.
     * @param id audio sink id
     */

    virtual Result setMasterAudioSink(const int& id)
      throw(const Exception&);
    /**
     * Set skew of an audio sink.
     * Sets the skew of an audio sink relative to the master audio sink.
     * @param id sink id
     * @param skew skew of this sink relative to master
     */

    virtual Result setAudioSkew(const int& id, const Interval& skew)
      throw(const Exception&);
    /**
     * Set skew of a video sink.
     * Sets the skew of a video sink relative to the master audio sink.
     * @param id sink id
     * @param skew skew of this sink relative to master
     */

    virtual Result setVideoSkew(const int& id, const Interval& skew)
      throw(const Exception&);


  };

  typedef auto_ptr<IMultiAudioVideoSynchronizer> IMultiAudioVideoSynchronizer_var;




}

#endif
