
#include "ISinkController.hpp"
#include "nmm/base/InterfaceFactory.hpp"

#include "ISinkSynchronizer.hpp"

namespace NMM
{
  
  
  Result ISinkController::setMaxDelay(const Interval& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Interval > value_value(value, false);
    Event event("ISinkController::setMaxDelay", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setMaxAverageDelay(const Interval& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Interval > value_value(value, false);
    Event event("ISinkController::setMaxAverageDelay", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setTheoLatency(const Interval& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Interval > value_value(value, false);
    Event event("ISinkController::setTheoLatency", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setDefaultTheoLatency(const Interval& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Interval > value_value(value, false);
    Event event("ISinkController::setDefaultTheoLatency", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setSinkSynchronizer(const ISinkSynchronizer* value, const int& id)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<ISinkSynchronizer* > value_value (const_cast<ISinkSynchronizer* >(value), false);
    TInValue<int > id_value(id, false);
    Event event("ISinkController::setSinkSynchronizer", &value_value, &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setAverageCount(const int& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > value_value(value, false);
    Event event("ISinkController::setAverageCount", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setMaxDiscardingCount(const int& value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > value_value(value, false);
    Event event("ISinkController::setMaxDiscardingCount", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::waitForReset()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ISinkController::waitForReset");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::restart()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ISinkController::restart");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::enableWaitToPresent(const bool& enable)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<bool > enable_value(enable, false);
    Event event("ISinkController::enableWaitToPresent", &enable_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkController::setMinWaitInterval(const Interval& min_wait)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Interval > min_wait_value(min_wait, false);
    Event event("ISinkController::setMinWaitInterval", &min_wait_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISinkController::enableFirstBufferInTime(const bool& enable)
      throw(const Exception&)
  {
    TInValue<bool > enable_value(enable, false);
    Event event("ISinkController::enableFirstBufferInTime", &enable_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISinkController::enableBlocking(const bool& enable)
      throw(const Exception&)
  {
    TInValue<bool > enable_value(enable, false);
    Event event("ISinkController::enableBlocking", &enable_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* ISinkController::ISinkController_type = "ISinkController";
  TIFactoryObject<ISinkController > __ISinkController_prototype("ISinkController");


  const char* ISinkController::setMaxDelay_event = "ISinkController::setMaxDelay";
  const char* ISinkController::setMaxAverageDelay_event = "ISinkController::setMaxAverageDelay";
  const char* ISinkController::setTheoLatency_event = "ISinkController::setTheoLatency";
  const char* ISinkController::setDefaultTheoLatency_event = "ISinkController::setDefaultTheoLatency";
  const char* ISinkController::setSinkSynchronizer_event = "ISinkController::setSinkSynchronizer";
  const char* ISinkController::setAverageCount_event = "ISinkController::setAverageCount";
  const char* ISinkController::setMaxDiscardingCount_event = "ISinkController::setMaxDiscardingCount";
  const char* ISinkController::waitForReset_event = "ISinkController::waitForReset";
  const char* ISinkController::restart_event = "ISinkController::restart";
  const char* ISinkController::enableWaitToPresent_event = "ISinkController::enableWaitToPresent";
  const char* ISinkController::setMinWaitInterval_event = "ISinkController::setMinWaitInterval";
  const char* ISinkController::enableFirstBufferInTime_event = "ISinkController::enableFirstBufferInTime";
  const char* ISinkController::enableBlocking_event = "ISinkController::enableBlocking";


}


