
#include "ISourceController.hpp"
#include "nmm/base/InterfaceFactory.hpp"

#include "ISourceSynchronizer.hpp"

namespace NMM
{
  
  
  void ISourceController::setMasterSkew(const Interval& interval)
      throw(const Exception&)
  {
    TInValue<Interval > interval_value(interval, false);
    Event event("ISourceController::setMasterSkew", &interval_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISourceController::setStartTime(const Time& time)
      throw(const Exception&)
  {
    TInValue<Time > time_value(time, false);
    Event event("ISourceController::setStartTime", &time_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  Result ISourceController::setSourceSynchronizer(const ISourceSynchronizer* value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<ISourceSynchronizer* > value_value (const_cast<ISourceSynchronizer* >(value), false);
    Event event("ISourceController::setSourceSynchronizer", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISourceController::setSyncMaster(const bool& master)
      throw(const Exception&)
  {
    TInValue<bool > master_value(master, false);
    Event event("ISourceController::setSyncMaster", &master_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* ISourceController::ISourceController_type = "ISourceController";
  TIFactoryObject<ISourceController > __ISourceController_prototype("ISourceController");


  const char* ISourceController::setMasterSkew_event = "ISourceController::setMasterSkew";
  const char* ISourceController::setStartTime_event = "ISourceController::setStartTime";
  const char* ISourceController::setSourceSynchronizer_event = "ISourceController::setSourceSynchronizer";
  const char* ISourceController::setSyncMaster_event = "ISourceController::setSyncMaster";


}


