
#ifndef NMM_ISourceControllerImpl_HPP
#define NMM_ISourceControllerImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ISourceController.hpp"

#include "IControllerImpl.hpp"

namespace NMM
{
  /**
   * SourceController.
   * Interface for a source controller which helps the
   * SourceSynchronizer performing synchronization by
   * controlling the appropriate nodes
   */

  class ISourceControllerImpl  : virtual public IControllerImpl { 
  public:
    ISourceControllerImpl() { addInterface(new ISourceController(*this)); registerEvents(); };

    virtual ~ISourceControllerImpl() {};

  protected:
    /**
     * This method sets the time skew from the master of synchronization
     */

    virtual void setMasterSkew(const Interval& interval) = 0;
    /**
     * This method sets the start time for the controller. With the help of a start time, the nodes
     * can be told to start at e.g. 1 seconds after instantiating the Clock. This avoids scheduling
     * problems as one node would always be started some time after the other.
     */

    virtual void setStartTime(const Time& time) = 0;
    /**
     * This method sets the Synchronizer this Controller shall communicate with
     */

    virtual Result setSourceSynchronizer(const ISourceSynchronizer* value) = 0;
    /**
     * This method propagates the master flag to the node
     */

    virtual void setSyncMaster(const bool& master) = 0;


  private:
    void registerEvents();
  };

}

#endif
