
#include "ISynchronizedSource.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  ISourceController* ISynchronizedSource::getController()
      throw(const Exception&)
  {
    TOutValue<ISourceController* > return_value;
    Event event("ISynchronizedSource::getController");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISynchronizedSource::setSyncMaster(const bool& master)
      throw(const Exception&)
  {
    TInValue<bool > master_value(master, false);
    Event event("ISynchronizedSource::setSyncMaster", &master_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ISynchronizedSource::isSyncMaster()
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ISynchronizedSource::isSyncMaster");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISynchronizedSource::setConstantRateProducer(const bool& const_rate)
      throw(const Exception&)
  {
    TInValue<bool > const_rate_value(const_rate, false);
    Event event("ISynchronizedSource::setConstantRateProducer", &const_rate_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ISynchronizedSource::isConstantRateProducer()
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ISynchronizedSource::isConstantRateProducer");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISynchronizedSource::ISynchronizedSource_type = "ISynchronizedSource";
  TIFactoryObject<ISynchronizedSource > __ISynchronizedSource_prototype("ISynchronizedSource");


  const char* ISynchronizedSource::getController_event = "ISynchronizedSource::getController";
  const char* ISynchronizedSource::setSyncMaster_event = "ISynchronizedSource::setSyncMaster";
  const char* ISynchronizedSource::isSyncMaster_event = "ISynchronizedSource::isSyncMaster";
  const char* ISynchronizedSource::setConstantRateProducer_event = "ISynchronizedSource::setConstantRateProducer";
  const char* ISynchronizedSource::isConstantRateProducer_event = "ISynchronizedSource::isConstantRateProducer";


}


