
#ifndef NMM_ISynchronizedSourceImpl_HPP
#define NMM_ISynchronizedSourceImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ISynchronizedSource.hpp"

#include "ISynchronizedImpl.hpp"
#include "ISourceControllerImpl.hpp"

namespace NMM
{

  class ISynchronizedSourceImpl  : virtual public ISynchronizedImpl { 
  public:
    ISynchronizedSourceImpl() { addInterface(new ISynchronizedSource(*this)); registerEvents(); };

    virtual ~ISynchronizedSourceImpl() {};

  protected:
    /**
     * Get the controller for this node.
     * The controller interface is needed by the synchronizer to communicate with it
     * and through it with the node
     */

    virtual ISourceController* getController() = 0;
    /**
     * With this method the node can be told to be the master of synchronization
     *  \param master true  if the node should be master
     *                false if not
     */

    virtual void setSyncMaster(const bool& master) = 0;
    /**
     * This method returns whether the node is the master of synchronization
     */

    virtual bool isSyncMaster() = 0;
    /**
     * With this method one can set whether the underlying node produces its data
     * at a constant rate or not.
     *  \param master true  if the node produces its data at a constant rate
     *                false if not
     */

    virtual void setConstantRateProducer(const bool& const_rate) = 0;
    /**
     * This method returns whether the node produces its data at a constant rate.
     */

    virtual bool isConstantRateProducer() = 0;


  private:
    void registerEvents();
  };

}

#endif
