
#include "IFileModeHandler.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IFileModeHandler::setMode(const IFileModeHandler::FileMode& mode)
      throw(const Exception&)
  {
    TInValue<IFileModeHandler::FileMode > mode_value(mode, false);
    Event event("IFileModeHandler::setMode", &mode_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  IFileModeHandler::FileMode IFileModeHandler::getMode()
      throw(const Exception&)
  {
    TOutValue<IFileModeHandler::FileMode > return_value;
    Event event("IFileModeHandler::getMode");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IFileModeHandler::setFormat(const IFileModeHandler::FileFormat& format)
      throw(const Exception&)
  {
    TInValue<IFileModeHandler::FileFormat > format_value(format, false);
    Event event("IFileModeHandler::setFormat", &format_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  IFileModeHandler::FileFormat IFileModeHandler::getFormat()
      throw(const Exception&)
  {
    TOutValue<IFileModeHandler::FileFormat > return_value;
    Event event("IFileModeHandler::getFormat");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IFileModeHandler::setFilenameMode(const IFileModeHandler::FilenameMode& mode)
      throw(const Exception&)
  {
    TInValue<IFileModeHandler::FilenameMode > mode_value(mode, false);
    Event event("IFileModeHandler::setFilenameMode", &mode_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  IFileModeHandler::FilenameMode IFileModeHandler::getFilenameMode()
      throw(const Exception&)
  {
    TOutValue<IFileModeHandler::FilenameMode > return_value;
    Event event("IFileModeHandler::getFilenameMode");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IFileModeHandler::setLoop(const bool& d)
      throw(const Exception&)
  {
    TInValue<bool > d_value(d, false);
    Event event("IFileModeHandler::setLoop", &d_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IFileModeHandler::IFileModeHandler_type = "IFileModeHandler";
  TIFactoryObject<IFileModeHandler > __IFileModeHandler_prototype("IFileModeHandler");
  TValueConstructor< IFileModeHandler::FileMode > __IFileModeHandler_FileMode_prototype(TypeInfo("IFileModeHandler::FileMode", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const IFileModeHandler::FileMode& t)
  {
    ov.beginType(TypeInfo("IFileModeHandler::FileMode", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, IFileModeHandler::FileMode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IFileModeHandler::FileMode", "NMM")) {
      throw InvalidTypeException(TypeInfo("IFileModeHandler::FileMode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }

  TValueConstructor< IFileModeHandler::FileFormat > __IFileModeHandler_FileFormat_prototype(TypeInfo("IFileModeHandler::FileFormat", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const IFileModeHandler::FileFormat& t)
  {
    ov.beginType(TypeInfo("IFileModeHandler::FileFormat", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, IFileModeHandler::FileFormat& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IFileModeHandler::FileFormat", "NMM")) {
      throw InvalidTypeException(TypeInfo("IFileModeHandler::FileFormat", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }

  TValueConstructor< IFileModeHandler::FilenameMode > __IFileModeHandler_FilenameMode_prototype(TypeInfo("IFileModeHandler::FilenameMode", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const IFileModeHandler::FilenameMode& t)
  {
    ov.beginType(TypeInfo("IFileModeHandler::FilenameMode", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, IFileModeHandler::FilenameMode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IFileModeHandler::FilenameMode", "NMM")) {
      throw InvalidTypeException(TypeInfo("IFileModeHandler::FilenameMode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }



  const char* IFileModeHandler::setMode_event = "IFileModeHandler::setMode";
  const char* IFileModeHandler::getMode_event = "IFileModeHandler::getMode";
  const char* IFileModeHandler::setFormat_event = "IFileModeHandler::setFormat";
  const char* IFileModeHandler::getFormat_event = "IFileModeHandler::getFormat";
  const char* IFileModeHandler::setFilenameMode_event = "IFileModeHandler::setFilenameMode";
  const char* IFileModeHandler::getFilenameMode_event = "IFileModeHandler::getFilenameMode";
  const char* IFileModeHandler::setLoop_event = "IFileModeHandler::setLoop";


}


