
#include "ITimeshifting.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void ITimeshifting::enableMemBuf(const unsigned int& size /* = 60 */)
      throw(const Exception&)
  {
    TInValue<unsigned int > size_value(size, false);
    Event event("ITimeshifting::enableMemBuf", &size_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITimeshifting::disableMemBuf(const bool& play /* = false */)
      throw(const Exception&)
  {
    TInValue<bool > play_value(play, false);
    Event event("ITimeshifting::disableMemBuf", &play_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ITimeshifting::isMemBuf()
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ITimeshifting::isMemBuf");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITimeshifting::keepData(const bool& keep)
      throw(const Exception&)
  {
    TInValue<bool > keep_value(keep, false);
    Event event("ITimeshifting::keepData", &keep_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  ITimeshifting::Mode ITimeshifting::getMode() const
      throw(const Exception&)
  {
    TOutValue<ITimeshifting::Mode > return_value;
    Event event("ITimeshifting::getMode");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITimeshifting::PlayMode ITimeshifting::getPlayMode() const
      throw(const Exception&)
  {
    TOutValue<ITimeshifting::PlayMode > return_value;
    Event event("ITimeshifting::getPlayMode");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITimeshifting::setModeLive(const bool& enable_memory_buffer /* = true */, const bool& send_sync_event /* = true */)
      throw(const Exception&)
  {
    TInValue<bool > enable_memory_buffer_value(enable_memory_buffer, false);
    TInValue<bool > send_sync_event_value(send_sync_event, false);
    Event event("ITimeshifting::setModeLive", &enable_memory_buffer_value, &send_sync_event_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setModeFBLive(const bool& send_sync_event /* = true */)
      throw(const Exception&)
  {
    TInValue<bool > send_sync_event_value(send_sync_event, false);
    Event event("ITimeshifting::setModeFBLive", &send_sync_event_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setModePlay(const bool& send_sync_reset /* = true */)
      throw(const Exception&)
  {
    TInValue<bool > send_sync_reset_value(send_sync_reset, false);
    Event event("ITimeshifting::setModePlay", &send_sync_reset_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setModePause()
      throw(const Exception&)
  {
    Event event("ITimeshifting::setModePause");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setModeFastForward(const int& speed /* = 5 */)
      throw(const Exception&)
  {
    TInValue<int > speed_value(speed, false);
    Event event("ITimeshifting::setModeFastForward", &speed_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setModeRewind(const int& speed /* = -5 */)
      throw(const Exception&)
  {
    TInValue<int > speed_value(speed, false);
    Event event("ITimeshifting::setModeRewind", &speed_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setFilename(const string& filename)
      throw(const Exception&)
  {
    TInValue<string > filename_value(filename, false);
    Event event("ITimeshifting::setFilename", &filename_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITimeshifting::setFileBufferThreshhold(const unsigned int& threshold) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TInValue<unsigned int > threshold_value(threshold, false);
    Event event("ITimeshifting::setFileBufferThreshhold", &threshold_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  Result ITimeshifting::setBufferSize(const unsigned int& size)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned int > size_value(size, false);
    Event event("ITimeshifting::setBufferSize", &size_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITimeshifting::startRecord()
      throw(const Exception&)
  {
    Event event("ITimeshifting::startRecord");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITimeshifting::stopRecord()
      throw(const Exception&)
  {
    Event event("ITimeshifting::stopRecord");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ITimeshifting::isRecording()
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ITimeshifting::isRecording");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITimeshifting::saveToFile(const string& filename)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > filename_value(filename, false);
    Event event("ITimeshifting::saveToFile", &filename_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Time ITimeshifting::getReadPosition() const /* limited(INode::STARTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Time > return_value;
    Event event("ITimeshifting::getReadPosition");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Time ITimeshifting::getStartTime() const /* limited(INode::STARTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Time > return_value;
    Event event("ITimeshifting::getStartTime");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Time ITimeshifting::getLivePosition() const /* limited(INode::STARTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Time > return_value;
    Event event("ITimeshifting::getLivePosition");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interval ITimeshifting::getLiveOffset() const /* limited(INode::STARTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Interval > return_value;
    Event event("ITimeshifting::getLiveOffset");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Interval ITimeshifting::getFileLength() const /* limited(INode::STARTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Interval > return_value;
    Event event("ITimeshifting::getFileLength");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITimeshifting::ITimeshifting_type = "ITimeshifting";
  TIFactoryObject<ITimeshifting > __ITimeshifting_prototype("ITimeshifting");
  TValueConstructor< ITimeshifting::Mode > __ITimeshifting_Mode_prototype(TypeInfo("ITimeshifting::Mode", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const ITimeshifting::Mode& t)
  {
    ov.beginType(TypeInfo("ITimeshifting::Mode", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, ITimeshifting::Mode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("ITimeshifting::Mode", "NMM")) {
      throw InvalidTypeException(TypeInfo("ITimeshifting::Mode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }

  TValueConstructor< ITimeshifting::PlayMode > __ITimeshifting_PlayMode_prototype(TypeInfo("ITimeshifting::PlayMode", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const ITimeshifting::PlayMode& t)
  {
    ov.beginType(TypeInfo("ITimeshifting::PlayMode", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, ITimeshifting::PlayMode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("ITimeshifting::PlayMode", "NMM")) {
      throw InvalidTypeException(TypeInfo("ITimeshifting::PlayMode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }



  const char* ITimeshifting::enableMemBuf_event = "ITimeshifting::enableMemBuf";
  const char* ITimeshifting::disableMemBuf_event = "ITimeshifting::disableMemBuf";
  const char* ITimeshifting::isMemBuf_event = "ITimeshifting::isMemBuf";
  const char* ITimeshifting::keepData_event = "ITimeshifting::keepData";
  const char* ITimeshifting::getMode_event = "ITimeshifting::getMode";
  const char* ITimeshifting::getPlayMode_event = "ITimeshifting::getPlayMode";
  const char* ITimeshifting::setModeLive_event = "ITimeshifting::setModeLive";
  const char* ITimeshifting::setModeFBLive_event = "ITimeshifting::setModeFBLive";
  const char* ITimeshifting::setModePlay_event = "ITimeshifting::setModePlay";
  const char* ITimeshifting::setModePause_event = "ITimeshifting::setModePause";
  const char* ITimeshifting::setModeFastForward_event = "ITimeshifting::setModeFastForward";
  const char* ITimeshifting::setModeRewind_event = "ITimeshifting::setModeRewind";
  const char* ITimeshifting::setFilename_event = "ITimeshifting::setFilename";
  const char* ITimeshifting::setFileBufferThreshhold_event = "ITimeshifting::setFileBufferThreshhold";
  const char* ITimeshifting::setBufferSize_event = "ITimeshifting::setBufferSize";
  const char* ITimeshifting::startRecord_event = "ITimeshifting::startRecord";
  const char* ITimeshifting::stopRecord_event = "ITimeshifting::stopRecord";
  const char* ITimeshifting::isRecording_event = "ITimeshifting::isRecording";
  const char* ITimeshifting::saveToFile_event = "ITimeshifting::saveToFile";
  const char* ITimeshifting::getReadPosition_event = "ITimeshifting::getReadPosition";
  const char* ITimeshifting::getStartTime_event = "ITimeshifting::getStartTime";
  const char* ITimeshifting::getLivePosition_event = "ITimeshifting::getLivePosition";
  const char* ITimeshifting::getLiveOffset_event = "ITimeshifting::getLiveOffset";
  const char* ITimeshifting::getFileLength_event = "ITimeshifting::getFileLength";
  
  
  Result ITimeshiftingListener::timeshiftingBegin() /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITimeshiftingListener::timeshiftingBegin");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITimeshiftingListener::live() /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITimeshiftingListener::live");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITimeshiftingListener::setProgress(const Time& ts_time) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Time > ts_time_value(ts_time, false);
    Event event("ITimeshiftingListener::setProgress", &ts_time_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITimeshiftingListener::ITimeshiftingListener_type = "ITimeshiftingListener";
  TIFactoryObject<ITimeshiftingListener > __ITimeshiftingListener_prototype("ITimeshiftingListener");


  const char* ITimeshiftingListener::timeshiftingBegin_event = "ITimeshiftingListener::timeshiftingBegin";
  Event* ITimeshiftingListener::create_timeshiftingBegin()
  {

    return new Event("ITimeshiftingListener::timeshiftingBegin");
  }

  const char* ITimeshiftingListener::live_event = "ITimeshiftingListener::live";
  Event* ITimeshiftingListener::create_live()
  {

    return new Event("ITimeshiftingListener::live");
  }

  const char* ITimeshiftingListener::setProgress_event = "ITimeshiftingListener::setProgress";
  Event* ITimeshiftingListener::create_setProgress(const Time& ts_time)
  {
    TInValue<Time >* ts_time_value = new TInValue<Time >();
    ts_time_value->setValue(ts_time);

    return new Event("ITimeshiftingListener::setProgress", ts_time_value);
  }



}


