
#ifndef NMM_IProgress_HPP
#define NMM_IProgress_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   *  @brief Progress sender interface
   *
   *  A progress sender is a node which, if enabled, sends progress events downstream.
   *  An IProgressListener interface can be used to catch these events.
   */

  class IProgress : virtual public Interface { 
  public:
    IProgress() {  setTypeInfo(TypeInfo(IProgress::IProgress_type, "NMM")); };

    IProgress(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IProgress::IProgress_type, "NMM")); setEventReceiver(parent); };

    virtual ~IProgress() {};

    static const char* IProgress_type;

    virtual  string getInterfaceType() const { return IProgress_type; }

    Interface* copy() const { return new IProgress(*this); }
    static const char* sendProgressInformation_event;
    static const char* setProgressInterval_event;
    /**
     *  @brief Enable or disable sending progress information
     *  @param enabled True to enable sending progress information
     *
     *  This method enables or disables sending of progress information. If enables, this node will send progress
     *  events (IProgressListener::setProgress) at regular intervals, defined by the node.
     */

    virtual void sendProgressInformation(const bool& value)
      throw(const Exception&);
    /**
     *  @brief Sets the interval for sending setProgress events
     *  @param The number of buffers after which a setProgress event should be sent
     *
     *  This method changes the interval in which setProgress events are sent.
     */

    virtual void setProgressInterval(const int& buffers)
      throw(const Exception&);


  };

  typedef auto_ptr<IProgress> IProgress_var;




}

#endif
