
#ifndef NMM_IProgressListenerImpl_HPP
#define NMM_IProgressListenerImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IProgressListener.hpp"


namespace NMM
{
  /**
   *  @brief Progress events
   *
   *  Instream events for sending progress information.
   *  Nodes that implement the IProgress interface send events from this interface.
   *  Applications that need progress information typically register listeners for these events.
   */

  class IProgressListenerImpl  : public virtual NMMObject { 
  public:
    IProgressListenerImpl() { addInterface(new IProgressListener(*this)); registerEvents(); };

    virtual ~IProgressListenerImpl() {};

  protected:
    /**
     *  @brief Progress information.
     *  @param numerator the current progress
     *  @param denominator the maximum progress
     *
     *  Instream event containing the current state of progress.
     */

    virtual void setProgress(const u_int64_t& numerator, const u_int64_t& denominator) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#endif
