
#include "ITVGuide.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result ITVGuide::insertChannel(const string& id /* = "" */, const string& name /* = "" */)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > id_value(id, false);
    TInValue<string > name_value(name, false);
    Event event("ITVGuide::insertChannel", &id_value, &name_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuide::insertChannelDescription(const ChannelDescription& cd)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<ChannelDescription > cd_value(cd, false);
    Event event("ITVGuide::insertChannelDescription", &cd_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuide::insertEvent(const ProgramDescription& pd)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<ProgramDescription > pd_value(pd, false);
    Event event("ITVGuide::insertEvent", &pd_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITVGuide::setDirectory(const string& path /* = "" */)
      throw(const Exception&)
  {
    TInValue<string > path_value(path, false);
    Event event("ITVGuide::setDirectory", &path_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  string ITVGuide::getDirectory()
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("ITVGuide::getDirectory");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<ChannelDescription >  ITVGuide::getChannelsList()
      throw(const Exception&)
  {
    TOutValue<list<ChannelDescription >  > return_value;
    Event event("ITVGuide::getChannelsList");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ITVGuide::getChannelById(const string& id)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<string > id_value(id, false);
    Event event("ITVGuide::getChannelById", &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ITVGuide::getChannelByName(const string& name)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<string > name_value(name, false);
    Event event("ITVGuide::getChannelByName", &name_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ProgramDescription ITVGuide::getEvent(const string& start, const string& end, const string& id)
      throw(const Exception&)
  {
    TOutValue<ProgramDescription > return_value;
    TInValue<string > start_value(start, false);
    TInValue<string > end_value(end, false);
    TInValue<string > id_value(id, false);
    Event event("ITVGuide::getEvent", &start_value, &end_value, &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<ProgramDescription >  ITVGuide::getEventsOnDayOnChannel(const string& day, const string& chan)
      throw(const Exception&)
  {
    TOutValue<list<ProgramDescription >  > return_value;
    TInValue<string > day_value(day, false);
    TInValue<string > chan_value(chan, false);
    Event event("ITVGuide::getEventsOnDayOnChannel", &day_value, &chan_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<ProgramDescription >  ITVGuide::search(const ProgramFilter& pf, const ITVGuidePeerObject* peer /* = NULL */)
      throw(const Exception&)
  {
    TOutValue<list<ProgramDescription >  > return_value;
    TInValue<ProgramFilter > pf_value(pf, false);
    TInValue<ITVGuidePeerObject* > peer_value(const_cast<ITVGuidePeerObject* >(peer), false);
    Event event("ITVGuide::search", &pf_value, &peer_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITVGuide::addExtDescription(const string& start, const string& end, const string& id, const string& desc)
      throw(const Exception&)
  {
    TInValue<string > start_value(start, false);
    TInValue<string > end_value(end, false);
    TInValue<string > id_value(id, false);
    TInValue<string > desc_value(desc, false);
    Event event("ITVGuide::addExtDescription", &start_value, &end_value, &id_value, &desc_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  ProgramDescription ITVGuide::getCurrentEventOnChannel(const string& channel)
      throw(const Exception&)
  {
    TOutValue<ProgramDescription > return_value;
    TInValue<string > channel_value(channel, false);
    Event event("ITVGuide::getCurrentEventOnChannel", &channel_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ProgramDescription ITVGuide::getFollowingEventOnChannel(const string& channel)
      throw(const Exception&)
  {
    TOutValue<ProgramDescription > return_value;
    TInValue<string > channel_value(channel, false);
    Event event("ITVGuide::getFollowingEventOnChannel", &channel_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<string >  ITVGuide::getCategories()
      throw(const Exception&)
  {
    TOutValue<list<string >  > return_value;
    Event event("ITVGuide::getCategories");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuide::start()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITVGuide::start");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuide::stop()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITVGuide::stop");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuide::init()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITVGuide::init");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVGuide::deinit()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("ITVGuide::deinit");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITVGuidePlugin* ITVGuide::getGrabberInterface(const string& type)
      throw(const Exception&)
  {
    TOutValue<ITVGuidePlugin* > return_value;
    TInValue<string > type_value(type, false);
    Event event("ITVGuide::getGrabberInterface", &type_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ITVGuide::addPeerHost(const string& host, const int& port)
      throw(const Exception&)
  {
    TInValue<string > host_value(host, false);
    TInValue<int > port_value(port, false);
    Event event("ITVGuide::addPeerHost", &host_value, &port_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITVGuide::clearPeerHosts()
      throw(const Exception&)
  {
    Event event("ITVGuide::clearPeerHosts");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITVGuide::cleanDatabase(const int& days)
      throw(const Exception&)
  {
    TInValue<int > days_value(days, false);
    Event event("ITVGuide::cleanDatabase", &days_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* ITVGuide::ITVGuide_type = "ITVGuide";
  TIFactoryObject<ITVGuide > __ITVGuide_prototype("ITVGuide");


  const char* ITVGuide::insertChannel_event = "ITVGuide::insertChannel";
  const char* ITVGuide::insertChannelDescription_event = "ITVGuide::insertChannelDescription";
  const char* ITVGuide::insertEvent_event = "ITVGuide::insertEvent";
  const char* ITVGuide::setDirectory_event = "ITVGuide::setDirectory";
  const char* ITVGuide::getDirectory_event = "ITVGuide::getDirectory";
  const char* ITVGuide::getChannelsList_event = "ITVGuide::getChannelsList";
  const char* ITVGuide::getChannelById_event = "ITVGuide::getChannelById";
  const char* ITVGuide::getChannelByName_event = "ITVGuide::getChannelByName";
  const char* ITVGuide::getEvent_event = "ITVGuide::getEvent";
  const char* ITVGuide::getEventsOnDayOnChannel_event = "ITVGuide::getEventsOnDayOnChannel";
  const char* ITVGuide::search_event = "ITVGuide::search";
  const char* ITVGuide::addExtDescription_event = "ITVGuide::addExtDescription";
  const char* ITVGuide::getCurrentEventOnChannel_event = "ITVGuide::getCurrentEventOnChannel";
  const char* ITVGuide::getFollowingEventOnChannel_event = "ITVGuide::getFollowingEventOnChannel";
  const char* ITVGuide::getCategories_event = "ITVGuide::getCategories";
  const char* ITVGuide::start_event = "ITVGuide::start";
  const char* ITVGuide::stop_event = "ITVGuide::stop";
  const char* ITVGuide::init_event = "ITVGuide::init";
  const char* ITVGuide::deinit_event = "ITVGuide::deinit";
  const char* ITVGuide::getGrabberInterface_event = "ITVGuide::getGrabberInterface";
  const char* ITVGuide::addPeerHost_event = "ITVGuide::addPeerHost";
  const char* ITVGuide::clearPeerHosts_event = "ITVGuide::clearPeerHosts";
  const char* ITVGuide::cleanDatabase_event = "ITVGuide::cleanDatabase";


}


