/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        chwellner <chwellner@graphics.cs.uni-sb.de>
 *                     
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "ProgramDescription.hpp"

#include <stdio.h>

namespace NMM{

  ProgramDescription::ProgramDescription()
    : NMM::SerializedValue("ProgramDescription","NMM"),
      sourceType(ProgramDescription::UNKNOWN)
  {
    registerValues();
  }

  ProgramDescription::ProgramDescription(sourceType_t source)
    : NMM::SerializedValue("ProgramDescription","NMM"),
      sourceType(source)
  {
    registerValues();
  }

  ProgramDescription::ProgramDescription(const ProgramDescription& pd)
    : NMM::SerializedValue("ProgramDescription","NMM"),
      startTime(""),
      endTime(""),
      duration(0),
      desc("")
  {
    registerValues();
    *this=pd;
  }

  ProgramDescription::~ProgramDescription(){}
  
  bool ProgramDescription::empty(){
    if(startTime.getRValue() == ""&& endTime.getRValue()== ""
       && channel.getRValue().getID() == "" && title.getRValue().empty()){
      return true;
    }else{
      return false;
    }
  }

  void ProgramDescription::setStartTime(string st){
    startTime = st;
  }

  string ProgramDescription::getStartTime() const {
    return startTime.getRValue();
  }
  
  void ProgramDescription::setEndTime(string et) {
    endTime = et;
  }
  string ProgramDescription::getEndTime() const {
    return endTime.getRValue();
  }
    

  void ProgramDescription::setChannelID(string id){
    channel.getRValue().setID(id);
  }
  string ProgramDescription::getChannelID() const{
    return channel.getRValue().getID();
  }
  
  void ProgramDescription::setChannelDescription(ChannelDescription cd){
    channel = cd;
  }
  ChannelDescription ProgramDescription::getChannelDescription() const{
    return channel.getRValue();
  }
  
  void ProgramDescription::setDuration(unsigned int d){
    duration = d;
  }
  unsigned int ProgramDescription::getDuration() const {
    return duration.getRValue();
  }
  
  void ProgramDescription::setTitles(list<string> titles){
    title = titles;
  }
  void ProgramDescription::addTitle(string t){
    (title.getRValue()).push_back(t);
  }
  list<string> ProgramDescription::getTitles() const {
    return title.getRValue();
  }

  void ProgramDescription::setDescription(string d){
    desc = d;
  }
  string ProgramDescription::getDescription() const {
    return desc.getRValue();
  }
  
  void ProgramDescription::addCategory(string c){
    category.getRValue().push_back(c);
  }
  list<string> ProgramDescription::getCategory() const {
    return category.getRValue();
  }

  ProgramDescription* ProgramDescription::copy(){
    return new ProgramDescription(*this);
  }

  void ProgramDescription::registerValues(){
    insertArgument(&startTime);
    insertArgument(&endTime);
    insertArgument(&channel);
    insertArgument(&duration);
    insertArgument(&title);
    insertArgument(&desc);
    insertArgument(&sourceType);
    insertArgument(&category);
  }

  bool ProgramDescription::operator<(const ProgramDescription &pd) const{

    if(startTime.getRValue() < pd.getStartTime()){
      return true;
    }else{
      if(startTime.getRValue() == pd.getStartTime() && channel.getRValue().getID() < pd.getChannelID()){
        return true;
      }
    }
        
    return false;
  }

  bool ProgramDescription::operator==(const ProgramDescription &pd) const{
    if(startTime.getRValue() == pd.getStartTime() && channel.getRValue().getID() == pd.getChannelID()){
      return true;
    }else{
      return false;
    }
    return false;
  }

  void ProgramDescription::localizeTimes(){
    startTime = transformTime(startTime.getRValue());
    endTime   = transformTime(endTime.getRValue());
  }
  
  string ProgramDescription::transformTime(string s){
    try{
      
      time_t localTime = time(NULL);
      struct tm *t = localtime(&localTime); //Just to be sure, that external time-variables of time.h are initialized
      int cdst = t->tm_isdst;
     
      tzset();

      time_t xmlTime = time(NULL);
      struct tm *abs_time = localtime(&xmlTime);
      abs_time->tm_sec   = 0;
      abs_time->tm_hour  = atoi( (s.substr(8,2)).c_str() );
      abs_time->tm_min   = atoi( (s.substr(10,2)).c_str() );
      abs_time->tm_mday  = atoi( (s.substr(6,2)).c_str() );
      abs_time->tm_mon   = atoi( (s.substr(4,2)).c_str() )-1;
      abs_time->tm_year  = atoi( (s.substr(0,4)).c_str() )-1900;
      xmlTime = mktime( abs_time );

      struct tm *xmltime_tm = localtime(&xmlTime);

      
      //if the date is in dst, convert it
      double timeDifference = 0.0;
      if(xmltime_tm->tm_isdst >= 0){
        xmlTime -= ((xmltime_tm->tm_isdst - cdst) * 3600);
        timeDifference = xmltime_tm->tm_isdst * 100.0;
      }

      unsigned int pos = s.find_first_of(' ');
      int xmlDifference= 0;
      if(pos != string::npos){
        //if no timezone-info is available, assume UTC
        xmlDifference = atoi(s.substr(pos+1,5).c_str())/100;
      }
      
      //should dst be respected when computing the timezone?
      if(daylight != 0 && t->tm_isdst == 1) xmlTime += 3600;

      char timeString[25];
      //Compute the difference between the xml-Timezone and our timezone and update the outputtime accordingly
      xmlTime += ((-1*timezone) - (xmlDifference*3600));
      timeDifference += -1*timezone/60/60 * 100; //in hours
      strftime(timeString, sizeof(timeString), "%Y%m%d%H%M", localtime(&xmlTime));
      sprintf(timeString, "%s %0+5.0f", timeString, timeDifference);

      //cerr << s << " " << timeString << " " 
      //     << daylight << " " << cdst << " " << xmltime_tm->tm_isdst << " " << timezone << " " << xmlDifference << " " << endl;

      return string(timeString);
      
    }catch (...){
      return s;
    }    

  }

  ostream& operator << (ostream& os, const ProgramDescription& pd){
    os << "<programme start=\"" << pd.getStartTime() << "\" stop=\"" << pd.getEndTime() 
       << "\" channel=\"" << pd.getChannelID() << "\">" << endl
       << "<duration unites=\"min\">" << pd.getDuration() << "</duration>" <<endl
       << "<desc>" << pd.getDescription() << "</desc>" << endl;

    list<string> l = pd.getTitles();
    for(list<string>::iterator i = l.begin(); i != l.end(); i++){
      os << "<title>" << *i << "</title>" << endl;
    }

    list<string> c = pd.getCategory();
    for(list<string>::iterator i = c.begin(); i != c.end(); i++){
      os << "<category>" << *i << "</category>" << endl;
    }
    os << "</programme> " << endl;
    
    return os;
  }

  OValueStream& operator<<(OValueStream& os, const list<ProgramDescription>& pd){
    return writeList<ProgramDescription>(os, pd, "ProgramDescription");
  }
  IValueStream& operator>>(IValueStream& is, list<ProgramDescription>& pd){
    return readList<ProgramDescription>(is, pd, "ProgramDescription");
  }
  
  TConstructor<ProgramDescription> nmm_pd("ProgramDescription", "NMM");
  TValueConstructor<list<ProgramDescription> > nmm_pd_list("list_ProgramDescription","NMM");


  OValueStream& operator<<(OValueStream& os, const list<ProgramDescriptionList>& pl){
    return writeList<ProgramDescriptionList>(os, pl, "ProgramDescriptionList");
  }
  IValueStream& operator>>(IValueStream& is, list<ProgramDescriptionList>& pl){
    return readList<ProgramDescriptionList>(is, pl, "ProgramDescriptionList");
  }
  
  TConstructor<ProgramDescriptionList> nmm_pl("ProgramDescriptionList", "NMM");
  TValueConstructor<list<ProgramDescriptionList> > nmm_pl_list("list_ProgramDescriptionList","NMM");
}
