
#ifndef NMM_ITVManager_HPP
#define NMM_ITVManager_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/interfaces/base/INode.hpp"
#include "nmm/interfaces/base/IUsageNotify.hpp"

namespace NMM
{
  /**
   *This interface is used by the TVManager to notify applications about certain Events
   */

  class ITVManager : virtual public Interface { 
  public:
    ITVManager() {  setTypeInfo(TypeInfo(ITVManager::ITVManager_type, "NMM")); };

    ITVManager(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ITVManager::ITVManager_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITVManager() {};

    static const char* ITVManager_type;

    virtual  string getInterfaceType() const { return ITVManager_type; }

    Interface* copy() const { return new ITVManager(*this); }
    static const char* setChannel_event;
    static const char* channelUp_event;
    static const char* channelDown_event;
    static const char* getTVCard_event;
    static const char* startGraph_event;
    static const char* setUsageNotify_event;
    static const char* releaseGraph_event;
    static const char* switchingSource_event;
    static const char* sharedSource_event;
    static const char* skippingChannel_event;

    virtual Result setChannel(const unsigned int& channel)
      throw(const Exception&);

    virtual Result channelUp()
      throw(const Exception&);

    virtual Result channelDown()
      throw(const Exception&);

    virtual INode* getTVCard()
      throw(const Exception&);

    virtual Result startGraph()
      throw(const Exception&);

    virtual void setUsageNotify(const IUsageNotify* u)
      throw(const Exception&);

    virtual void releaseGraph()
      throw(const Exception&);
    /**
     *Indicates on channel-switch that the next channel is not available on the current TVCard and the source will be switched
     */

    virtual Result switchingSource()
      throw(const Exception&);
    /**
     *Indicates that the application couldn't request the source exclusive and is only slave
     */

    virtual Result sharedSource()
      throw(const Exception&);
    /**
     *Indocates, that the next channel is unavailable and therefore left out while channel-hopping
     */

    virtual Result skippingChannel()
      throw(const Exception&);


  };

  typedef auto_ptr<ITVManager> ITVManager_var;




}

#endif
