
#ifndef NMM_IVNC_HPP
#define NMM_IVNC_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class IVNC : virtual public Interface { 
  public:
  enum Mode { 
    REAL_TIME , 
    REQUESTED , 
    BLANK_SHEET  };

    IVNC() {  setTypeInfo(TypeInfo(IVNC::IVNC_type, "NMM")); };

    IVNC(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IVNC::IVNC_type, "NMM")); setEventReceiver(parent); };

    virtual ~IVNC() {};

    static const char* IVNC_type;

    virtual  string getInterfaceType() const { return IVNC_type; }

    Interface* copy() const { return new IVNC(*this); }
    static const char* setTolerance_event;
    static const char* setInterval_event;
    static const char* setShared_event;
    static const char* getTolerance_event;
    static const char* getInterval_event;
    static const char* setPasswd_event;
    static const char* setPasswdPath_event;
    static const char* setMode_event;
    static const char* isShared_event;
    static const char* sendKeyEvent_event;
    static const char* sendPointerEvent_event;
    static const char* sendFrameBufferUpdateRequest_event;

    virtual void setTolerance(const float& tolerance)
      throw(const Exception&);

    virtual void setInterval(const float& interval)
      throw(const Exception&);

    virtual void setShared(const bool& unshared)
      throw(const Exception&);

    virtual float getTolerance() const
      throw(const Exception&);

    virtual float getInterval() const
      throw(const Exception&);

    virtual void setPasswd(const string& passwd)
      throw(const Exception&);

    virtual void setPasswdPath(const string& passwdpath)
      throw(const Exception&);

    virtual void setMode(const IVNC::Mode& mode)
      throw(const Exception&);

    virtual bool isShared() const
      throw(const Exception&);

    virtual void sendKeyEvent(const unsigned long& key, const int& down)
      throw(const Exception&);

    virtual void sendPointerEvent(const int& x, const int& y, const int& buttonmask)
      throw(const Exception&);

    virtual void sendFrameBufferUpdateRequest()
      throw(const Exception&);


  };

  typedef auto_ptr<IVNC> IVNC_var;

  OValueStream& operator<<(OValueStream& ov, const IVNC::Mode& t);
  IValueStream& operator>>(IValueStream& iv, IVNC::Mode& t);



}

#endif
