
#ifndef NMM_IBuffersPassedEvent_HPP
#define NMM_IBuffersPassedEvent_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   * Node that sends the number of buffers passed upstream.
   * This interface is for setting the number of buffers after which to send an upstream event
   * containing the number of buffers passed. The event is received by a node that implements
   * IBuffersPassedEventReceiver.
   */

  class IBuffersPassedEventSender : virtual public Interface { 
  public:
    IBuffersPassedEventSender() {  setTypeInfo(TypeInfo(IBuffersPassedEventSender::IBuffersPassedEventSender_type, "NMM")); };

    IBuffersPassedEventSender(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IBuffersPassedEventSender::IBuffersPassedEventSender_type, "NMM")); setEventReceiver(parent); };

    virtual ~IBuffersPassedEventSender() {};

    static const char* IBuffersPassedEventSender_type;

    virtual  string getInterfaceType() const { return IBuffersPassedEventSender_type; }

    Interface* copy() const { return new IBuffersPassedEventSender(*this); }
    static const char* setBufferCountInterval_event;
    static const char* getBufferCountInterval_event;
    /**
     * Set buffer count interval.
     * Sets the number of buffers after which to send the upstream event.
     * Setting it to zero disables sending the event. Buffers processed while the interval is zero
     * are not counted in upstream events.
     */

    virtual void setBufferCountInterval(const int& value)
      throw(const Exception&);
    /**
     * Get buffer count interval.
     * Gets the number of buffers after which to send the upstream event.
     * Zero means no upstream events are sent.
     */

    virtual int getBufferCountInterval() const
      throw(const Exception&);


  };

  typedef auto_ptr<IBuffersPassedEventSender> IBuffersPassedEventSender_var;


  /**
   * Node that receives events containing the number of buffers passed.
   * These events are sent upstream from a node which implements IBuffersPassedEventSender.
   */

  class IBuffersPassedEventReceiver : virtual public Interface { 
  public:
    IBuffersPassedEventReceiver() {  setTypeInfo(TypeInfo(IBuffersPassedEventReceiver::IBuffersPassedEventReceiver_type, "NMM")); };

    IBuffersPassedEventReceiver(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IBuffersPassedEventReceiver::IBuffersPassedEventReceiver_type, "NMM")); setEventReceiver(parent); };

    virtual ~IBuffersPassedEventReceiver() {};

    static const char* IBuffersPassedEventReceiver_type;

    virtual  string getInterfaceType() const { return IBuffersPassedEventReceiver_type; }

    Interface* copy() const { return new IBuffersPassedEventReceiver(*this); }
    static const char* buffersPassed_event;
    static Event* create_buffersPassed(const int& buffers);
    static const char* getUpstreamBuffersPassed_event;
    static const char* resetUpstreamBufferCounter_event;
  /* instream (protected) and non-instream (public) */        /**
     * Receive number of buffers processed recently.
     * This is an upstream event. The value is the number of buffers processed since the last event.
     */

    virtual Result buffersPassed(const int& buffers) /* in-stream */ 
      throw(const Exception&);
  /* non-instream (public) */        /**
     * Get number of buffers processed below.
     * Returns the number of buffers processed below in the graph, according to the upstream events
     * we received.
     */

    virtual int getUpstreamBuffersPassed() const
      throw(const Exception&);
    /**
     * Reset number of buffers processed below.
     */

    virtual void resetUpstreamBufferCounter()
      throw(const Exception&);


  };

  typedef auto_ptr<IBuffersPassedEventReceiver> IBuffersPassedEventReceiver_var;




}

#endif
