
#ifndef NMM_IEncodeStatsImpl_HPP
#define NMM_IEncodeStatsImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IEncodeStats.hpp"


namespace NMM
{
  /**
   * Encoding statistics.
   * Interface for encoders that provide statistics by sending events
   * to an IEncodeStatsListener.
   */

  class IEncodeStatsImpl  : public virtual NMMObject { 
  public:
    IEncodeStatsImpl() { addInterface(new IEncodeStats(*this)); registerEvents(); };

    virtual ~IEncodeStatsImpl() {};

  protected:
    /**
     * Toggle sending frame count events.
     * Turns sending frame count events on or off. A frame count event
     * contains the number of frames encoded so far and the encoding
     * framerate.
     */

    virtual void sendFrameCount(const bool& value) = 0;


  private:
    void registerEvents();
  };
  /**
   * Encoding statistics events.
   * Contains events that are sent downstream by an encoder that implements
   * IEncodeStats.
   */

  class IEncodeStatsListenerImpl  : public virtual NMMObject { 
  public:
    IEncodeStatsListenerImpl() { addInterface(new IEncodeStatsListener(*this)); registerEvents(); };

    virtual ~IEncodeStatsListenerImpl() {};

  protected:
    /**
     * Frame count event.
     * Statistical event sent by the encoder.
     * @param count number of frames encoded so far
     * @param fps current encoding framerate in frames per second
     */

    virtual void setFrameCount(const unsigned int& count, const float& fps) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#endif
