
#include "IMethodCall.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IMethodCall::noArgumentNoReturn()
      throw(const Exception&)
  {
    Event event("IMethodCall::noArgumentNoReturn");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int IMethodCall::noArgumentIntReturn()
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("IMethodCall::noArgumentIntReturn");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int IMethodCall::oneArgumentInIntReturn(const int& value1)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInValue<int > value1_value(value1, false);
    Event event("IMethodCall::oneArgumentInIntReturn", &value1_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int IMethodCall::twoArgumentInIntReturn(const int& value1, const int& value2)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInValue<int > value1_value(value1, false);
    TInValue<int > value2_value(value2, false);
    Event event("IMethodCall::twoArgumentInIntReturn", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int IMethodCall::threeArgumentInIntReturn(const int& value1, const int& value2, const int& value3)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInValue<int > value1_value(value1, false);
    TInValue<int > value2_value(value2, false);
    TInValue<int > value3_value(value3, false);
    Event event("IMethodCall::threeArgumentInIntReturn", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int IMethodCall::fourArgumentInIntReturn(const int& value1, const int& value2, const int& value3, const int& value4)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInValue<int > value1_value(value1, false);
    TInValue<int > value2_value(value2, false);
    TInValue<int > value3_value(value3, false);
    TInValue<int > value4_value(value4, false);
    Event event("IMethodCall::fourArgumentInIntReturn", &value1_value, &value2_value, &value3_value, &value4_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int IMethodCall::fiveArgumentInIntReturn(const int& value1, const int& value2, const int& value3, const int& value4, const int& value5)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInValue<int > value1_value(value1, false);
    TInValue<int > value2_value(value2, false);
    TInValue<int > value3_value(value3, false);
    TInValue<int > value4_value(value4, false);
    TInValue<int > value5_value(value5, false);
    Event event("IMethodCall::fiveArgumentInIntReturn", &value1_value, &value2_value, &value3_value, &value4_value, &value5_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int IMethodCall::oneArgumentOutIntReturn(int& value1)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TOutValue<int > value1_value(value1);
    Event event("IMethodCall::oneArgumentOutIntReturn", &value1_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value1 = value1_value.getRValue();
    return return_value.getRValue();
  }
  
  int IMethodCall::twoArgumentOutIntReturn(int& value1, int& value2)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TOutValue<int > value1_value(value1);
    TOutValue<int > value2_value(value2);
    Event event("IMethodCall::twoArgumentOutIntReturn", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value1 = value1_value.getRValue();
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  int IMethodCall::threeArgumentOutIntReturn(int& value1, int& value2, int& value3)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TOutValue<int > value1_value(value1);
    TOutValue<int > value2_value(value2);
    TOutValue<int > value3_value(value3);
    Event event("IMethodCall::threeArgumentOutIntReturn", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value1 = value1_value.getRValue();
    value2 = value2_value.getRValue();
    value3 = value3_value.getRValue();
    return return_value.getRValue();
  }
  
  int IMethodCall::fourArgumentOutIntReturn(int& value1, int& value2, int& value3, int& value4)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TOutValue<int > value1_value(value1);
    TOutValue<int > value2_value(value2);
    TOutValue<int > value3_value(value3);
    TOutValue<int > value4_value(value4);
    Event event("IMethodCall::fourArgumentOutIntReturn", &value1_value, &value2_value, &value3_value, &value4_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value1 = value1_value.getRValue();
    value2 = value2_value.getRValue();
    value3 = value3_value.getRValue();
    value4 = value4_value.getRValue();
    return return_value.getRValue();
  }
  
  int IMethodCall::fiveArgumentOutIntReturn(int& value1, int& value2, int& value3, int& value4, int& value5)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TOutValue<int > value1_value(value1);
    TOutValue<int > value2_value(value2);
    TOutValue<int > value3_value(value3);
    TOutValue<int > value4_value(value4);
    TOutValue<int > value5_value(value5);
    Event event("IMethodCall::fiveArgumentOutIntReturn", &value1_value, &value2_value, &value3_value, &value4_value, &value5_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value1 = value1_value.getRValue();
    value2 = value2_value.getRValue();
    value3 = value3_value.getRValue();
    value4 = value4_value.getRValue();
    value5 = value5_value.getRValue();
    return return_value.getRValue();
  }
  
  int IMethodCall::oneArgumentInOutIntReturn(int& value1)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInOutValue<int > value1_value;
    value1_value.setValue(value1);
    Event event("IMethodCall::oneArgumentInOutIntReturn", &value1_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value1 = value1_value.getRValue();
    return return_value.getRValue();
  }
  
  int IMethodCall::twoArgumentInOutIntReturn(int& value1, int& value2)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInOutValue<int > value1_value;
    value1_value.setValue(value1);
    TInOutValue<int > value2_value;
    value2_value.setValue(value2);
    Event event("IMethodCall::twoArgumentInOutIntReturn", &value1_value, &value2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value1 = value1_value.getRValue();
    value2 = value2_value.getRValue();
    return return_value.getRValue();
  }
  
  int IMethodCall::threeArgumentInOutIntReturn(int& value1, int& value2, int& value3)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInOutValue<int > value1_value;
    value1_value.setValue(value1);
    TInOutValue<int > value2_value;
    value2_value.setValue(value2);
    TInOutValue<int > value3_value;
    value3_value.setValue(value3);
    Event event("IMethodCall::threeArgumentInOutIntReturn", &value1_value, &value2_value, &value3_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value1 = value1_value.getRValue();
    value2 = value2_value.getRValue();
    value3 = value3_value.getRValue();
    return return_value.getRValue();
  }
  
  int IMethodCall::fourArgumentInOutIntReturn(int& value1, int& value2, int& value3, int& value4)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInOutValue<int > value1_value;
    value1_value.setValue(value1);
    TInOutValue<int > value2_value;
    value2_value.setValue(value2);
    TInOutValue<int > value3_value;
    value3_value.setValue(value3);
    TInOutValue<int > value4_value;
    value4_value.setValue(value4);
    Event event("IMethodCall::fourArgumentInOutIntReturn", &value1_value, &value2_value, &value3_value, &value4_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value1 = value1_value.getRValue();
    value2 = value2_value.getRValue();
    value3 = value3_value.getRValue();
    value4 = value4_value.getRValue();
    return return_value.getRValue();
  }
  
  int IMethodCall::fiveArgumentInOutIntReturn(int& value1, int& value2, int& value3, int& value4, int& value5)
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInOutValue<int > value1_value;
    value1_value.setValue(value1);
    TInOutValue<int > value2_value;
    value2_value.setValue(value2);
    TInOutValue<int > value3_value;
    value3_value.setValue(value3);
    TInOutValue<int > value4_value;
    value4_value.setValue(value4);
    TInOutValue<int > value5_value;
    value5_value.setValue(value5);
    Event event("IMethodCall::fiveArgumentInOutIntReturn", &value1_value, &value2_value, &value3_value, &value4_value, &value5_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value1 = value1_value.getRValue();
    value2 = value2_value.getRValue();
    value3 = value3_value.getRValue();
    value4 = value4_value.getRValue();
    value5 = value5_value.getRValue();
    return return_value.getRValue();
  }
  
  void IMethodCall::stringInArgument(const string& value1)
      throw(const Exception&)
  {
    TInValue<string > value1_value(value1, false);
    Event event("IMethodCall::stringInArgument", &value1_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IMethodCall::stringOutArgument(string& value1)
      throw(const Exception&)
  {
    TOutValue<string > value1_value(value1);
    Event event("IMethodCall::stringOutArgument", &value1_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value1 = value1_value.getRValue();
  }
  
  void IMethodCall::stringInOutArgument(string& value1)
      throw(const Exception&)
  {
    TOutValue<string > value1_value(value1);
    Event event("IMethodCall::stringInOutArgument", &value1_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    value1 = value1_value.getRValue();
  }
  
  int IMethodCall::getMethodCallCounter()
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("IMethodCall::getMethodCallCounter");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IMethodCall::resetMethodCallCounter()
      throw(const Exception&)
  {
    Event event("IMethodCall::resetMethodCallCounter");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IMethodCall::IMethodCall_type = "IMethodCall";
  TIFactoryObject<IMethodCall > __IMethodCall_prototype("IMethodCall");


  const char* IMethodCall::noArgumentNoReturn_event = "IMethodCall::noArgumentNoReturn";
  const char* IMethodCall::noArgumentIntReturn_event = "IMethodCall::noArgumentIntReturn";
  const char* IMethodCall::oneArgumentInIntReturn_event = "IMethodCall::oneArgumentInIntReturn";
  const char* IMethodCall::twoArgumentInIntReturn_event = "IMethodCall::twoArgumentInIntReturn";
  const char* IMethodCall::threeArgumentInIntReturn_event = "IMethodCall::threeArgumentInIntReturn";
  const char* IMethodCall::fourArgumentInIntReturn_event = "IMethodCall::fourArgumentInIntReturn";
  const char* IMethodCall::fiveArgumentInIntReturn_event = "IMethodCall::fiveArgumentInIntReturn";
  const char* IMethodCall::oneArgumentOutIntReturn_event = "IMethodCall::oneArgumentOutIntReturn";
  const char* IMethodCall::twoArgumentOutIntReturn_event = "IMethodCall::twoArgumentOutIntReturn";
  const char* IMethodCall::threeArgumentOutIntReturn_event = "IMethodCall::threeArgumentOutIntReturn";
  const char* IMethodCall::fourArgumentOutIntReturn_event = "IMethodCall::fourArgumentOutIntReturn";
  const char* IMethodCall::fiveArgumentOutIntReturn_event = "IMethodCall::fiveArgumentOutIntReturn";
  const char* IMethodCall::oneArgumentInOutIntReturn_event = "IMethodCall::oneArgumentInOutIntReturn";
  const char* IMethodCall::twoArgumentInOutIntReturn_event = "IMethodCall::twoArgumentInOutIntReturn";
  const char* IMethodCall::threeArgumentInOutIntReturn_event = "IMethodCall::threeArgumentInOutIntReturn";
  const char* IMethodCall::fourArgumentInOutIntReturn_event = "IMethodCall::fourArgumentInOutIntReturn";
  const char* IMethodCall::fiveArgumentInOutIntReturn_event = "IMethodCall::fiveArgumentInOutIntReturn";
  const char* IMethodCall::stringInArgument_event = "IMethodCall::stringInArgument";
  const char* IMethodCall::stringOutArgument_event = "IMethodCall::stringOutArgument";
  const char* IMethodCall::stringInOutArgument_event = "IMethodCall::stringInOutArgument";
  const char* IMethodCall::getMethodCallCounter_event = "IMethodCall::getMethodCallCounter";
  const char* IMethodCall::resetMethodCallCounter_event = "IMethodCall::resetMethodCallCounter";


}


