
#ifndef NMM_IMethodCall_HPP
#define NMM_IMethodCall_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   * Interface with different method calls for performance tests
   */

  class IMethodCall : virtual public Interface { 
  public:
    IMethodCall() {  setTypeInfo(TypeInfo(IMethodCall::IMethodCall_type, "NMM")); };

    IMethodCall(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IMethodCall::IMethodCall_type, "NMM")); setEventReceiver(parent); };

    virtual ~IMethodCall() {};

    static const char* IMethodCall_type;

    virtual  string getInterfaceType() const { return IMethodCall_type; }

    Interface* copy() const { return new IMethodCall(*this); }
    static const char* noArgumentNoReturn_event;
    static const char* noArgumentIntReturn_event;
    static const char* oneArgumentInIntReturn_event;
    static const char* twoArgumentInIntReturn_event;
    static const char* threeArgumentInIntReturn_event;
    static const char* fourArgumentInIntReturn_event;
    static const char* fiveArgumentInIntReturn_event;
    static const char* oneArgumentOutIntReturn_event;
    static const char* twoArgumentOutIntReturn_event;
    static const char* threeArgumentOutIntReturn_event;
    static const char* fourArgumentOutIntReturn_event;
    static const char* fiveArgumentOutIntReturn_event;
    static const char* oneArgumentInOutIntReturn_event;
    static const char* twoArgumentInOutIntReturn_event;
    static const char* threeArgumentInOutIntReturn_event;
    static const char* fourArgumentInOutIntReturn_event;
    static const char* fiveArgumentInOutIntReturn_event;
    static const char* stringInArgument_event;
    static const char* stringOutArgument_event;
    static const char* stringInOutArgument_event;
    static const char* getMethodCallCounter_event;
    static const char* resetMethodCallCounter_event;

    virtual void noArgumentNoReturn()
      throw(const Exception&);

    virtual int noArgumentIntReturn()
      throw(const Exception&);

    virtual int oneArgumentInIntReturn(const int& value1)
      throw(const Exception&);

    virtual int twoArgumentInIntReturn(const int& value1, const int& value2)
      throw(const Exception&);

    virtual int threeArgumentInIntReturn(const int& value1, const int& value2, const int& value3)
      throw(const Exception&);

    virtual int fourArgumentInIntReturn(const int& value1, const int& value2, const int& value3, const int& value4)
      throw(const Exception&);

    virtual int fiveArgumentInIntReturn(const int& value1, const int& value2, const int& value3, const int& value4, const int& value5)
      throw(const Exception&);

    virtual int oneArgumentOutIntReturn(int& value1)
      throw(const Exception&);

    virtual int twoArgumentOutIntReturn(int& value1, int& value2)
      throw(const Exception&);

    virtual int threeArgumentOutIntReturn(int& value1, int& value2, int& value3)
      throw(const Exception&);

    virtual int fourArgumentOutIntReturn(int& value1, int& value2, int& value3, int& value4)
      throw(const Exception&);

    virtual int fiveArgumentOutIntReturn(int& value1, int& value2, int& value3, int& value4, int& value5)
      throw(const Exception&);

    virtual int oneArgumentInOutIntReturn(int& value1)
      throw(const Exception&);

    virtual int twoArgumentInOutIntReturn(int& value1, int& value2)
      throw(const Exception&);

    virtual int threeArgumentInOutIntReturn(int& value1, int& value2, int& value3)
      throw(const Exception&);

    virtual int fourArgumentInOutIntReturn(int& value1, int& value2, int& value3, int& value4)
      throw(const Exception&);

    virtual int fiveArgumentInOutIntReturn(int& value1, int& value2, int& value3, int& value4, int& value5)
      throw(const Exception&);

    virtual void stringInArgument(const string& value1)
      throw(const Exception&);

    virtual void stringOutArgument(string& value1)
      throw(const Exception&);

    virtual void stringInOutArgument(string& value1)
      throw(const Exception&);

    virtual int getMethodCallCounter()
      throw(const Exception&);

    virtual void resetMethodCallCounter()
      throw(const Exception&);


  };

  typedef auto_ptr<IMethodCall> IMethodCall_var;




}

#endif
