
#ifndef NMM_ISVGEncodeImpl_HPP
#define NMM_ISVGEncodeImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ISVGEncode.hpp"


namespace NMM
{

  class ISVGEncodeImpl  : public virtual NMMObject { 
  public:
    ISVGEncodeImpl() { addInterface(new ISVGEncode(*this)); registerEvents(); };

    virtual ~ISVGEncodeImpl() {};

  protected:
      /**
       * Sets the painting color
       * @param red red-value in RGB-Color
       * @param green green-value in RGB-Color
       * @param blue blue-value in RGB-Color
       */

    virtual void setColor(const int& red, const int& green, const int& blue) = 0;
      /**
       * Sets the dimension for the SVG-graphic
       * @param width the width of the graphic
       * @param height the height of the graphic
       */

    virtual void setDimension(const int& width, const int& height) = 0;
      /**
       * Returns the accuracy for the SVGFreehand object. Every nth pixel is taken for the bezier-curve.
       * @return the accuracy of a SVGFreehand object.
       */

    virtual int getPixelTaken() = 0;
      /**
       * Sets the accuracy for the SVGFreehand object. Every nth pixel is taken for the bezier-curve.
       * @param pixel_taken the accuracy of a SVGFreehand object.
       */

    virtual void setPixelTaken(const int& pixel_taken) = 0;
      /**
       * Sets a flag wether a SVG file should be written or not
       * @param withFile a flag wether a SVG file should be written or not
       */

    virtual void setWithFile(const bool& withFile) = 0;
      /**
       * Sets the stroke width of the SVG objects.
       * @param stroke_width the stroke width of the SVG objects
       */

    virtual void setStrokeWidth(const int& stroke_width) = 0;
      /**
       * Adds the point (x|y) to a SVGLine. If mouse button just pressed a new object will be created. If mouse button is released the object is finished.
       * @param x x-coordinate of the point
       * @param y y-coordinate of the point
       * @param buttonmask the buttonmask of the mousebuttons
       */

    virtual void drawSVGLine(const int& x, const int& y, const int& buttonmask) = 0;
      /**
       * Adds the point (x|y) to a SVGRectangle. If mouse button just pressed a new object will be created. If mouse button is released the object is finished.
       * @param x x-coordinate of the point
       * @param y y-coordinate of the point
       * @param buttonmask the buttonmask of the mousebuttons
       */ 

    virtual void drawSVGRectangle(const int& x, const int& y, const int& buttonmask) = 0;
      /**
       * Adds the point (x|y) to a SVGFreehand. If mouse button just pressed a new object will be created. If mouse is moved the point will be added to the object. If mouse button is released the object is finished.
       * @param x x-coordinate of the point
       * @param y y-coordinate of the point
       * @param buttonmask the buttonmask of the mousebuttons
       */

    virtual void drawSVGFreehand(const int& x, const int& y, const int& buttonmask) = 0;
      /**
       * Adds the point (x|y) to a SVGPointer. If mouse button just pressed a new object will be created. If mouse is moved the point will be added to the object. If mouse button is released the object is finished.
       * @param x x-coordinate of the point
       * @param y y-coordinate of the point
       * @param buttonmask the buttonmask of the mousebuttons
       */

    virtual void drawSVGPointer(const int& x, const int& y, const int& buttonmask) = 0;
      /**
       * Creates a new annotation and write the data to the SVG file wether the flag writeFile is set and there are objects in history. The objects in the history will then be deleted.
       */

    virtual void newAnnotation() = 0;
      /**
       * Empty the actual history and have a new empty annotation.
       */

    virtual void clearAnnotation() = 0;
      /**
       * Deletes the last created object in history
       */

    virtual void undo() = 0;
      /**
       * Restores the last deleted object in history
       */

    virtual void redo() = 0;
      /**
       * Repaints the current picture
       */

    virtual void repaint() = 0;


  private:
    void registerEvents();
  };

}

#endif
