
#ifndef NMM_IStillFrame_HPP
#define NMM_IStillFrame_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   * Interface for still frame handling.
   * This interface provides instream events for indicating buffers that contain still frames.
   * A node sends two events to indicate that the last video frame contained in the buffers
   * between these two events is a stillframe.
   *
   * This looks a bit complicated but was necessary because the DVD reader indicates still frames
   * only after they have been read, and the output of the DVD reader is muxed data which contains
   * not only video data.
   * If a node can more easily detect a stillframe, it simply sends the two events before and
   * after sending the buffer that contains the still frame.
   */

  class IStillFrame : virtual public Interface { 
  public:
    IStillFrame() {  setTypeInfo(TypeInfo(IStillFrame::IStillFrame_type, "NMM")); };

    IStillFrame(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IStillFrame::IStillFrame_type, "NMM")); setEventReceiver(parent); };

    virtual ~IStillFrame() {};

    static const char* IStillFrame_type;

    virtual  string getInterfaceType() const { return IStillFrame_type; }

    Interface* copy() const { return new IStillFrame(*this); }
    static const char* setStillFrame_event;
    static Event* create_setStillFrame(const bool& value);
  /* instream (protected) and non-instream (public) */        /**
     * Set still frame flag.
     * Instream event to set the still frame flag.
     * The still frame flag must be set to @a true before and to @a false after the data
     * which contains the still frame.
     * Nodes which respond to these events must internally keep track of the still frame flag.
     * @param value of the still frame flag
     */

    virtual Result setStillFrame(const bool& value) /* in-stream */ 
      throw(const Exception&);


  };

  typedef auto_ptr<IStillFrame> IStillFrame_var;




}

#endif
