
#ifndef NMM_IDisplayEventSender_HPP
#define NMM_IDisplayEventSender_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/interfaces/base/INode.hpp"

namespace NMM
{
  /**
   * Display event sender.
   * A node which can send methods from IDisplayEvents. This interface is for checking
   * whether a node can send such events and for enabling/disabling them.
   *
   * The node can send display events exactly if it supports this interface.
   * Display events are disabled by default.
   */

  class IDisplayEventSender : virtual public Interface { 
  public:
    IDisplayEventSender() {  setTypeInfo(TypeInfo(IDisplayEventSender::IDisplayEventSender_type, "NMM")); };

    IDisplayEventSender(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IDisplayEventSender::IDisplayEventSender_type, "NMM")); setEventReceiver(parent); };

    virtual ~IDisplayEventSender() {};

    static const char* IDisplayEventSender_type;

    virtual  string getInterfaceType() const { return IDisplayEventSender_type; }

    Interface* copy() const { return new IDisplayEventSender(*this); }
    static const char* enableDisplayEvents_event;
    static const char* disableDisplayEvents_event;
    /**
     * Enable sending display events.
     * Display events are disabled by default.
     * @param config_file filename of configuration file that maps keys to aliases.
     */

    virtual void enableDisplayEvents(const string& config_file = "") /* limited(INode::STARTED) */
      throw(const Exception&, const IllegalStateException&);
    /**
     * Disable sending display events.
     * Display events are disabled by default.
     */

    virtual void disableDisplayEvents()
      throw(const Exception&);


  };

  typedef auto_ptr<IDisplayEventSender> IDisplayEventSender_var;




}

#endif
