
#ifndef NMM_IDisplayEventSenderImpl_HPP
#define NMM_IDisplayEventSenderImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IDisplayEventSender.hpp"

#include "nmm/interfaces/base/INodeImpl.hpp"

namespace NMM
{
  /**
   * Display event sender.
   * A node which can send methods from IDisplayEvents. This interface is for checking
   * whether a node can send such events and for enabling/disabling them.
   *
   * The node can send display events exactly if it supports this interface.
   * Display events are disabled by default.
   */

  class IDisplayEventSenderImpl  : public virtual NMMObject { 
  public:
    IDisplayEventSenderImpl() { addInterface(new IDisplayEventSender(*this)); registerEvents(); };

    virtual ~IDisplayEventSenderImpl() {};

  protected:
    /**
     * Enable sending display events.
     * Display events are disabled by default.
     * @param config_file filename of configuration file that maps keys to aliases.
     */

    virtual void enableDisplayEvents(const string& config_file = "") /* limited(INode::STARTED) */ = 0;
    /**
     * Disable sending display events.
     * Display events are disabled by default.
     */

    virtual void disableDisplayEvents() = 0;


  private:
    void registerEvents();
  };

}

#endif
