
#ifndef NMM_IVideoDisplayImpl_HPP
#define NMM_IVideoDisplayImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IVideoDisplay.hpp"

#include "nmm/interfaces/base/INodeImpl.hpp"

namespace NMM
{
  /**
   * General display interface.
   * This interface provides all general display methods.
   */

  class IVideoDisplayImpl  : public virtual NMMObject { 
  public:
    IVideoDisplayImpl() { addInterface(new IVideoDisplay(*this)); registerEvents(); };

    virtual ~IVideoDisplayImpl() {};

  protected:
    /**
     * Set fullscreen mode.
     * @param value true for fullscreen, false for windowed display mode
     */

    virtual void setFullscreen(const bool& value) = 0;
    /**
     * Get fullscreen mode.
     * @return true if the display is in fullscreen mode, false if it is in windowed mode
     */

    virtual bool getFullscreen() const = 0;
    /**
     * test if display has fullscreen support.
     * @return true if fullscreen mode can be turned on, false otherwise
     */

    virtual bool hasFullscreen() const = 0;
    /**
     * Test if display is fullscreen only.
     * @return true if fullscren mode is the default and cannot be turned off.
     */

    virtual bool isFullscreenOnly() const = 0;

    virtual bool makeScreenshot(const string& filename) const = 0;
    /**
     * Set window alignment.
     */

    virtual void setLeftWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */ = 0;

    virtual void setRightWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */ = 0;

    virtual void setCenterWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */ = 0;


  private:
    void registerEvents();
  };

}

#endif
