
#ifndef NMM_IOSDOverlay_HPP
#define NMM_IOSDOverlay_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "IOSDSetFont.hpp"

namespace NMM
{

  class IOSDOverlay : virtual public IOSDSetFont { 
  public:
    IOSDOverlay() {  setTypeInfo(TypeInfo(IOSDOverlay::IOSDOverlay_type, "NMM")); };

    IOSDOverlay(NMMObject& parent) : IOSDSetFont(parent) {  setTypeInfo(TypeInfo(IOSDOverlay::IOSDOverlay_type, "NMM")); setEventReceiver(parent); };

    virtual ~IOSDOverlay() {};

    static const char* IOSDOverlay_type;

    virtual  string getInterfaceType() const { return IOSDOverlay_type; }

    Interface* copy() const { return new IOSDOverlay(*this); }
    static const char* addText_event;
    static const char* printText_event;
    static const char* removeText_event;
    static const char* addProgessBar_event;
    static const char* addImage_event;
    static const char* setItemOffset_event;
    static const char* setItemText_event;
    static const char* setItemEnabled_event;
    static const char* disableItems_event;
    static const char* doPause_event;
    static const char* update_event;
    /**
     * Add a text item.
     * @param id key to access the text item
     * @param text text to be displayed
     * @param x_offset X position where the text should be displayed
     * @param y_offset Y position where the text should be displayed
     */

    virtual Result addText(const string& id, const string& text, const size_t& xoffset, const size_t& yoffset)
      throw(const Exception&);
    /**
     * Add a text item or add text to an existing item.
     * This method behaves like addText, but it adds a new text (with addText) if the
     * given id is not found
     * @param id key to access the text item
     * @param text text to be displayed
     * @param x_offset X position where the text should be displayed
     * @param y_offset Y position where the text should be displayed
     * @param seconds determines when the whole osd will be cleared (0 = never)
     */

    virtual Result printText(const string& id, const string& text, const size_t& xoffset, const size_t& yoffset, const int& seconds)
      throw(const Exception&);
    /**
     * Remove a text item
     * @param id key to access the text item
     */

    virtual Result removeText(const string& id)
      throw(const Exception&);
    /**
     * Add a progess bar.
     * @param id key to access the text item
     * @parm value value to be displayed
     * @param x_offset determine the x-position where the text should be displayed
     * @param y_offset determine the y-position where the text should be displayed
     */

    virtual Result addProgessBar(const string& id, const int& value, const int& x_offset, const int& y_offset)
      throw(const Exception&);
    /**
     * Add an image item.
     * @param id key to access the image item
     * @param x_offset X position of the item
     * @param y_offset Y position of the item
     */

    virtual Result addImage(const string& id, const string& url, const int& x_offset, const int& y_offset)
      throw(const Exception&);
    /**
     * Changes the Posistion of an item.
     * @param id key to identify the item
     * @param x_offset new X position of the item
     * @param y_offset new Y position of the item
     */

    virtual Result setItemOffset(const string& id, const int& x_offset, const int& y_offset)
      throw(const Exception&);
    /**
     * Changes the text content of a text item.
     * @param id key to identify the text item
     * @param text new text of the text item
     */

    virtual Result setItemText(const string& id, const string& text)
      throw(const Exception&);
    /**
     * Enables or disables an item.
     * @param id key to identify the item
     * @param enable tells whether to display the item or not
     */

    virtual Result setItemEnabled(const string& id, const bool& enable)
      throw(const Exception&);
    /**
     * Disables all items, so no item is displayed any more
     */

    virtual Result disableItems()
      throw(const Exception&);
    /**
     * Set pause mode.
     * @param pause pause mode flag
     */

    virtual Result doPause(const bool& pause)
      throw(const Exception&);

    virtual Result update()
      throw(const Exception&);


  };

  typedef auto_ptr<IOSDOverlay> IOSDOverlay_var;




}

#endif
