
#ifndef NMM_IOSDOverlayImpl_HPP
#define NMM_IOSDOverlayImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IOSDOverlay.hpp"

#include "IOSDSetFontImpl.hpp"

namespace NMM
{

  class IOSDOverlayImpl  : virtual public IOSDSetFontImpl { 
  public:
    IOSDOverlayImpl() { addInterface(new IOSDOverlay(*this)); registerEvents(); };

    virtual ~IOSDOverlayImpl() {};

  protected:
    /**
     * Add a text item.
     * @param id key to access the text item
     * @param text text to be displayed
     * @param x_offset X position where the text should be displayed
     * @param y_offset Y position where the text should be displayed
     */

    virtual Result addText(const string& id, const string& text, const size_t& xoffset, const size_t& yoffset) = 0;
    /**
     * Add a text item or add text to an existing item.
     * This method behaves like addText, but it adds a new text (with addText) if the
     * given id is not found
     * @param id key to access the text item
     * @param text text to be displayed
     * @param x_offset X position where the text should be displayed
     * @param y_offset Y position where the text should be displayed
     * @param seconds determines when the whole osd will be cleared (0 = never)
     */

    virtual Result printText(const string& id, const string& text, const size_t& xoffset, const size_t& yoffset, const int& seconds) = 0;
    /**
     * Remove a text item
     * @param id key to access the text item
     */

    virtual Result removeText(const string& id) = 0;
    /**
     * Add a progess bar.
     * @param id key to access the text item
     * @parm value value to be displayed
     * @param x_offset determine the x-position where the text should be displayed
     * @param y_offset determine the y-position where the text should be displayed
     */

    virtual Result addProgessBar(const string& id, const int& value, const int& x_offset, const int& y_offset) = 0;
    /**
     * Add an image item.
     * @param id key to access the image item
     * @param x_offset X position of the item
     * @param y_offset Y position of the item
     */

    virtual Result addImage(const string& id, const string& url, const int& x_offset, const int& y_offset) = 0;
    /**
     * Changes the Posistion of an item.
     * @param id key to identify the item
     * @param x_offset new X position of the item
     * @param y_offset new Y position of the item
     */

    virtual Result setItemOffset(const string& id, const int& x_offset, const int& y_offset) = 0;
    /**
     * Changes the text content of a text item.
     * @param id key to identify the text item
     * @param text new text of the text item
     */

    virtual Result setItemText(const string& id, const string& text) = 0;
    /**
     * Enables or disables an item.
     * @param id key to identify the item
     * @param enable tells whether to display the item or not
     */

    virtual Result setItemEnabled(const string& id, const bool& enable) = 0;
    /**
     * Disables all items, so no item is displayed any more
     */

    virtual Result disableItems() = 0;
    /**
     * Set pause mode.
     * @param pause pause mode flag
     */

    virtual Result doPause(const bool& pause) = 0;

    virtual Result update() = 0;


  private:
    void registerEvents();
  };

}

#endif
