
#include "IBitmapReader.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  bool IBitmapReader::loadImage(const string& id, const string& filename, const IBitmapReader::Format& f /* = IBitmapReader::RGBA */)
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<string > id_value(id, false);
    TInValue<string > filename_value(filename, false);
    TInValue<IBitmapReader::Format > f_value(f, false);
    Event event("IBitmapReader::loadImage", &id_value, &filename_value, &f_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IBitmapReader::getImage(const string& id, unsigned int& w, unsigned int& h)
      throw(const Exception&)
  {
    TInValue<string > id_value(id, false);
    TOutValue<unsigned int > w_value(w);
    TOutValue<unsigned int > h_value(h);
    Event event("IBitmapReader::getImage", &id_value, &w_value, &h_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    w = w_value.getRValue();
    h = h_value.getRValue();
  }
  
  bool IBitmapReader::createImage(const string& id, const unsigned int& width, const unsigned int& height, const unsigned int& color1, const unsigned int& color2, const unsigned int& color3, const IBitmapReader::Format& f /* = IBitmapReader::RGBA */)
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<string > id_value(id, false);
    TInValue<unsigned int > width_value(width, false);
    TInValue<unsigned int > height_value(height, false);
    TInValue<unsigned int > color1_value(color1, false);
    TInValue<unsigned int > color2_value(color2, false);
    TInValue<unsigned int > color3_value(color3, false);
    TInValue<IBitmapReader::Format > f_value(f, false);
    Event event("IBitmapReader::createImage", &id_value, &width_value, &height_value, &color1_value, &color2_value, &color3_value, &f_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IBitmapReader::freeImage(const string& id)
      throw(const Exception&)
  {
    TInValue<string > id_value(id, false);
    Event event("IBitmapReader::freeImage", &id_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IBitmapReader::freeAll()
      throw(const Exception&)
  {
    Event event("IBitmapReader::freeAll");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IBitmapReader::IBitmapReader_type = "IBitmapReader";
  TIFactoryObject<IBitmapReader > __IBitmapReader_prototype("IBitmapReader");
  TValueConstructor< IBitmapReader::Format > __IBitmapReader_Format_prototype(TypeInfo("IBitmapReader::Format", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const IBitmapReader::Format& t)
  {
    ov.beginType(TypeInfo("IBitmapReader::Format", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, IBitmapReader::Format& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("IBitmapReader::Format", "NMM")) {
      throw InvalidTypeException(TypeInfo("IBitmapReader::Format", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }



  const char* IBitmapReader::loadImage_event = "IBitmapReader::loadImage";
  const char* IBitmapReader::getImage_event = "IBitmapReader::getImage";
  const char* IBitmapReader::createImage_event = "IBitmapReader::createImage";
  const char* IBitmapReader::freeImage_event = "IBitmapReader::freeImage";
  const char* IBitmapReader::freeAll_event = "IBitmapReader::freeAll";


}


