
#include "IWidgetFactory.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  IWidgetComposite* IWidgetFactory::createWidgetComposite()
      throw(const Exception&)
  {
    TOutValue<IWidgetComposite* > return_value;
    Event event("IWidgetFactory::createWidgetComposite");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IProgressBar* IWidgetFactory::createProgressBar()
      throw(const Exception&)
  {
    TOutValue<IProgressBar* > return_value;
    Event event("IWidgetFactory::createProgressBar");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITextView* IWidgetFactory::createTextView()
      throw(const Exception&)
  {
    TOutValue<ITextView* > return_value;
    Event event("IWidgetFactory::createTextView");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITimeView* IWidgetFactory::createTimeView()
      throw(const Exception&)
  {
    TOutValue<ITimeView* > return_value;
    Event event("IWidgetFactory::createTimeView");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IImageView* IWidgetFactory::createImageView()
      throw(const Exception&)
  {
    TOutValue<IImageView* > return_value;
    Event event("IWidgetFactory::createImageView");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IRadioButtons* IWidgetFactory::createRadioButtons()
      throw(const Exception&)
  {
    TOutValue<IRadioButtons* > return_value;
    Event event("IWidgetFactory::createRadioButtons");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IButtons* IWidgetFactory::createButtons()
      throw(const Exception&)
  {
    TOutValue<IButtons* > return_value;
    Event event("IWidgetFactory::createButtons");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IBorderDecorator* IWidgetFactory::createBorderDecorator(const IWidget& internal)
      throw(const Exception&)
  {
    TOutValue<IBorderDecorator* > return_value;
    TInValue<IWidget > internal_value(internal, false);
    Event event("IWidgetFactory::createBorderDecorator", &internal_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IHeaderDecorator* IWidgetFactory::createHeaderDecorator(const IWidget& internal)
      throw(const Exception&)
  {
    TOutValue<IHeaderDecorator* > return_value;
    TInValue<IWidget > internal_value(internal, false);
    Event event("IWidgetFactory::createHeaderDecorator", &internal_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IPlaneDecorator* IWidgetFactory::createPlaneDecorator(const IWidget& internal)
      throw(const Exception&)
  {
    TOutValue<IPlaneDecorator* > return_value;
    TInValue<IWidget > internal_value(internal, false);
    Event event("IWidgetFactory::createPlaneDecorator", &internal_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IScrollDecorator* IWidgetFactory::createScrollDecorator(const IWidget& internal)
      throw(const Exception&)
  {
    TOutValue<IScrollDecorator* > return_value;
    TInValue<IWidget > internal_value(internal, false);
    Event event("IWidgetFactory::createScrollDecorator", &internal_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IWidgetFactory::IWidgetFactory_type = "IWidgetFactory";
  TIFactoryObject<IWidgetFactory > __IWidgetFactory_prototype("IWidgetFactory");


  const char* IWidgetFactory::createWidgetComposite_event = "IWidgetFactory::createWidgetComposite";
  const char* IWidgetFactory::createProgressBar_event = "IWidgetFactory::createProgressBar";
  const char* IWidgetFactory::createTextView_event = "IWidgetFactory::createTextView";
  const char* IWidgetFactory::createTimeView_event = "IWidgetFactory::createTimeView";
  const char* IWidgetFactory::createImageView_event = "IWidgetFactory::createImageView";
  const char* IWidgetFactory::createRadioButtons_event = "IWidgetFactory::createRadioButtons";
  const char* IWidgetFactory::createButtons_event = "IWidgetFactory::createButtons";
  const char* IWidgetFactory::createBorderDecorator_event = "IWidgetFactory::createBorderDecorator";
  const char* IWidgetFactory::createHeaderDecorator_event = "IWidgetFactory::createHeaderDecorator";
  const char* IWidgetFactory::createPlaneDecorator_event = "IWidgetFactory::createPlaneDecorator";
  const char* IWidgetFactory::createScrollDecorator_event = "IWidgetFactory::createScrollDecorator";


}


