
#include "IChannelInfo.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result IChannelInfo::hasChannel(int& c1, const string& c2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TOutValue<int > c1_value(c1);
    TInValue<string > c2_value(c2, false);
    Event event("IChannelInfo::hasChannel", &c1_value, &c2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    c1 = c1_value.getRValue();
    return return_value.getRValue();
  }
  
  TVChannelList IChannelInfo::getChannelList()
      throw(const Exception&)
  {
    TOutValue<TVChannelList > return_value;
    Event event("IChannelInfo::getChannelList");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IChannelInfo::setChannelConfigFile(const string& filename) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TInValue<string > filename_value(filename, false);
    Event event("IChannelInfo::setChannelConfigFile", &filename_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }

  const char* IChannelInfo::IChannelInfo_type = "IChannelInfo";
  TIFactoryObject<IChannelInfo > __IChannelInfo_prototype("IChannelInfo");


  const char* IChannelInfo::hasChannel_event = "IChannelInfo::hasChannel";
  const char* IChannelInfo::getChannelList_event = "IChannelInfo::getChannelList";
  const char* IChannelInfo::setChannelConfigFile_event = "IChannelInfo::setChannelConfigFile";


}


