
#include "ITVCard.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void ITVCard::setResolution(const int& x, const int& y) /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    Event event("ITVCard::setResolution", &x_value, &y_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITVCard::getResolution(int& x, int& y)
      throw(const Exception&)
  {
    TOutValue<int > x_value(x);
    TOutValue<int > y_value(y);
    Event event("ITVCard::getResolution", &x_value, &y_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    x = x_value.getRValue();
    y = y_value.getRValue();
  }
  
  Result ITVCard::setChannel(const int& channel) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Result > return_value;
    TInValue<int > channel_value(channel, false);
    Event event("ITVCard::setChannel", &channel_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ITVCard::getChannel(int& c1, string& c2)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TOutValue<int > c1_value(c1);
    TOutValue<string > c2_value(c2);
    Event event("ITVCard::getChannel", &c1_value, &c2_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    c1 = c1_value.getRValue();
    c2 = c2_value.getRValue();
    return return_value.getRValue();
  }

  const char* ITVCard::ITVCard_type = "ITVCard";
  TIFactoryObject<ITVCard > __ITVCard_prototype("ITVCard");


  const char* ITVCard::setResolution_event = "ITVCard::setResolution";
  const char* ITVCard::getResolution_event = "ITVCard::getResolution";
  const char* ITVCard::setChannel_event = "ITVCard::setChannel";
  const char* ITVCard::getChannel_event = "ITVCard::getChannel";


}


