
#ifndef NMM_IVISCACamera_HPP
#define NMM_IVISCACamera_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class IVISCACamera : virtual public Interface { 
  public:
    IVISCACamera() {  setTypeInfo(TypeInfo(IVISCACamera::IVISCACamera_type, "NMM")); };

    IVISCACamera(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IVISCACamera::IVISCACamera_type, "NMM")); setEventReceiver(parent); };

    virtual ~IVISCACamera() {};

    static const char* IVISCACamera_type;

    virtual  string getInterfaceType() const { return IVISCACamera_type; }

    Interface* copy() const { return new IVISCACamera(*this); }
    static const char* IF_Clear_event;
    static const char* AddressSet_event;
    static const char* getNumberOfCameras_event;
    static const char* CommandCancel_event;
    static const char* PowerOn_event;
    static const char* PowerOff_event;
    static const char* AutoPowerOff_event;
    static const char* ZoomStop_event;
    static const char* ZoomTeleStd_event;
    static const char* ZoomWideStd_event;
    static const char* ZoomTeleVar_event;
    static const char* ZoomWideVar_event;
    static const char* ZoomDirect_event;
    static const char* ZoomDigitalZoomOnOff_event;
    static const char* FocusStop_event;
    static const char* FocusFarStd_event;
    static const char* FocusNearStd_event;
    static const char* FocusFarVar_event;
    static const char* FocusNearVar_event;
    static const char* FocusDirect_event;
    static const char* FocusAutoFocus_event;
    static const char* FocusManualFocus_event;
    static const char* FocusAutoManual_event;
    static const char* FocusOnePushTrigger_event;
    static const char* FocusInfinity_event;
    static const char* FocusAFSensHigh_event;
    static const char* FocusAFSensLow_event;
    static const char* FocusNearLimit_event;
    static const char* WhiteBalance_event;
    static const char* WhiteBalanceAuto_event;
    static const char* WhiteBalanceIndoor_event;
    static const char* WhiteBalanceOutdoor_event;
    static const char* WhiteBalanceOnePush_event;
    static const char* WhiteBalanceAT_event;
    static const char* WhiteBalanceManual_event;
    static const char* WhiteBalanceOnePushTrigger_event;
    static const char* RGainReset_event;
    static const char* RGainUp_event;
    static const char* RGainDown_event;
    static const char* RGainDirect_event;
    static const char* BGainReset_event;
    static const char* BGainUp_event;
    static const char* BGainDown_event;
    static const char* BGainDirect_event;
    static const char* AutomaticExposureFull_event;
    static const char* AutomaticExposureManual_event;
    static const char* AutomaticExposureShutterPriority_event;
    static const char* AutomaticExposureIrisPriority_event;
    static const char* AutomaticExposureGainPriority_event;
    static const char* AutomaticExposureBright_event;
    static const char* AutomaticExposureShutterAuto_event;
    static const char* AutomaticExposureIrisAuto_event;
    static const char* AutomaticExposureGainAuto_event;
    static const char* SlowShutter_event;
    static const char* ShutterReset_event;
    static const char* ShutterUp_event;
    static const char* ShutterDown_event;
    static const char* ShutterDirect_event;
    static const char* IrisReset_event;
    static const char* IrisUp_event;
    static const char* IrisDown_event;
    static const char* IrisDirect_event;
    static const char* GainReset_event;
    static const char* GainUp_event;
    static const char* GainDown_event;
    static const char* GainDirect_event;
    static const char* BrightReset_event;
    static const char* BrightUp_event;
    static const char* BrightDown_event;
    static const char* BrightDirect_event;
    static const char* ExposureCompensationOnOff_event;
    static const char* ExposureCompensationReset_event;
    static const char* ExposureCompensationUp_event;
    static const char* ExposureCompensationDown_event;
    static const char* ExposureCompensationDirect_event;
    static const char* BackLightOnOff_event;
    static const char* ApertureReset_event;
    static const char* ApertureUp_event;
    static const char* ApertureDown_event;
    static const char* ApertureDirect_event;
    static const char* WideModeOff_event;
    static const char* WideModeCinema_event;
    static const char* WideMode16to9Full_event;
    static const char* LR_Reverse_event;
    static const char* Freeze_event;
    static const char* PictureEffectOff_event;
    static const char* PictureEffectPastel_event;
    static const char* PictureEffectNegArt_event;
    static const char* PictureEffectSepia_event;
    static const char* PictureEffectBW_event;
    static const char* PictureEffectSolarize_event;
    static const char* PictureEffectMosaic_event;
    static const char* PictureEffectSlim_event;
    static const char* PictureEffectStretch_event;
    static const char* DigitalEffectOff_event;
    static const char* DigitalEffectStill_event;
    static const char* DigitalEffectFlash_event;
    static const char* DigitalEffectLumi_event;
    static const char* DigitalEffectTrail_event;
    static const char* DigitalEffectLevel_event;
    static const char* Memory_event;
    static const char* MemoryReset_event;
    static const char* MemorySet_event;
    static const char* MemoryRecall_event;
    static const char* DataScreenOnOff_event;
    static const char* IRReceiveOnOff_event;
    static const char* IRReceiveReturn_event;
    static const char* PanTiltDriveMotion_event;
    static const char* PanTiltDriveUp_event;
    static const char* PanTiltDriveDown_event;
    static const char* PanTiltDriveLeft_event;
    static const char* PanTiltDriveRight_event;
    static const char* PanTiltDriveUpLeft_event;
    static const char* PanTiltDriveUpRight_event;
    static const char* PanTiltDriveDownLeft_event;
    static const char* PanTiltDriveDownRight_event;
    static const char* PanTiltDriveStop_event;
    static const char* PanTiltDrivePosition_event;
    static const char* PanTiltDriveAbsolute_event;
    static const char* PanTiltDriveRelative_event;
    static const char* PanTiltDriveAbsoluteDeg_event;
    static const char* PanTiltDriveRelativeDeg_event;
    static const char* PanTiltDriveHome_event;
    static const char* PanTiltDriveReset_event;
    static const char* PanTiltLimitSet_event;
    static const char* PanTiltLimitClear_event;
    static const char* PowerInq_event;
    static const char* AutoPowerOffInq_event;
    static const char* DZoomModeInq_event;
    static const char* ZoomPosInq_event;
    static const char* FocusModeInq_event;
    static const char* FocusPosInq_event;
    static const char* AFModeInq_event;
    static const char* FocusNearLimitInq_event;
    static const char* WhiteBalanceModeInq_event;
    static const char* RGainInq_event;
    static const char* BGainInq_event;
    static const char* AutoExposureModeInq_event;
    static const char* SlowShutterModeInq_event;
    static const char* ShutterPosInq_event;
    static const char* IrisPosInq_event;
    static const char* GainPosInq_event;
    static const char* BrightPosInq_event;
    static const char* ExposureCompensationModeInq_event;
    static const char* ExposureCompensationPosInq_event;
    static const char* BackLightModeInq_event;
    static const char* ApertureInq_event;
    static const char* WideModeInq_event;
    static const char* LR_ReverseModeInq_event;
    static const char* FreezeModeInq_event;
    static const char* PictureEffectModeInq_event;
    static const char* DigitalEffectModeInq_event;
    static const char* DigitalEffectLevelInq_event;
    static const char* MemoryInq_event;
    static const char* DatascreenInq_event;
    static const char* PanTiltModeInq_event;
    static const char* PanTiltMaxSpeedInq_event;
    static const char* PanTiltPosInq_event;
    static const char* VideoSystemInq_event;
    static const char* DeviceTypeInq_event;
    static const char* verbose_event;
    static const char* islocked_event;
    /**
     * IF_Clear 
     * 
     * Clears the command buffers in the camera and 
     * cancels the command currently beeing executed.
     *
     * this function will be called automaticly by the init
     * function 
     *
     * input  : the number of the camera [1;7,8]
     *          camera_id 8 means broadcast, all 
     *          cameras will clear their command
     *          buffers
     * output : SUCCESS when success, else FAILURE
     */

    virtual Result IF_Clear(const unsigned char& value)
      throw(const Exception&);
    /**
     * AddressSet
     *
     * Scan all cameras connected to the VISCA-bus 
     * and dispatch one VISCA-address for each camera found.
     *
     * This function will be called automaticly by the 
     * init function
     *
     * input  : none
     * output : SUCCESS when success, else FAILURE
     */

    virtual Result AddressSet()
      throw(const Exception&);
    /**
     * getNumberOfCameras
     *
     * Returns the number of cameras connected to the VISCA-bus
     *
     * input  : none
     * output : number of cameras found
     */

    virtual Result getNumberOfCameras(unsigned char& value)
      throw(const Exception&);
    /**
     * CommandCancel
     * 
     * Cancel the command currently executed by the camera
     * with the address <camera identificator> and the socket
     * number <socket id>.
     *
     * input  : camera identificator and socket id
     * output : SUCCESS, if the command is canceled
     *          FAILURE, if not
     */

    virtual Result CommandCancel(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&);
    /**
     * Power functions
     *
     * The following functions controlls the powerstate 
     * of the cameras. Each camera can be controlled seperatly,
     * selectecd by the <camera id>. The AutoPowerOff-function sets 
     * the timer after the camera goes in standby-mode, when no 
     * VISCA-cammand arrived 
     *
     * input  : camera id
     * output : SUCCESS if success, else FAILURE
     */

    virtual Result PowerOn(const unsigned char& value)
      throw(const Exception&);

    virtual Result PowerOff(const unsigned char& value)
      throw(const Exception&);

    virtual Result AutoPowerOff(const unsigned char& value1, const int& value2)
      throw(const Exception&);
    /**
     * Zoom functions
     *
     * The following functions controls the zoom of the camera. 
     * All that functions expect the <camera_id> as parameter.
     * Some of them allow additional speed settings 
     *
     * - The Range for the Zoom:
     *   optical zoom : [    0;16384]
     *   digital zoom : [16384;28672]
     * - ZoomDZoom(On|Off) turn the DigitalZoom On or Off.
     */

    virtual Result ZoomStop(const unsigned char& value)
      throw(const Exception&);

    virtual Result ZoomTeleStd(const unsigned char& value)
      throw(const Exception&);

    virtual Result ZoomWideStd(const unsigned char& value)
      throw(const Exception&);

    virtual Result ZoomTeleVar(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&);

    virtual Result ZoomWideVar(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&);

    virtual Result ZoomDirect(const unsigned char& value1, const int& value2)
      throw(const Exception&);

    virtual Result ZoomDigitalZoomOnOff(const unsigned char& value1, const bool& value2)
      throw(const Exception&);
    /**
     * Focus functions
     * 
     * the Focus* functions are controling the settings of te
     * camera's focus. All functions expect the <camera_id> 
     * as first parameter, some an additional parameter
     */

    virtual Result FocusStop(const unsigned char& value)
      throw(const Exception&);

    virtual Result FocusFarStd(const unsigned char& value)
      throw(const Exception&);

    virtual Result FocusNearStd(const unsigned char& value)
      throw(const Exception&);

    virtual Result FocusFarVar(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&);

    virtual Result FocusNearVar(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&);

    virtual Result FocusDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&);

    virtual Result FocusAutoFocus(const unsigned char& value)
      throw(const Exception&);

    virtual Result FocusManualFocus(const unsigned char& value)
      throw(const Exception&);

    virtual Result FocusAutoManual(const unsigned char& value)
      throw(const Exception&);

    virtual Result FocusOnePushTrigger(const unsigned char& value)
      throw(const Exception&);

    virtual Result FocusInfinity(const unsigned char& value)
      throw(const Exception&);

    virtual Result FocusAFSensHigh(const unsigned char& value)
      throw(const Exception&);

    virtual Result FocusAFSensLow(const unsigned char& value)
      throw(const Exception&);

    virtual Result FocusNearLimit(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&);
    /**
     * WhiteBalance
     */

    virtual Result WhiteBalance(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&);

    virtual Result WhiteBalanceAuto(const unsigned char& value)
      throw(const Exception&);

    virtual Result WhiteBalanceIndoor(const unsigned char& value)
      throw(const Exception&);

    virtual Result WhiteBalanceOutdoor(const unsigned char& value)
      throw(const Exception&);

    virtual Result WhiteBalanceOnePush(const unsigned char& value)
      throw(const Exception&);

    virtual Result WhiteBalanceAT(const unsigned char& value)
      throw(const Exception&);

    virtual Result WhiteBalanceManual(const unsigned char& value)
      throw(const Exception&);

    virtual Result WhiteBalanceOnePushTrigger(const unsigned char& value)
      throw(const Exception&);
    /**
     * RGain
     */

    virtual Result RGainReset(const unsigned char& value)
      throw(const Exception&);

    virtual Result RGainUp(const unsigned char& value)
      throw(const Exception&);

    virtual Result RGainDown(const unsigned char& value)
      throw(const Exception&);

    virtual Result RGainDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&);
    /**
     * BGain
     */

    virtual Result BGainReset(const unsigned char& value)
      throw(const Exception&);

    virtual Result BGainUp(const unsigned char& value)
      throw(const Exception&);

    virtual Result BGainDown(const unsigned char& value)
      throw(const Exception&);

    virtual Result BGainDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&);
    /**
     * Automatic Exposure 
     */

    virtual Result AutomaticExposureFull(const unsigned char& value)
      throw(const Exception&);

    virtual Result AutomaticExposureManual(const unsigned char& value)
      throw(const Exception&);

    virtual Result AutomaticExposureShutterPriority(const unsigned char& value)
      throw(const Exception&);

    virtual Result AutomaticExposureIrisPriority(const unsigned char& value)
      throw(const Exception&);

    virtual Result AutomaticExposureGainPriority(const unsigned char& value)
      throw(const Exception&);

    virtual Result AutomaticExposureBright(const unsigned char& value)
      throw(const Exception&);

    virtual Result AutomaticExposureShutterAuto(const unsigned char& value)
      throw(const Exception&);

    virtual Result AutomaticExposureIrisAuto(const unsigned char& value)
      throw(const Exception&);

    virtual Result AutomaticExposureGainAuto(const unsigned char& value)
      throw(const Exception&);
    /**
     * Slow Shutter
     */

    virtual Result SlowShutter(const unsigned char& value1, const bool& value2)
      throw(const Exception&);
    /**
     * Shutter
     */

    virtual Result ShutterReset(const unsigned char& value)
      throw(const Exception&);

    virtual Result ShutterUp(const unsigned char& value)
      throw(const Exception&);

    virtual Result ShutterDown(const unsigned char& value)
      throw(const Exception&);

    virtual Result ShutterDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&);
    /**
     * Iris
     */

    virtual Result IrisReset(const unsigned char& value)
      throw(const Exception&);

    virtual Result IrisUp(const unsigned char& value)
      throw(const Exception&);

    virtual Result IrisDown(const unsigned char& value)
      throw(const Exception&);

    virtual Result IrisDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&);
    /**
     * Gain
     */

    virtual Result GainReset(const unsigned char& value)
      throw(const Exception&);

    virtual Result GainUp(const unsigned char& value)
      throw(const Exception&);

    virtual Result GainDown(const unsigned char& value)
      throw(const Exception&);

    virtual Result GainDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&);
    /**
     * Bright
     */

    virtual Result BrightReset(const unsigned char& value)
      throw(const Exception&);

    virtual Result BrightUp(const unsigned char& value)
      throw(const Exception&);

    virtual Result BrightDown(const unsigned char& value)
      throw(const Exception&);

    virtual Result BrightDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&);
    /**
     * Exposure Compensation
     */

    virtual Result ExposureCompensationOnOff(const unsigned char& value1, const bool& value2)
      throw(const Exception&);

    virtual Result ExposureCompensationReset(const unsigned char& value)
      throw(const Exception&);

    virtual Result ExposureCompensationUp(const unsigned char& value)
      throw(const Exception&);

    virtual Result ExposureCompensationDown(const unsigned char& value)
      throw(const Exception&);

    virtual Result ExposureCompensationDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&);
    /** 
     * BackLight
     */

    virtual Result BackLightOnOff(const unsigned char& value1, const bool& value2)
      throw(const Exception&);
    /**
     * Aperture
     */

    virtual Result ApertureReset(const unsigned char& value)
      throw(const Exception&);

    virtual Result ApertureUp(const unsigned char& value)
      throw(const Exception&);

    virtual Result ApertureDown(const unsigned char& value)
      throw(const Exception&);

    virtual Result ApertureDirect(const unsigned char& value1, const unsigned short& value2)
      throw(const Exception&);
    /**
     * Wide Mode
     */

    virtual Result WideModeOff(const unsigned char& value)
      throw(const Exception&);

    virtual Result WideModeCinema(const unsigned char& value)
      throw(const Exception&);

    virtual Result WideMode16to9Full(const unsigned char& value)
      throw(const Exception&);
    /**
     *  Mirror image on or off
     */

    virtual Result LR_Reverse(const unsigned char& value1, const bool& value2)
      throw(const Exception&);
    /**
     *  Freeze image on or off
     */

    virtual Result Freeze(const unsigned char& value1, const bool& value2)
      throw(const Exception&);
    /**
     * Picture Effects
     *
     * These functions allows to turn internal effects on or off. 
     * They expect as parameter the <camera_id>.
     */

    virtual Result PictureEffectOff(const unsigned char& value)
      throw(const Exception&);

    virtual Result PictureEffectPastel(const unsigned char& value)
      throw(const Exception&);

    virtual Result PictureEffectNegArt(const unsigned char& value)
      throw(const Exception&);

    virtual Result PictureEffectSepia(const unsigned char& value)
      throw(const Exception&);

    virtual Result PictureEffectBW(const unsigned char& value)
      throw(const Exception&);

    virtual Result PictureEffectSolarize(const unsigned char& value)
      throw(const Exception&);

    virtual Result PictureEffectMosaic(const unsigned char& value)
      throw(const Exception&);

    virtual Result PictureEffectSlim(const unsigned char& value)
      throw(const Exception&);

    virtual Result PictureEffectStretch(const unsigned char& value)
      throw(const Exception&);
    /**
     * Digital Effects
     *
     * These functions allows to turn internal digital effects 
     * on or off. They expect as parameter the <camera_id>.
     *
     * Please don't call DigitalEffect( in unsigned char value1, in unsigned char ), its a parent
     * functions for all the other effect functions  
     */

    virtual Result DigitalEffectOff(const unsigned char& value)
      throw(const Exception&);

    virtual Result DigitalEffectStill(const unsigned char& value)
      throw(const Exception&);

    virtual Result DigitalEffectFlash(const unsigned char& value)
      throw(const Exception&);

    virtual Result DigitalEffectLumi(const unsigned char& value)
      throw(const Exception&);

    virtual Result DigitalEffectTrail(const unsigned char& value)
      throw(const Exception&);

    virtual Result DigitalEffectLevel(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&);
    /**
     * Memory (user defined presets)
     */

    virtual Result Memory(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&);

    virtual Result MemoryReset(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&);

    virtual Result MemorySet(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&);

    virtual Result MemoryRecall(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&);
    /**
     * DataScreen (OSD)
     */

    virtual Result DataScreenOnOff(const unsigned char& value1, const bool& value2)
      throw(const Exception&);
    /**
     * IR (remote controller)
     */

    virtual Result IRReceiveOnOff(const unsigned char& value1, const bool& value2)
      throw(const Exception&);

    virtual Result IRReceiveReturn(const unsigned char& value1, const bool& value2)
      throw(const Exception&);
    /** 
     * PanTiltDriveMotion
     *
     * input  : camera-identifier value1,
     *          motion x-direction, motion y-direction, 
     *          speed x-direction, speed y-direction
     * output : SUCCESS, if command send with success
     *          FAILURE, if an error occured
     * 
     * used for PanTiltDrive-motion functions
     */

    virtual Result PanTiltDriveMotion(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3, const unsigned char& value4, const unsigned char& value5)
      throw(const Exception&);

    virtual Result PanTiltDriveUp(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&);

    virtual Result PanTiltDriveDown(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&);

    virtual Result PanTiltDriveLeft(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&);

    virtual Result PanTiltDriveRight(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&);

    virtual Result PanTiltDriveUpLeft(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&);

    virtual Result PanTiltDriveUpRight(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&);

    virtual Result PanTiltDriveDownLeft(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&);

    virtual Result PanTiltDriveDownRight(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&);

    virtual Result PanTiltDriveStop(const unsigned char& value1, const unsigned char& value2, const unsigned char& value3)
      throw(const Exception&);
    /** 
     * PanTiltDrivePosition
     *
     * input  : camera-identifier,
     *          position-mode (relative or absolute)
     *          motion x-direction in coordinates
     *          morion y-direction in coordinates
     *          speed x-direction, speed y-direction
     * output : SUCCESS, if command send with success
     *          FAILURE, if an error occured
     * 
     * used for PanTiltDrive-position functions
     */

    virtual Result PanTiltDrivePosition(const unsigned char& value1, const unsigned char& value2, const int& value3, const int& value4, const unsigned char& value5, const unsigned char& value6)
      throw(const Exception&);

    virtual Result PanTiltDriveAbsolute(const unsigned char& value1, const int& value2, const int& value3, const unsigned char& value4, const unsigned char& value5)
      throw(const Exception&);

    virtual Result PanTiltDriveRelative(const unsigned char& value1, const int& value2, const int& value3, const unsigned char& value4, const unsigned char& value5)
      throw(const Exception&);

    virtual Result PanTiltDriveAbsoluteDeg(const unsigned char& value1, const int& value2, const int& value3, const unsigned char& value4, const unsigned char& value5)
      throw(const Exception&);

    virtual Result PanTiltDriveRelativeDeg(const unsigned char& value1, const int& value2, const int& value3, const unsigned char& value4, const unsigned char& value5)
      throw(const Exception&);

    virtual Result PanTiltDriveHome(const unsigned char& value)
      throw(const Exception&);

    virtual Result PanTiltDriveReset(const unsigned char& value)
      throw(const Exception&);

    virtual Result PanTiltLimitSet(const unsigned char& value1, const unsigned char& value2, const int& value3, const int& value4)
      throw(const Exception&);

    virtual Result PanTiltLimitClear(const unsigned char& value1, const unsigned char& value2)
      throw(const Exception&);
    /**
     * Inquiry Command List 
     */
    /**
     * PowerInq
     *
     * input : The camera ID
     * ouput : The status of the Camera
     *          true = On
     *          false =Off(Standby)
     */

    virtual Result PowerInq(const unsigned char& value1, bool& value2)
      throw(const Exception&);

    virtual Result AutoPowerOffInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&);

    virtual Result DZoomModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&);

    virtual Result ZoomPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&);

    virtual Result FocusModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&);

    virtual Result FocusPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&);

    virtual Result AFModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&);

    virtual Result FocusNearLimitInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&);

    virtual Result WhiteBalanceModeInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&);

    virtual Result RGainInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&);

    virtual Result BGainInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&);

    virtual Result AutoExposureModeInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&);

    virtual Result SlowShutterModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&);

    virtual Result ShutterPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&);

    virtual Result IrisPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&);

    virtual Result GainPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&);

    virtual Result BrightPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&);

    virtual Result ExposureCompensationModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&);

    virtual Result ExposureCompensationPosInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&);

    virtual Result BackLightModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&);

    virtual Result ApertureInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&);

    virtual Result WideModeInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&);

    virtual Result LR_ReverseModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&);

    virtual Result FreezeModeInq(const unsigned char& value1, bool& value2)
      throw(const Exception&);

    virtual Result PictureEffectModeInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&);

    virtual Result DigitalEffectModeInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&);

    virtual Result DigitalEffectLevelInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&);

    virtual Result MemoryInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&);

    virtual Result DatascreenInq(const unsigned char& value1, bool& value2)
      throw(const Exception&);

    virtual Result PanTiltModeInq(const unsigned char& value1, unsigned short& value2)
      throw(const Exception&);

    virtual Result PanTiltMaxSpeedInq(const unsigned char& value1, unsigned char& value2, unsigned char& value3)
      throw(const Exception&);

    virtual Result PanTiltPosInq(const unsigned char& value1, int& value2, int& value3)
      throw(const Exception&);

    virtual Result VideoSystemInq(const unsigned char& value1, unsigned char& value2)
      throw(const Exception&);

    virtual Result DeviceTypeInq(const unsigned char& value1, unsigned short& value2, unsigned short& value3, unsigned short& value4, unsigned char& value5)
      throw(const Exception&);
    /**
     * misc stuff
     */

    virtual void verbose(const string& value1, const string& value2)
      throw(const Exception&);

    virtual bool islocked(const unsigned char& value)
      throw(const Exception&);


  };

  typedef auto_ptr<IVISCACamera> IVISCACamera_var;




}

#endif
