/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Dostert Roger <dostert@studcs.uni-sb.de>
 *                    
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _JOYSTICKEVENTS_HPP_
#define _JOYSTICKEVENTS_HPP_ _JOYSTICKEVENTS_HPP_

#include <linux/joystick.h>
#include <string>
#include <signal.h>
#include <fcntl.h>

#include "nmm/utils/thread/Thread.hpp"
#include "nmm/base/EventDispatcher.hpp"


namespace NMM{

  /*
   * 
   */
  class JoystickEventsInterface {
  public:
    JoystickEventsInterface() {};
    virtual ~JoystickEventsInterface() {};
    virtual Result JoystickAxisScale(unsigned char&, short int&, int&) = 0;
  };


  class JoystickEvents {
  public:
    typedef signed short int sword; // Signed Word, not Sword ;)
    
    /*
     * constructor and destructor
     */
    JoystickEvents();
    ~JoystickEvents();

    /*
     * set the joystick-device
     */
    Result set_devicename(string&);

    /*
     * get the name of the current joystick device
     */
    string get_devicename(void);

    /**
     * Connects this event-class to a dispatcher (listener)
     */
    void connectTo( EventDispatcher *);
 
    /**
     * Starts the event-handling in a new thread
     */
    //    bool start(Result (*function)(unsigned char&, short int&, int&));
    bool start(JoystickEventsInterface*);

    /*
     * stops event-handling (exit thread)
     */
    void stop();

  private:
    bool                      running;     /* the producer thread is running            */
    int                       dev_handle;  /* file-handle to joystick device            */
    string                    dev_name;    /* name of joystick device (e.g. "/dev/js0") */
    int                       num_buttons; /* found number of joystick buttons          */
    int                       num_axis;    /* found number of joystick axes             */

    Thread                    thread;      /* Thread ID                                 */
    pid_t                     thread_pid;  /* process-id of the thread                  */
    EventDispatcher           *dispatcher; 

    JoystickEventsInterface   *axisscale;  /* interface to the axes scaling function    */
    js_event                  jevent;      /* structure for one Joystick-driver-event   */


    /* 
     * status of all joystick buttons and axes 
     * because different joysticks can have different amount of 
     * buttons and axis they are declared as pointer instead as arrays
     */
    bool                      *buttons;
    sword                     *axis;


    /**
     * This method is merely a way to start the thread on this
     * object's method.
     */
    static void* getEvent_static(void *me);
    
    /**
     * The event-handling function
     * Events can be received by all registered input-devices
     * (like X-Display, Console, mouse, joystick, ...)
     */
    void getEvent();
  };

}; /* namespace NMM */

#endif /* _JOYSTICKEVENTS_HPP_ */
