/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "Timer.hpp"

namespace NMM {
  Timer::Timer (TimerListener& listener) : __is_running(false), __listener(listener), __thread_condition(__condition_mutex), __sec(0), __nsec(0)
  {

  }

  Timer::~Timer() {
    stop();
  }

  void Timer::start(int sec, int nsec) {
    MutexGuard mg(__condition_mutex);
    if ( !__is_running ) {
      __sec         = sec;
      __nsec        = nsec;
      __is_running  = true;
      activateThread();
    }
  }

  void Timer::stop() {
    if ( __is_running ) {
      __condition_mutex.lock();
      __is_running = false;
      __thread_condition.notify();
      __condition_mutex.unlock();
      deactivateThread();
    }
  }

  void Timer::run() {
    while (__is_running) {
      __condition_mutex.lock();  // lock the mutex
      __thread_condition.relativeTimedWait(__sec, __nsec);

      if (__is_running) {
        __condition_mutex.unlock();
        __listener.timeout(this);
      } else {
        __condition_mutex.unlock();
      }
    }
  }

} // NMM

