/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
#ifndef NMM_TVGUIDE_PLUGIN_MANAGER_HPP
#define NMM_TVGUIDE_PLUGIN_MANAGER_HPP NMM_TVGUIDE_PLUGIN_MANAGER_HPP

#include <string>
#include <list>
#include "nmm/utils/Singleton.hpp"
#include "nmm/misc/tvguide/TVGuidePlugin.hpp"

using namespace std;

namespace NMM{

  /** This class is our manager to store all plugins. This class must be designed as singelton to  
   *  provide a uniqe access point for plugins and application 
   */	
  class TVGuidePluginManager {
    friend class CreateUsingNew<TVGuidePluginManager>;
  public:
    void registerPlugin(TVGuidePlugin* plugin);
    void unregisterPlugin(TVGuidePlugin* plugin);

    list<TVGuidePlugin*>& getPluginList() { return __plugin_list; }
    const list<TVGuidePlugin*>& getPluginList() const {return __plugin_list;}
  private:
    TVGuidePluginManager();
    ~TVGuidePluginManager();
    TVGuidePluginManager(const TVGuidePluginManager& ) {};
    
  private:
    list<TVGuidePlugin*> __plugin_list;
  };

  typedef Singleton<TVGuidePluginManager, CreateUsingNew> STVGuidePluginManager;
}

#endif
