/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *                     
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_NXTVEPGPLUGIN_HPP
#define NMM_NXTVEPGPLUGIN_HPP NMM_NXTVEPGPLUGIN_HPP

#include "nmm/interfaces/misc/tvguide/INxtvepgPluginImpl.hpp"

#include "nmm/misc/tvguide/TVGuide.hpp"
#include "nmm/misc/tvguide/TVGuidePlugin.hpp"

#include "nmm/utils/thread/ThreadedObject.hpp"
#include "nmm/misc/timer/TimerEvents.hpp"

#include "nmm/misc/tvguide/TVGXercescDatabase.hpp"


namespace NMM{

  class NxtvepgPlugin : public INxtvepgPluginImpl ,
                        public TVGuidePlugin
  {
  private:
    string configFile;

    //when running in daemon-Mode (not used)
    string daemonHost;
    int daemonPort;

    //this is used for XML-Dump
    list<string> provider;
    string period;
    string dbdir;
    string nxtvepgTmpFile;
    string language;
    bool termUpdate;
    bool useTvsplit;
    string tmpDir;
    TimerProducer* timer;

    XercesTranscoder *transcoder;
    DOMImplementation *impl;
    DOMBuilder *parser;

  public:
    NxtvepgPlugin();
    ~NxtvepgPlugin();

    TEDObject0<NxtvepgPlugin> update_timer;

  protected:
    Result doStart();
    Result doStop();
    Result doInit();
    Result doDeinit();

    void setLocation(const string &loc);
    void setPort(const int&p);

    void addProvider(const string& p);
    list<string> getProvider();

    void setConfigFile(const string &file);
    string getConfigFile();

    void setDBDir(const string& db);
    string getDBDir();

    void setLocale(const string& l);
    string getLocale();

    void setPeriod(const string &p);
    string getPeriod();

    void setUseTv_split(const bool &b){useTvsplit = b;}
    bool getUseTv_split(){return useTvsplit;}

    Result restartUpdate();

  private:
    void readPluginConfig();
    void writePluginConfig();

    int readXMLTV(const string& file);

  };

}

#endif
