/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_CDDBSERVER_HPP
#define NMM_CDDBSERVER_HPP NMM_CDDBSERVER_HPP

#include "nmm/NMMTypes.hpp"
#include <string>
#include <vector>

const int CDDB_BUFFER_SIZE = 10240;
const std::string REDOUT = "\x1B[01;32m";
const std::string NORMALOUT = "\x1B[0;0m";

namespace NMM {

  class CDDBServer {

    /**
     * Exceptions
     */
    class CDDBError {};
    class SocketError: public CDDBError {};
    class RemoteError: public CDDBError {};

  private:
    struct CDDBTocEntry {
      long seconds;
      long frames;
      std::string name;
    };
    std::string title;
    std::string artist;
    std::string year;
    std::string genre;

    std::string server;
    int port;
    std::vector<CDDBTocEntry*> toc;
    std::string query_string;
    std::string answer_string;    /* the cddb server answer */
    int socketd;

    unsigned int cddbSum(int);

    long computeCDDBId();

    void prepareQuery();

    Result parseAnswer();

    std::string readSocket();

    void writeSocket(std::string);

    Result remoteQuery();

  public:
    CDDBServer(std::string, int);
    ~CDDBServer();

    /**
     * Adds a track at the end of the internal track list
     */
    void addTrack(long, long);

    /**
     * make the CDDB Query
     */
    Result doQuery(); 

    /**
     *  gets a name of a track
     */
    std::string getName(unsigned int);

    /**
     *  gets the title of the cd
     */
    std::string getTitle();

    /**
     *  gets the artist of the cd
     */
    std::string getArtist();

    /**
     *  gets the year of the cd
     */
    std::string getYear();

    /**
     *  gets the genre of the cd
     */
    std::string getGenre();
  };

}

#endif //NMM_CDDBSERVER_HPP
