/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_TABLEOFCONTENTS_HPP_
#define _NMM_TABLEOFCONTENTS_HPP_ _NMM_TABLEOFCONTENTS_HPP_

#include <vector>
#include <string>

#include "CDDBServer.hpp"

namespace NMM {

  class TableOfContents {
  private:
    typedef struct TocEntry {
      long start_sector;
      long end_sector;
      std::string name;
      bool is_audio;
    }; 
    std::vector<TocEntry*> toc;
    std::string title;
    std::string artist;
    std::string year;
    std::string genre;

    bool size_set;
    std::string cddb_server;
    int cddb_port;

  protected:
  public:
    TableOfContents();
    ~TableOfContents();

    unsigned int size();

    Result setSize(unsigned int);

    void setStartSector(int num, long sec);
    long getStartSector(int num);

    void setEndSector(int num, long sec);
    long getEndSector(int num);

    long getSectors(int num);

    long getSeconds(int num);

    void setName(int num, std::string name);
    std::string getName(int num);

    std::string getTitle();
    std::string getArtist();
    std::string getYear();
    std::string getGenre();

    void setAudio(int num, bool audio);
    bool getAudio(int num);

    int getTrackBySector(long sector);

    std::string toString();


    long getCDDBFirstFrames();

    long getCDDBFrames(int num);

    long getCDDBFirstSeconds();

    long getCDDBSeconds(int num);

    Result queryCDDBServer();

    void addCDDBServer(std::string, int);
  };

} // namespace NMM

#endif // _NMM_TABLEOFCONTENTS_HPP_


