/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_MPEGAUDIOENCODENODE_HPP
#define NMM_MPEGAUDIOENCODENODE_HPP NMM_MPEGAUDIOENCODENODE_HPP

#include "nmm/base/GenericProcessorNode.hpp"

#include "nmm/interfaces/file/ITrackImpl.hpp"
#include "nmm/interfaces/general/IEncodeImpl.hpp"
#include "nmm/interfaces/audio/ITagInfoImpl.hpp"

#include <string>

#include <lame/lame.h>

namespace NMM {

  class MPEGAudioEncodeNode :
    public GenericProcessorNode,
    public ITrackImpl,
    public IEncodeImpl,
    public ITagInfoImpl
  {
  private:
    ThreadMutex mae_mutex;
    
    const int audio_codec;
    
    string filename;
    string year;
    string genre;
    string artist;
    string title;
    string album;
    
  public:
    MPEGAudioEncodeNode(const char* name = "MPEGAudioEncodeNode",
                        const StreamQueue::Mode queue_mode = StreamQueue::MODE_SUSPEND,
                        const int queue_size = StreamQueue::DEFAULT_SIZE);
    
    ~MPEGAudioEncodeNode();
    
  protected:
    virtual Result doInit();
    virtual Result doDeinit();
    virtual Result doInitOutput();
    virtual Result doDeinitOutput();
    virtual Result doActivate();
    virtual Result doDeactivate();
    
    virtual Message* processBuffer(Buffer *in_buffer); 
    
    virtual Result startTrack(const string& filename);
    virtual Result endTrack();
    virtual Result finished();

    virtual Result TagInfo(const string& title, const string& album, const string& _artist, const string& _year, const string& _genre);

    /**
     * IEncode interface.
     * The bitrate parameter is not used.
     * The valid range for the quality is [0...9] (integer).
     */
    virtual void setQuality(const float& q);
    virtual float getQuality();
    virtual void setBitrate(const int& i);
    virtual int getBitrate();

  private:
    enum EVENTSTATE { START, DATA, END, FINISHED, TEST };
    EVENTSTATE event;

    lame_global_flags *gfp;

    bool encoding;
    int channels;
    uint lame_quality;

  }; /* class MPEGAudioEncodeNode */

} /* namespace NMM */

#endif //NMM_MPEGAUDIOENCODENODE_HPP
