/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_RAWREADNODE_HPP
#define NMM_RAWREADNODE_HPP

#include "nmm/base/GenericSourceNode.hpp"
#include "nmm/interfaces/file/IFileHandlerImpl.hpp"
#include "nmm/interfaces/file/IFileModeHandlerImpl.hpp"

namespace NMM {

  /**
   * RawReadNode 
   */
  class RawReadNode :
    public GenericSourceNode,
    public IFileHandlerImpl,
    public IFileModeHandlerImpl
  {
  private:
    string          file_name;
    int             file_des;
    ThreadMutex     file_name_mutex;
    bool            send_format;
    bool            loop;
    bool            send_event;
    size_t          frame_size;
    
    IFileModeHandler::FileMode   read_mode; 
    IFileModeHandler::FileFormat file_format;
    
  protected:
    virtual Result doInit();
    
    virtual Result doInitOutput();
    virtual Result doDeinitOutput();
    
    // IFileHandler interface
    virtual void setFilename(const string& name);
    
    // IFileModeHandler interface
    virtual void setMode(const IFileModeHandler::FileMode& mode);
    virtual IFileModeHandler::FileMode getMode();
    virtual void setFormat(const IFileModeHandler::FileFormat& format);
    virtual IFileModeHandler::FileFormat getFormat();
    virtual void setFilenameMode(const IFileModeHandler::FilenameMode& mode);
    virtual IFileModeHandler::FilenameMode getFilenameMode();
    virtual void setLoop(const bool& d);
    
    virtual Message *processBuffer(Buffer *);
    
  public:
    RawReadNode(const char* name = "RawReadNode",
		StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
		int = StreamQueue::DEFAULT_SIZE);
    
    virtual ~RawReadNode();
  };
  
}

#endif 

