/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_AVIREADNODE_HPP
#define NMM_AVIREADNODE_HPP NMM_AVIREADNODE_HPP

#include "nmm/base/GenericSourceNode.hpp"
#include "nmm/interfaces/base/sync/ISyncResetImpl.hpp"
#include "nmm/interfaces/file/IFileHandlerImpl.hpp"
#include "nmm/interfaces/file/ISeekableImpl.hpp"
#include "nmm/interfaces/general/progress/IProgressImpl.hpp"

// do forward declaration of needed structs from avilib/avilib.h
struct avi_t;

namespace NMM {
  
  /**
   * AVIReadNode 
   */
  class AVIReadNode :
    public GenericSourceNode,
    public ISyncResetImpl,
    public IFileHandlerImpl,
    public ISeekableImpl,
    public IProgressImpl
  {
    
  private:
    int             av_pos;             ///< position of video in file * 2 (count video and audio)
    int             av_offset;          ///< to which position (audio + video) did we seek?
    int             read_size;          ///< how large is the video frame we read
    string          file_name;          ///< our input filename
    bool            seeked;             ///< was a seek handled shortly?
    bool            seek_backwards;     ///< seek backwards?
    bool            end_of_file;        ///< EOF reached?
    bool            begin_of_file;      ///< are we currently at the beginning of the file
    bool            all_at_the_beginning; ///< flag to decide wheter to send BufferLabel event
    bool            send_progress;      ///< should the node send progress information?
    bool            send_format;         ///< should we send a setFormat-Event?
    ThreadMutex     file_name_mutex;
    ThreadMutex     seek_mutex;
    avi_t*          avi_file;           ///< struct from avilib, contains file handle
    int             width;              ///< Width of Video Frames
    int             height;             ///< Height of Video Frames
    int             num_frames;         ///< Number of Video Frames
    double          frame_rate;         ///< Video Framerate
    int             int_frame_rate;     ///< internal Framerate, integer representation
    const int       frame_rate_base;    ///< constant between frame_rate and int_frame_rate
    char*           video_compressor;   ///< Video Compressor String
    int             audio_channels;     ///< Number of audio channels
    int             audio_bits;         ///< Number of audio bits
    int             audio_format;       ///< Audio format, see avilib.h for formats
    int             audio_rate;         ///< Audio samplerate
    int             audio_bytes;        ///< total number of audio bytes in the file
    int             audio_bitrate;      ///< audio bitrate
    int             is_key_frame;       ///< flag for information: is the current frame a key frame?
    int             aud_buf_size;       ///< Buffer size for audio buffers
    const int       vid_buf_bound;      ///< Upper Bound for size of most video frames
    int             buffers_before_progress;
    int             buffers_progress;
    Buffer*         __current_buffer;   ///< stores the buffer buffer pointer if an key frame event is send
    bool            valid_audio;        ///< indicates if we send the first buffer, used to mark the first audiotimstamp as valid
    
    virtual Result doInit();
    virtual Result doInitOutput();
    
    virtual Message* processBuffer(Buffer *);   ///< standard produceBuffer
    
  protected:

    // IFileHandler interface
    virtual void setFilename(const string& name);
    
    // ISeekable interface
    virtual bool hasPercentSeek() const;
    virtual void seekPercent(const Rational& percent);
    virtual void seekPercentTo(const Rational& percent);
    virtual bool hasBytesSeek() const;
    virtual void seekBytes(const int64_t& offset);
    virtual void seekBytesTo(const u_int64_t& bytes);
    virtual bool hasTimeSeek() const;
    virtual void seekTime(const Interval& interval);
    virtual void seekTimeTo(const Time& time);
    
    // ISyncReset interface
    virtual Result syncReset();

    // IProgress interface
    virtual void sendProgressInformation(const bool& in);
    virtual void setProgressInterval(const int& buffers);

  public:

    AVIReadNode(const char* name = "AVIReadNode",
		StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
		int = StreamQueue::DEFAULT_SIZE); ///< Constructor
    virtual ~AVIReadNode();                        ///< Destructor
  };
  
}

#endif 

