/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_MPEGREADNODE_HPP
#define NMM_MPEGREADNODE_HPP NMM_MPEGREADNODE_HPP

#include "MPEGCheckFormat.hpp"

#include "nmm/base/GenericSourceNode.hpp"

#include "nmm/interfaces/file/IFileHandlerImpl.hpp"
#include "nmm/interfaces/file/ISeekableImpl.hpp"
#include "nmm/interfaces/file/IBufferSizeImpl.hpp"
#include "nmm/interfaces/general/progress/IProgressImpl.hpp"

#include "nmm/base/sync/StreamTimer.hpp"


namespace NMM {

  class MPEGReadNode :
    public GenericSourceNode,
    public MPEGCheckFormat,
    public IProgressImpl,
    public IFileHandlerImpl,
    public ISeekableImpl,
    public IBufferSizeImpl
  {
    
  private:
    string           filename;
    bool             send_start;
    bool             send_progress;
    bool             send_format;
    mpeg_file_format file_format;

    // seeking & file position
    u_int64_t        filesize;
    u_int64_t        filepos;
    int64_t          seekto;

    // progress
    int              buffers_progress;
    int              buffers_before_progress;
    StreamTimer stream_timer;
    
    // need for CheckFormat
    int MPEG_Read_Buffer( unsigned char *buf, int size );
    void MPEG_Seek_Begin();
    
  protected:
    ifstream stream;
    int buffer_size;
    
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doDeinitOutput();
    
    virtual Message* processBuffer(Buffer *);

    // IFileHandler interface
    virtual void setFilename(const string& name);
    
    // ISeekable interface
    virtual bool hasPercentSeek() const;
    virtual void seekPercent(const Rational& percent);
    virtual void seekPercentTo(const Rational& percent);
    virtual bool hasBytesSeek() const;
    virtual void seekBytes(const int64_t& offset);
    virtual void seekBytesTo(const u_int64_t& bytes);
    virtual bool hasTimeSeek() const;
    virtual void seekTime(const Interval& interval);
    virtual void seekTimeTo(const Time& time);
    
    // IProgress interface
    virtual void sendProgressInformation(const bool& in);
    virtual void setProgressInterval(const int& buffers);

    // IBufferSize interface
    virtual void setBufferSize(const int& size);
    virtual int getBufferSize();
        
  public:
    MPEGReadNode(const char *name = "MPEGReadNode",
		 StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
		 int = StreamQueue::DEFAULT_SIZE);
    
    virtual ~MPEGReadNode();
  };

}

#endif //NMM_MPEGREADNODE_HPP
