/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_PNGREADNODE_HPP_
#define _NMM_PNGREADNODE_HPP_ _NMM_PNGREADNODE_HPP_

#include "nmm/base/GenericSourceNode.hpp"
#include "nmm/interfaces/file/IMultipleSourceFileHandlerImpl.hpp"
#include "nmm/interfaces/file/IFileModeHandlerImpl.hpp"
#include "nmm/interfaces/file/IFileHandlerImpl.hpp"
#include "nmm/interfaces/video/image/ICreateImageImpl.hpp"

namespace NMM {

  class PNGReadNode : public GenericSourceNode,
		      public IMultipleSourceFileHandlerImpl,
		      public IFileModeHandlerImpl,
		      public ICreateImageImpl,
          public IFileHandlerImpl {

  private:
    bool last_image_done;      ///< set true when a picture is completely sent (including end_track)
    bool end_track;
    unsigned int image_width;  ///< width of the image in the input stream
    unsigned int image_height; ///< height of the image in the input stream
    int frame_count;           ///< How often was the picture send?
    unsigned char* data;       ///< Bitmap data (rgba format)
    int buff_size;
    string cur_filename;
    list<string> filenames;
    string fileext;
    const string deffileext;
    IFileModeHandler::FilenameMode filename_mode;
    ThreadMutex filename_mutex;   ///< Thread-Mutex
    bool format_changed;

    /**
     * loads a (PNG) Bitmap
     * \param filename filename of the bitmap
     * \param width pointer to a variable that holds the bitmap width after loading
     * \param height pointer to a variable that holds the bitmap height after loading
     * \returns pointer to the bitmap data (rgba format)
     */
    unsigned char* loadBitmap( string filename, unsigned int* width, unsigned int* height  );

    bool openImage();

    void setWOPInternal();

  protected:
    Result doInit();
    Result doInitOutput();
    Result doActivate();
    Message* processBuffer(Buffer *);

    /* This method calls addFilename */
    virtual void setFilename(const string& file);

    // IMultipleSourceFileHandler interface
    virtual void addFilename(const string&);

    // IFileModeHandler interface
    virtual void setMode(const IFileModeHandler::FileMode& mode);
    virtual IFileModeHandler::FileMode getMode();
    virtual void setFormat(const IFileModeHandler::FileFormat& format);
    virtual IFileModeHandler::FileFormat getFormat();
    virtual void setFilenameMode(const IFileModeHandler::FilenameMode& mode);
    virtual IFileModeHandler::FilenameMode getFilenameMode();
    virtual void setLoop(const bool& d);
    
    // ICreateImage interface
    virtual Result createImage(const unsigned int& color, const unsigned int& w, const unsigned int& h);

  public:

    PNGReadNode(const char *name = "PNGReadNode",
		StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
		int = StreamQueue::DEFAULT_SIZE);
    ~PNGReadNode();


  };

}

#endif //_NMM_PNGREADNODE_HPP_
