/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_TIMEDISPLAYNODE_HPP_
#define _NMM_TIMEDISPLAYNODE_HPP_

#include "nmm/base/GenericFilterNode.hpp"
#include "nmm/interfaces/video/filter/ITextDisplayImpl.hpp"

#include <png.h>

namespace NMM {

  class TimeDisplayNode :
    public GenericFilterNode,
    public ITextDisplayImpl
  {
  private:
    png_bytep *png_rows;               ///< stores the PNG image
    png_uint_32 png_width, png_height; ///< PNG image width and height
    
    int x_offset, y_offset; ///< display position
    string logo_filename;   ///< filename for the clock font image
    
    int image_width;        ///< width of the image in the input stream
    int image_height;       ///< height of the image in the input stream
    
    /**
     * Loads and prepares the PNG image.
     */
    Result preparePNGLogo(string filename);
    
    /**
     * Sets a pixel on the buffer image.
     */
    void setPixel(char *image, int x, int y, 
		  unsigned char r, unsigned char g, unsigned char b);
    
    /**
     * Displays the time string on the buffer image
     */
    void paintTime(char *image, char *text, int x, int y);
    
    /**
     * Returns the index of a given character in the clock font image
     */
    int charPos(char c);
    
  protected:
    /* ==== from GenericProcessorNode ==== */
    
    /**
     * The standard doInit method from Node. Effects in TimeDisplayNode:
     * loads the clock font.
     */
    virtual Result doInit();
    
    /**
     * The standard doInitOutput method from Node. Effects in TimeDisplayNode:
     * obtain and check the size of the incoming buffer image.
     */
    virtual Result doInitOutput();
    
    /**
     * The standard processBuffer method from GenericProcessorNode.
     * Effects in TimeDisplayNode: display the current (system) time
     * on the image in the buffer.
     */
    virtual Buffer *processBuffer(Buffer *in_buffer);
    
    /**
     * ITextDisplay interface.
     */
    void setFontImage(const string& path);
    void setOffset(const int& x, const int& y);
    void getOffset(int& x, int& y) const;
    
  public:
    TimeDisplayNode(const char *name = "TimeDisplayNode",
		    const StreamQueue::Mode queue_mode = StreamQueue::MODE_SUSPEND, 
		    const int queue_size = 1);
    ~TimeDisplayNode();
  };
}

#endif
