/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "MultipleOSDNode.hpp"
#include "nmm/base/format/Property.hpp"
#include "nmm/base/registry/Plugin.hpp"
#include "nmm/utils/SecurityCheck.hpp"

#include <stdio.h>
#include <iostream>
#include <assert.h>
#include <string.h>

#ifndef NULL
#define NULL 0
#endif

NMM_REGISTER_PLUGIN(NMM::MultipleOSDNode, "MultipleOSDNode")

namespace NMM {

  //****************************** OSDItem ********************************************

  OSDItem::OSDItem() 
  { 
    depth = width = height = x_offset = y_offset = 0; 
    enabled = true; 
  }

  //****************************** OSDText ********************************************
  OSDText::OSDText() 
  {
    osd_text= new char[256];
    osd_font= 0;
    osd_progbar_type= -1;
    osd_progbar_value= 100;   // 0..255
    dxs = 0;
    dys = 0;
    
  }

  Result OSDText::setFont(font_desc_t* osd_font) 
  {
    this->osd_font = osd_font;
    return SUCCESS;
  }

  Result OSDText::setText(const string text ) 
  {    
    const char *s = text.c_str();
    memcpy(osd_text, s, strlen(s) + 1);

    return SUCCESS;
  }

  Result OSDText::draw(char *data, int color, size_t width, size_t height)
  {
    current_data = data;
    dxs = x_offset;
    dys = y_offset;
    image_width = width;
    image_height = height;
    this->color = color;

    GLOBAL_DEBUG_STREAM("DRAW TEXT " << osd_text << " " << color << " " << image_height << " " << image_width << endl);
    // return SUCCESS;

    if(!osd_font) return FAILURE; // no font

    if(osd_text){
      draw_text_osd();
    }
 
    if(osd_progbar_type >= 0) {
      draw_text_progbar();
    }
    
    return SUCCESS;
  }

  void OSDText::draw_alpha_rgb32(int w,int h, unsigned char* src, unsigned char *srca, int srcstride, unsigned char* dstbase,int dststride)
  {
    int y;
    for(y=0;y<h;y++){
      register int x;
      for(x=0;x<w;x++){
	if(srca[x]){
	  dstbase[4*x+0]=src[x];
	  dstbase[4*x+1]=src[x];
	  dstbase[4*x+2]=src[x];
	  dstbase[4*x+3]=255-srca[x];
	}
      }
      src+=srcstride;
      srca+=srcstride;
      dstbase+=dststride;
    }
    return;
  }
  
  void OSDText::draw_alpha_yv12(int w,int h, unsigned char* src, unsigned char *srca, int srcstride, unsigned char* dstbase,int dststride){
  
    int y;
    unsigned char *alpha;
    int alpha_offset;
    
    alpha_offset = image_width*image_height;
    alpha_offset = alpha_offset + (alpha_offset/4)*2; 
    alpha = dstbase + alpha_offset;
    
    for(y=0;y<h;y++){
      register int x;
      for(x=0;x<w;x++){
	if(srca[x]) { 
	  dstbase[x]= src[x];
	  alpha[x] = 255-srca[x];
	}
      }
      src+=srcstride;
      srca+=srcstride;
      dstbase+=dststride;
      alpha+=dststride;
    }
    return;
  }

  void OSDText::draw_alpha(int x0,int y0, int w,int h, unsigned char* src, unsigned char *srca, int stride) { 
    switch( color ) {
    case RGBA32:
      draw_alpha_rgb32(w,h,src,srca,stride,(unsigned char *) current_data+image_width*4*y0+(x0*4),image_width*4);
      break;
      
    case YVA12:
      draw_alpha_yv12(w,h,src,srca,stride,(unsigned char *) current_data+image_width*y0+x0,image_width);
      break;
    }
  }
  
  void OSDText::draw_text_osd()
  {
    int len=strlen(osd_text);
    int j;
    int x;
    int y;
    
    x = dxs;
    y = dys;
   
    for(j=0;j<len;j++){
      int c=osd_text[j];
      int font=osd_font->font[c];
      if(font>=0)
        draw_alpha(x,y,
		   osd_font->width[c],
		   osd_font->pic_a[font]->h,
		   osd_font->pic_b[font]->bmp+osd_font->start[c],
		   osd_font->pic_a[font]->bmp+osd_font->start[c],
		   osd_font->pic_a[font]->w);
      x+=osd_font->width[c]+osd_font->charspace;
    }
  }

  void OSDText::draw_text_progbar()
  {
    int i;
    int y=dys/2;
    int x;
    int c,font;
    int width=(dxs*2/3-osd_font->width[0x10]-osd_font->width[0x12]);
    int elems=width/osd_font->width[0x11];
    int mark=(osd_progbar_value*(elems+1))>>8;
    x=(dxs-width)/2;

    c=osd_progbar_type;font=osd_font->font[c];
    if(osd_progbar_type>0 && font>=0)
      draw_alpha(x-osd_font->width[c]-osd_font->spacewidth,y,
		 osd_font->width[c],
		 osd_font->pic_a[font]->h,
		 osd_font->pic_b[font]->bmp+osd_font->start[c],
		 osd_font->pic_a[font]->bmp+osd_font->start[c],
		 osd_font->pic_a[font]->w);

    c=OSD_PB_START;font=osd_font->font[c];
    if(font>=0)
      draw_alpha(x,y,
		 osd_font->width[c],
		 osd_font->pic_a[font]->h,
		 osd_font->pic_b[font]->bmp+osd_font->start[c],
		 osd_font->pic_a[font]->bmp+osd_font->start[c],
		 osd_font->pic_a[font]->w);
    x+=osd_font->width[c];

    for(i=0;i<elems;i++){
      c=(i<mark)?OSD_PB_0:OSD_PB_1;font=osd_font->font[c];
      if(font>=0)
	draw_alpha(x,y,
		   osd_font->width[c],
		   osd_font->pic_a[font]->h,
		   osd_font->pic_b[font]->bmp+osd_font->start[c],
		   osd_font->pic_a[font]->bmp+osd_font->start[c],
		   osd_font->pic_a[font]->w);
      x+=osd_font->width[c];
    }

    c=OSD_PB_END;font=osd_font->font[c];
    if(font>=0)
      draw_alpha(x,y,
		 osd_font->width[c],
		 osd_font->pic_a[font]->h,
		 osd_font->pic_b[font]->bmp+osd_font->start[c],
		 osd_font->pic_a[font]->bmp+osd_font->start[c],
		 osd_font->pic_a[font]->w);
  }

  //****************************** OSDImagePNG ****************************************

  OSDImagePNG::OSDImagePNG()
  {
    image_width = 0; 
    image_height = 0;
  }

  Result OSDImagePNG::draw(char *data, int /*color*/, size_t width, size_t height)
  {
    GLOBAL_DEBUG_STREAM( "DRAW image" << endl);
    image_width = width;
    image_height = height;
    paintPNGLogo(data, x_offset, y_offset);

    return SUCCESS;
  }

  Result OSDImagePNG::load(const string url) 
  {
    return preparePNGLogo(url.c_str());
  }

  /* ==== Internal methods follow ==== */

  void OSDImagePNG::setPixel(char *image, int x, int y,
			  unsigned char r, unsigned char g, unsigned char b, unsigned char a) {
    unsigned char *ptr;
    ptr = (unsigned char *)image;
    ptr += 4 * (image_width * y + x);
    ptr[0] = b;
    ptr[1] = g;
    ptr[2] = r;
    ptr[3] = a;
  }

  Result OSDImagePNG::preparePNGLogo(const char *filename) {

    // SEE THE LIBPNG-EXAMPLE.c FOR DETAILS!

    png_structp png_ptr;
    png_infop info_ptr;
    //unsigned int sig_read = 0;
    //png_uint_32 width, height;
    int bit_depth, color_type, interlace_type;
    FILE *fp;
    
    png_rows = NULL;
    
    if(!SSecurityCheck::getInstance().checkFilenameForReading(filename)) {
      GLOBAL_ERROR_STREAM("Security check not passed for file " << filename << ", check .nmmrc for options" << endl);
      return FAILURE;
    }
    if ((fp = fopen(filename, "rb")) == NULL) {
      GLOBAL_ERROR_STREAM("Could not open file " << filename << endl);
      return FAILURE;
    }
    
    png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
				     NULL, NULL, NULL);
    
    if (png_ptr == NULL) {
      fclose(fp);
      return FAILURE;
    }
    
    info_ptr = png_create_info_struct(png_ptr);
    if (info_ptr == NULL) {
      fclose(fp);
      png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
      return FAILURE;
    }

    if (setjmp(png_ptr->jmpbuf)) {
      png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
      fclose(fp);
      return FAILURE;
    }

    png_init_io(png_ptr, fp);

    png_read_info(png_ptr, info_ptr);
    
    png_get_IHDR(png_ptr, info_ptr, &png_width, &png_height,
		 &bit_depth, &color_type,
		 &interlace_type, NULL, NULL);

    
    if (bit_depth == 8 && color_type == PNG_COLOR_TYPE_RGB) {
      png_set_filler(png_ptr, 0x00, PNG_FILLER_AFTER);
      png_set_bgr(png_ptr);
      png_read_update_info(png_ptr, info_ptr);
    }

    png_rows = new png_bytep[png_height];

    for (unsigned int row = 0; row < png_height; row++) {
      png_rows[row] = new png_byte[png_get_rowbytes(png_ptr, info_ptr)];
    }
    
    png_read_image(png_ptr, png_rows);
    
    png_read_end(png_ptr, info_ptr);
    
    png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
    
    fclose(fp);
    
    return SUCCESS;
  }

  void OSDImagePNG::paintPNGLogo(char *image, int xoff, int yoff){
    unsigned char *pngimg;
    unsigned char *img; 

    // neg. offsets mean offset from OTHER border

    if (xoff < 0) xoff = (image_width + xoff) - png_width;
    if (yoff < 0) yoff = (image_height + yoff) - png_height;

    // clip to visible area
    // following variables are in coords of the logo!

    int x_start, x_stop;
    int y_start, y_stop;
    
    x_start = 0; x_stop = png_width;
    if (xoff + x_start < 0) x_start = -xoff;    
    if (xoff + x_stop > image_width) x_stop = image_width - xoff;

    y_start = 0; y_stop = png_height;
    if (yoff + y_start < 0) y_start = -yoff;  
    if (yoff + y_stop > image_height) y_stop = image_height - yoff;

    if ((x_start >= x_stop) || (y_start >= y_stop)) return;
    // logo out of visible area

    assert((x_start >= 0) 
           && ("x_start negativ in MultipleOSDNode::paintPNGLogo!"));
    assert((x_stop <= (int)png_width)
           && ("x_stop Ueberlauf in MultipleOSDNode::paintPNGLogo!"));
    assert((y_start >= 0) 
           && ("y_start negativ in MultipleOSDNode::paintPNGLogo!"));
    assert((y_stop <= (int)png_height)
           && ("y_stop Ueberlauf in MultipleOSDNode::paintPNGLogo!"));
    assert((xoff + x_start >= 0)
           && ("Bild faengt zu weit links an in MultipleOSDNode::paintPNGLogo!"));
    assert((xoff + x_stop < image_width)
           && ("Bild hoert zu weit rechts auf in MultipleOSDNode::paintPNGLogo!"));
    assert((yoff + y_start >= 0)
           && ("Bild faengt zu weit oben an in MultipleOSDNode::paintPNGLogo!"));
    assert((yoff + y_stop < image_height)
           && ("Bild hoert zu weit unten auf in MultipleOSDNode::paintPNGLogo!"));

    for (int y = y_start; y < y_stop; y++) {
      assert ((y >= 0) && (y < (int)png_height)
              && ("y out of bounds in MultipleOSDNode::paintPNGLogo!"));
      pngimg = (unsigned char*) png_rows[y] + 4 * x_start;
      assert ((4 * (xoff + image_width * (yoff + y))
               <= image_width * image_height * 4)
              && ("image pointer out of bounds in MultipleOSDNode::paintPNGLogo!"));
      img = (unsigned char*)image
        + 4 * (xoff + x_start+ image_width * (yoff + y));
      for (int x = x_start; x < x_stop; x++) {
	unsigned char r, g, b, a;
	/*unsigned char r0, g0, b0;*/
        assert(((unsigned char*)pngimg - (unsigned char*)png_rows[y]
                < 4 * (int)png_width)
               && ("pngimg out of bounds in MultipleOSDNode::paintPNGLogo!"));
	r = pngimg[0];
	g = pngimg[1];
	b = pngimg[2];
	a = pngimg[3];
	
	setPixel(image, xoff+x, yoff+y, r, g, b, a);
	    
	pngimg += 4; img += 4;
      }
    }    
  }

  //****************************** MultipleOSD ****************************************

  MultipleOSDNode::MultipleOSDNode(const char *name /* = "MultipleOSDNode" */,
				   StreamQueue::Mode mode /* = StreamQueue::MODE_SUSPEND */,
				   int queue_size /* = StreamQueue::DEFAULT_SIZE */)
    : GenericSourceNode(name, mode, queue_size) {

    NMM::Format *myOutputFormat;
    myOutputFormat=getOwnOutputProperty()->addNewFormat("video/raw");
    myOutputFormat->addWildcard(Format::x_resolution_param);
    myOutputFormat->addWildcard(Format::y_resolution_param);
    myOutputFormat->addStringParamValue(Format::colorspace_param,"rgba32");
    myOutputFormat->addStringParamValue(Format::channelorder_param,"bgra");
    myOutputFormat->addIntParamValue(Format::bitperpixel_param,32);
    myOutputFormat->addStringParamValue(Format::endian_param, "little");
    myOutputFormat->addWildcard(Format::framerate_param);
    myOutputFormat->addStringParamValue(Format::format_param,"packet");
    
    myOutputFormat=getOwnOutputProperty()->addNewFormat("video/raw");
    myOutputFormat->addWildcard(Format::x_resolution_param);
    myOutputFormat->addWildcard(Format::y_resolution_param);
    myOutputFormat->addStringParamValue(Format::colorspace_param,"yva12");
    myOutputFormat->addStringParamValue(Format::channelorder_param,"yvua");
    myOutputFormat->addIntParamValue(Format::bitperpixel_param,20);
    myOutputFormat->addStringParamValue(Format::endian_param, "little");
    myOutputFormat->addWildcard(Format::framerate_param);
    myOutputFormat->addStringParamValue(Format::format_param,"planar");

    osd_cond = new ThreadCondition(osd_mutex);	
    new_text_flag = false;

    /*
    getEventDispatcher()->addEvent("text", new TEDOBJECT4(MultipleOSDNode, string, string, size_t, size_t, this, &MultipleOSDNode::addText)); 
    getEventDispatcher()->addEvent("image", new TEDOBJECT4(MultipleOSDNode, string, string, size_t, size_t, this, &MultipleOSDNode::addImage)); 

    getEventDispatcher()->addEvent("item_enabled", new TEDOBJECT2(MultipleOSDNode, string, bool, this, &MultipleOSDNode::setItemEnabled));

    getEventDispatcher()->addEvent("update", new TEDOBJECT0(MultipleOSDNode, this, &MultipleOSDNode::update)); 
    */
  }

  MultipleOSDNode::~MultipleOSDNode() { 
    delete osd_cond;
  }

  Result MultipleOSDNode::addText(const string id, const string text, const size_t x_offset, const size_t y_offset)
  {
    osd_mutex.lock();

    OSDText *t = new OSDText();
    t->x_offset = x_offset;
    t->y_offset = y_offset;
    assert(osd_font != 0);
    t->setFont(osd_font);
    t->setText(text);

    items[id] = t; 

    osd_mutex.unlock();
    return SUCCESS;
  }

  Result MultipleOSDNode::addImage(const string id, const string url, const size_t x_offset, const size_t y_offset)
  {
    osd_mutex.lock();
    
    OSDImagePNG *t = new OSDImagePNG();
    t->x_offset = x_offset;
    t->y_offset = y_offset;
    t->load(url);

    items[id.c_str()] = t; 
    
    osd_mutex.unlock();
    return SUCCESS;
  }

  Result MultipleOSDNode::setOffset(const string id, const size_t x_offset, const size_t y_offset)
  {
    osd_mutex.lock();
    OSDItem *item = items.find(id)->second;
    if(item) {
      item->x_offset = x_offset ; 
      item->y_offset = y_offset ; 
      osd_mutex.unlock();
      return SUCCESS;
    }
    else {
      osd_mutex.unlock();
      return FAILURE;
    }
  }

  Result MultipleOSDNode::setText(const string id, const string text)
  {
    osd_mutex.lock();
    OSDText *ptr = dynamic_cast<OSDText *>(items.find(id)->second);
		    
    if(ptr != 0) {
      ptr->setText(text);
      osd_mutex.unlock();
      return SUCCESS;
    }
    else {
      osd_mutex.unlock();
      return FAILURE;
    }
  }

  Result MultipleOSDNode::setItemEnabled(const string id, const bool enable)
  {
    osd_mutex.lock();
    items.find(id)->second->enabled = enable;
    osd_mutex.unlock();

    return SUCCESS;
  }

  Result MultipleOSDNode::DisableItems()
  {
    osd_mutex.lock();

    for(items_t::iterator i = items.begin(); i != items.end(); ++i)
      i->second->enabled = false;

    osd_mutex.unlock();

    return SUCCESS;
  }

  Result MultipleOSDNode::update()
  {    
    osd_mutex.lock();
    new_text_flag = true;
    osd_cond->notify();
    osd_mutex.unlock();

    DEBUG_STREAM("DRAW update" << endl);
    return SUCCESS;
  }

  Result MultipleOSDNode::setFont(char *font_name, float font_factor ) 
  {
    osd_mutex.lock();
    if(! osd_font ) {
      osd_font = read_font_desc( font_name, font_factor, 0 );
      if( !osd_font ) {
	osd_mutex.unlock();
	return FAILURE;
      }
    }
    
    osd_mutex.unlock();
    return SUCCESS;
  }

  Result MultipleOSDNode::doActivate() 
  {
    int frame_size;
    
    const Format* o_format = getOutputFormat();
    if (o_format == 0) {
      ERROR_STREAM( "Invalid output format (NULL-ptr)" << endl);
      return FAILURE;
    }

    string colorspace = o_format->getStringValue(Format::colorspace_param);
    image_width  = o_format->getIntValue(Format::x_resolution_param);
    image_height = o_format->getIntValue(Format::y_resolution_param);

    if ((image_width <= 0) || (image_height <= 0)) {
      ERROR_STREAM( "Invalid Image-width or -height" << endl);
      return FAILURE;     
    }
    
    if(colorspace == "rgba32") {
      color = RGBA32;
      buff_size = image_width*image_height*4;
    }
     
    if(colorspace == "yva12") {
      frame_size = image_width * image_height;
      buff_size = 2*frame_size + (frame_size/4)*2; // 4Y + 1U + 1V + Alpha
      color = YVA12;
    }

    return SUCCESS;
  }
  
  Buffer *MultipleOSDNode::processBuffer(Buffer *) {
    Buffer *out;
    
    osd_mutex.lock();
    while ( new_text_flag == false ) {
      DEBUG_STREAM("Waiting for DRAW..." << endl);
      osd_cond->wait();
      DEBUG_STREAM("awake for DRAW" << endl );
    }
    
    if ((out = getNewBuffer( buff_size )) != 0) {
      out->setUsed( buff_size ); 
      memset(out->getData(), 0, buff_size);
               
      for(items_t::iterator i = items.begin(); i != items.end(); ++i) {
	DEBUG_STREAM("DRAWING ... " << i->first << endl);
	if(i->second->enabled) {
	  i->second->draw( out->getData(), color, image_width, image_height );
	}
	DEBUG_STREAM("DRAWED ... " << endl);
      }
      
       
     
    }
    new_text_flag = false;

    osd_mutex.unlock();
    return out;
  }

  
}
