/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:  David Maass (dama@graphics.cs.uni-sb.de)       
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_OSDELEMENTCOMPOSIT_HPP
#define _NMM_OSDELEMENTCOMPOSIT_HPP

#include "nmm/interfaces/video/osd/manager/IOSDManager.hpp"
#include "nmm/interfaces/video/osd/manager/IWidgetFactory.hpp"

#include "OSDElementBase.hpp"

namespace NMM {
  /**
   * OSD Element Composit
   */
  class OSDElementComposit:
    public OSDElementBase {

  public:

    virtual ~OSDElementComposit();
    typedef struct {
      unsigned int  xpos;
      unsigned int  ypos;
      unsigned int  width;
      unsigned int  height;
      unsigned int  win_active_color;
      unsigned int  win_inactive_color;
      unsigned int  sel_active_color;
      unsigned int  sel_active_color2;
      unsigned int  sel_inactive_color;
      string        name;
    } win_properties_t;

    virtual Result selectLeft()  = 0;
    virtual Result selectRight() = 0;
    virtual Result selectUp();
    virtual Result selectDown();

    int addElement(OSDElementBase*);
    void removeElement(OSDElementBase*);
    void removeElement(int);

    virtual void setWinProperties(int, win_properties_t* ) = 0;
    virtual void createWindows() = 0;
    
    void update();
    void show();
    void hide();
    inline bool isHidden() {return hidden;};

  protected:
    IOSDManager_var    manager;
    IWidgetFactory_var factory;
    vector<OSDElementBase*> elements;
    vector<OSDElementBase*>::iterator pos;
    bool hidden;
    uint focus;
  };
}

#endif
