/* NMM - Network-Integrated Multimedia Environment
 *
 * Copyright (C) 2002 Network-Multimedia Workgroup,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_CONVERTER_HPP
#define _NMM_CNVERTER_HPP

extern "C" {
  /**
   * converts rgba bitmap to yv12a
   * \param yuv_buffer pointer to destination buffer (make sure enough buffer space is allocated)
   * \param rgb_buffer pointer to rgba bitmap
   * \param width width of the bitmap
   * \param height height of the bitmap
   */
  void RGBAtoYV12A( unsigned char* yuv_buffer, unsigned char* rgbbuffer, int width, int height);
  /**
   * computes the Y-component of an YUV-image from RGB-values
   * \param R red color component
   * \param G green color component
   * \param B blue color component
   * \return Y color component 
  */
  double getYComponent( unsigned char R, unsigned char G, unsigned char B );
  /**
   * computes the U-component of an YUV-image from RGB-values
   * \param R red color component
   * \param G green color component
   * \param B blue color component
   * \return U color component
   */
  double getUComponent( unsigned char R, unsigned char G, unsigned char B );
  /**
   * computes the V-component of an YUV-image from RGB-values
   * \param R red color component
   * \param G green color component
   * \param B blue color component
   * \return V color component
   */
  double getVComponent( unsigned char R, unsigned char G, unsigned char B );

}
#endif

