/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "plane.hpp"

namespace NMM {

  /*****************************************************************************************/
  /* PlaneDecorator									 */
  /*****************************************************************************************/
  PlaneDecorator::PlaneDecorator() {
    setPlaneColor( 0xff000000 );
  }
  
  void PlaneDecorator::setPlaneColor( const u_int32_t& color ) {
    planecolor = color;
  }
  
  void PlaneDecorator::draw( workspace_t* ws ) {

    drawPlane( ws );
    Decorator::draw( ws );
  }
  
  void PlaneDecorator::drawPlane( workspace_t* ws ) {
    unsigned int* frame = (unsigned int*) ws->data; // pointer to workspace (destination)
    unsigned int  x, y;

    unsigned int v_width, v_height;
    v_width  = adjustDimension( xpos, width, ws->width );
    v_height = adjustDimension( ypos, height, ws->height );

    for( y=ypos; y < (ypos+v_height); y++ ) {
      for( x=xpos; x < (xpos+v_width); x++ ) {
	frame[x+y*ws->stride]  = planecolor;
      }
    }
  }

}
