/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_CALLBACK_HPP_
#define _NMM_CALLBACK_HPP_ _NMM_CALLBACK_HPP_


namespace NMM {

  /**
  * Class to handle callback functions
  */
  class Callback {
    public:
      virtual ~Callback() {};

      /**
      * we need only a function to start the callback
      */
      virtual void run( void* ) {};
  };

  /**
  * a callback function consists of a class and a member function,
  * to handle the class name, we need a template
  */
  template<class cl>
  class TCallback : public Callback {
    /* the callback function gets a void pointer to pass data to it */
    typedef void (cl::*callbackfunc_t) ( void* );

    protected:
      /* pointer to the class itself */
      cl*              classdata;

      /* offset to member function */
      callbackfunc_t   callbackfunc;

    public:
      TCallback( cl* cldata, callbackfunc_t cbfunc ) {
        classdata    = cldata;
	callbackfunc = cbfunc;
      };

      void run( void* w ) {
        ((classdata)->*(callbackfunc))( w );
      };


  };



} // NMM

#endif //_NMM_CALLBACK_HPP_
