/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */

/*
 * Code is based on spu_mixer.c that is part of Ogle, a free linux DVD player.
 * Copyrighted 2000-2001 by Bjrn Englund, Hkan Hjort
 */

#include <stdio.h>
#include <iostream>
#include "SPUClass.hpp"
#include <string.h>

SPUDecoder::SPUDecoder() {
  image = NULL;
  /* fill color scheme with zero */
  memset( color_scheme, 0, sizeof(unsigned int)*4*2 );
  setMode( RGBA );
}

SPUDecoder::~SPUDecoder() {
}

void SPUDecoder::setMode( color_mode m ) {
  mode = m;
}

SPUDecoder::color_mode SPUDecoder::getMode() {
  return( mode );
}

unsigned int SPUDecoder::getWidth() {
  return width;
}

unsigned int SPUDecoder::getHeight() {
  return height;
}

unsigned int SPUDecoder::getXPosition() {
  return x_start;
}

unsigned int SPUDecoder::getYPosition() {
  return y_start;
}

bool SPUDecoder::isCSOk() {
  return( cs_ok );
}

bool SPUDecoder::isMenu() {
  return( is_menu );
}

void SPUDecoder::initColorScheme( unsigned int scheme_nr, unsigned int state_nr, unsigned int value ) {
  color_scheme[scheme_nr][state_nr] = value;
}

void SPUDecoder::setColorScheme( unsigned int scheme_nr, unsigned int state_nr ) {
  unsigned int value = color_scheme[scheme_nr][state_nr];

  alpha[0] = value & 0xf;
  alpha[1] = (value >> 4) & 0xf;
  alpha[2] = (value >> 8) & 0xf;
  alpha[3] = (value >> 12) & 0xf;

  color[0] = (value >> 16) & 0xf;
  color[1] = (value >> 20) & 0xf;
  color[2] = (value >> 24) & 0xf;
  color[3] = (value >> 28);
}


unsigned char* SPUDecoder::decode( unsigned char** packet, int* size, unsigned int* palette ) {
  int ret;

  ret = decodeCS( *packet, *size );
  *size = ret;

  if ( cs_ok ) {
    allocateMemory();
    decodeBitmap( *packet, palette, image );
  }
  if ( ret >= 0 ) *packet += spu_size;

  return( image );
}

int SPUDecoder::decodeCS( unsigned char* packet, unsigned int size ) {
  unsigned char command;
  unsigned int  temp;
  unsigned int  next_cs_offset;
  unsigned int  last_cs_offset;
  unsigned int  start_time;

  cs_ok = false;
  is_menu = false;

  /* set data pointer to begin of the packet */
  setByte( packet );

  /* first 2 bytes contains the spu size */
  spu_size = getBytes(2);
  //cout << "SPU Size: " << spu_size << endl;
  if ( spu_size == 0 ) return( size-2 );
  if ( spu_size > size ) return( size-spu_size );

  /* get offset to the Control Sequence */
  cs_offset = getBytes(2);

  next_cs_offset = cs_offset;

  do {
    last_cs_offset = next_cs_offset;
    //cout << "CS offset: " << last_cs_offset << endl;
    if ( last_cs_offset > spu_size ) return( size-spu_size );


    /* set data pointer to begin of the control sequence */
    setByte( packet+next_cs_offset );

    /* first 2 bytes of the control sequence contains the start time */
    start_time = getBytes(2);
    //cout << "Start Time: " << start_time << endl;

    next_cs_offset = getBytes(2);
    while( (command = getBytes(1)) != 0xff ) {
      //cout << "Display Control Command: " << (int) command;

      switch( command ) {
        case 0x00: /* Menu */
          //cout << "  Menu" << endl;
	  is_menu = true;
        break;

        case 0x01: /* display start */
          //cout << "  display start" << endl;
        break;

        case 0x02: /* display stop */
          //cout << "  display stop" << endl;
        break;

        case 0x03: /* set colors */
          //cout << "  set colors" << endl;
	  temp = getBytes(2);
	  color[0] = temp & 0xf;
	  color[1] = (temp>>4) & 0xf;
	  color[2] = (temp>>8) & 0xf;
	  color[3] = (temp>>12);
        break;

        case 0x04: /* set alpha */
          //cout << "  set alpha" << endl;
	  temp = getBytes(2);
	  alpha[0] = temp & 0xf;
	  alpha[1] = (temp>>4) & 0xf;
	  alpha[2] = (temp>>8) & 0xf;
	  alpha[3] = (temp>>12);
        break;

        case 0x05: /* set screen position and dimension */
	{
	  temp = getBytes(3);
	  x_start            = temp>>12;
	  unsigned int x_end = temp & 0xfff;

	  temp = getBytes(3);
	  y_start            = temp>>12;
	  unsigned int y_end = temp & 0xfff;

	  width  = x_end - x_start + 1;
	  height = y_end - y_start + 1;

	  real_width  = width;
	  real_height = height;

	  if ( mode == YV12A ) {
            if ( width%2 ) width++;
            if ( height%2 ) height++;
	  }

	  //cout << "  Position: " << x_start << ", " << y_start << " Dimension: " << width << ", " << height << endl;
	}
        break;

        case 0x06: /* set offset of fiedls */
          //cout << "  set offset of fields" << endl;
	  fieldoffset[0] = getBytes(2);
	  fieldoffset[1] = getBytes(2);
        break;

        case 0x07: /* wipe */
        {
          unsigned int type;
          //cout << "  wipe" << endl;

	  temp = getBytes(2);
	  temp = getBytes(2);
	  temp = getBytes(2);
	  type = (temp >> 12) & 0x0f;

	  temp = getBytes(2);
	  temp = getBytes(2);
	  temp = getBytes(2);

	  if ( type == 2 ) {
	    temp = getBytes(2);
	    temp = getBytes(2);
	    temp = getBytes(2);
	  }

	  if ( type == 1 || type == 2 ) {
	    temp = getBytes(2);
	    temp = getBytes(2);
	  } else {
	    //cout << "  unknown cmd 07 type" << endl;
	  }

        }
        break;

        default:
          //cout << "  unkonown Display Control Command" << endl;
	  return(size-spu_size);
        break;
      }//switch
    } //while
  } while( next_cs_offset != last_cs_offset );

  cs_ok = true;
  return( size-spu_size );
}

void SPUDecoder::drawRGBA( unsigned int* addr, unsigned int length, unsigned int color, unsigned int alpha ) {
  color |= alpha<<24;
  for( unsigned int i=0; i<length; i++ )
    addr[i] = color;
}

void SPUDecoder::drawYV12A( unsigned char* plane1, unsigned char* plane2, unsigned char* plane3, unsigned char* plane4,
		            unsigned int field, unsigned int length, unsigned int color, unsigned int alpha ) {
  char y = color>>16 & 0xff;

  for( unsigned int i=0; i<length; i++ ) {
    plane1[i] = y;
    plane4[i] = alpha;
  }

  if ( field == 1 ) {
    char u = color>>8 & 0xff;
    char v = color & 0xff;
    unsigned int odd = (unsigned int) plane1 & 1;
    for( unsigned int i=odd; i<=((length+1-odd)>>1); i++ ) {
      plane2[i] = u;
      plane3[i] = v;
    }
  }

}

bool SPUDecoder::allocateMemory() {
  if ( mode == RGBA ) {
    image = new unsigned char[width*height*4];
  }
  else {
    unsigned int t = width*height;
    unsigned int size = (unsigned int) (t*2.5);
    image = new unsigned char[size]; // 1*y_plane + 1/4*u_plane + 1/4*v_plane + 1*alpha
  }

  if ( image ) return( true );
  else return ( false );
}

void SPUDecoder::decodeBitmap( unsigned char* packet, unsigned int* palette, unsigned char* dimage ) {
  unsigned char* plane1 = dimage;
  unsigned char* plane2 = dimage;
  unsigned char* plane3 = dimage;
  unsigned char* plane4 = dimage;

  if ( mode == RGBA ) {
    plane1 = dimage;
    plane2 = NULL;
    plane3 = NULL;
    plane4 = NULL;
  }
  else {
    unsigned int t = width*height;
    plane1 = dimage;
    plane2 = dimage + t;
    plane3 = plane2 + (t>>2);
    plane4 = plane3 + (t>>2);
    memset(plane4, 0, t);
  }

  unsigned int x = 0;
  unsigned int y = 0;
  field = 0;

  setByte( packet+fieldoffset[0] );
  while( (fieldoffset[1] < cs_offset) && (y < real_height) ) {
    unsigned int vlc;
    unsigned int length;
    unsigned int colorid;
    unsigned int p_color;
    unsigned int p_alpha;

    vlc = getNibble();
    if ( vlc < 0x4 ) {
      vlc = (vlc << 4) | getNibble();
      if ( vlc < 0x10 ) {
        vlc = (vlc << 4) | getNibble();
	if ( vlc < 0x40 ) {
	  vlc = (vlc << 4) | getNibble();
	}
      }
    }

    length = vlc>>2;

    if ( length == 0 ) {
      /* fill current line with color */
      length = width-x;
    }

    if ( length+x > width ) {
      length = width-x;
    }

    colorid = vlc&3;

    p_color = color[colorid];
    p_alpha = (alpha[colorid]<<4);

    /* at this point we have a color in p_color that points to a palette entry (0..15) */
    /* and a alpha value (0..255) */
    /* we must draw the color p_color with alpha p_alpha 'length' times in the actual line (RLE coded)*/
    unsigned int real_color = palette[p_color];
    if ( mode == RGBA ) drawRGBA( (unsigned int* ) (plane1+(x<<2)), length, real_color, p_alpha );
    else drawYV12A( plane1+x, plane2+(x>>1), plane3+(x>>1), plane4+x,
		    field, length, real_color, p_alpha );


    x = x + length;
    if ( x >= real_width ) {
      x = 0;
      y++;

      if ( mode == RGBA ) {
        plane1 += width<<2; // next line
      } else {
        plane1 += width;
	if ( field == 1 ) {
	  plane2 += width>>1;
	  plane3 += width>>1;
	}
	plane4 += width;
      }

      field = 1-field;
      setByte( packet+fieldoffset[field] );
    }
  } //while

}

void SPUDecoder::setByte( unsigned char* byte ) {
  byte_pos = byte;
  aligned = true;
}

unsigned int SPUDecoder::getBytes( unsigned int num ) {
  unsigned int result = 0;

  if( num>4 ) {
    //cout << "getBytes used with more than 4 bytes" << endl;
    return( result );
  }

  while( num > 0 ) {
    result <<= 8;
    result |= *byte_pos;
    byte_pos++;
    num--;
  }

  return( result );
}

unsigned char SPUDecoder::getNibble() {
  if ( aligned ) {
    fieldoffset[field]++;
    next = getBytes(1);
    aligned = false;
    return next>>4;
  } else {
    aligned = true;
    return next&0xf;
  }
}
