/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "Benchmark.hpp"

namespace NMM {

  Benchmark::Benchmark(NamedObject *obj , const char* msg )
  {
    setNamedObject(obj);
    setMessage(msg);
    fps_looptimer1 = 0;
    fps_looptimer2 = 0;
    enabled = false;
    frame_count = total_bytes = fps_looptimer_diff = bpf = bps = 0;
    fps = 0.0;
  }

  Benchmark::~Benchmark() 
  {
    destroyLoopTimer(fps_looptimer1);
    destroyLoopTimer(fps_looptimer2);
  }
    
  void Benchmark::setNamedObject(NamedObject *obj)
  {
    master = obj;
  } 

  void Benchmark::setMessage(const char *msg) 
  {
    if(msg != 0) {
      message = msg;
    }
    else {
      message = "Benchmark>";
    }
  }

  void Benchmark::setEnabled(bool enabled) 
  {
    this->enabled = enabled; 
    if(enabled) {
      frame_count = total_bytes = fps_looptimer_diff = bpf = bps = 0;
      fps = 0.0;
      if(!fps_looptimer1) fps_looptimer1 = createLoopTimer();
      if(!fps_looptimer2) fps_looptimer2 = createLoopTimer();
      startLoopTimer(fps_looptimer1);
    }
  }

  void Benchmark::benchmark(const long used_bytes)
  {
    if(enabled) {
      ++frame_count;
      total_bytes += used_bytes;
      if((frame_count % 10) == 0) {
	startLoopTimer(fps_looptimer2);
	fps_looptimer_diff =  millisDifference(fps_looptimer1, fps_looptimer2);
	fps = (10000.0 / ((float)fps_looptimer_diff));
	bps = (long) ((1000.0 * (float) total_bytes) / (float) fps_looptimer_diff);
	bpf = (total_bytes / 10);
	if(master != 0) {
	  NODE_DEBUG_STREAM(master, " " << dec << message 
	  << " last 10 frames with " << bpf
	  << " bytes/frame, " << fps
	  << " fps, " << bps << " bytes/s " << endl);
	}
	else {
	  GLOBAL_MESSAGE_STREAM( " " << dec << message 
	  << " last 10 frames with " << bpf 
	  << " bytes/frame, " << fps
	  << " fps, " << bps << " bytes/s " << endl);
	}
	startLoopTimer(fps_looptimer1);
	total_bytes = 0;
      }
    }
  }
}


