/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef WEBSERVER_HPP
#define WEBSERVER_HPP WEBSERVER_HPP

#include "WebServerProcessor.hpp"
#include "WebRequestHandler.hpp"
#include "Socket.hpp"
#include "nmm/utils/net/SockAcceptor.hpp"
#include "nmm/utils/thread/ThreadedObject.hpp"

namespace NMM{
  /**
   * This class implements an embedded webserver to include in applications.
   */

  class WebServer : public ThreadedObject{
  private:
    unsigned int serverPort;
    list<WebServerProcessor*> handlerThreads;
    int clientHandlers;
    bool allowAll;
    SockAcceptor *server;
    bool running;
    
  public:
    WebServer(unsigned int port);
    ~WebServer();
    
    void start();
    void stop();
    
    void setNumberOfClientHandlers(int number);
    int getNumberOfClientHandlers();
    
    void enableAllowAll();
    void disableAllowAll();
    bool getAllowAll();
    
    void registerHandler(WebRequestHandler *handler);
    
  protected:
    void run();
  };
}
#endif
  
