/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef WEBSERVERPROCESSOR_HPP
#define WEBSERVERPROCESSOR_HPP WEBSERVERPROCESSOR_HPP

#include "WebRequestHandler.hpp"
#include <string>
#include <list>
#include <map>
#include "Socket.hpp"
#include "nmm/utils/thread/ThreadedObject.hpp"

namespace NMM{

  class WebServerProcessor : public ThreadedObject{
  private:
    static list<Socket*> clientRequestPool;
    static map<string, WebRequestHandler*> handlers;
    static ThreadMutex mutex;
    static ThreadCondition tcond;

    ThreadMutex accessMutex;

    bool keepRunning;
    string threadName;
    
  public:
    WebServerProcessor(unsigned int threadNr);
    ~WebServerProcessor();
    
    static void processRequest(Socket *request);
    static void registerHandler(WebRequestHandler* handler);
    void notifyProcessors();
    
    
    void stop();
    void start();
  protected:
    void run();
    
  };
}
#endif
