/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "EventPlayer.hpp"
#include "nmm/base/serialize/StdStream.hpp"
#include "nmm/base/Event.hpp"

#include <fstream>

namespace NMM {
  EventPlayer::EventPlayer(const string& file, long delay)
    :  __value_stream (0), __std_i_stream (0), __i_stream (0), __delay(delay)
  {
    /* create the __o_stream to read the events */
    __i_stream = new std::ifstream(file.c_str());
    if (! *__i_stream )
      throw Exception("Could not open file " + file );


    string i_value_stream_type;
    (*__i_stream) >> i_value_stream_type;
    i_value_stream_type = SValueFactory::getInstance().getOValueStreamPartner(i_value_stream_type);
    /* create an instance of the desired value_stream */
    __value_stream = SValueFactory::getInstance().createIValueStream(i_value_stream_type);
    if (!__value_stream)
      throw Exception("Unknown ValueStream " + i_value_stream_type);

    __std_i_stream = new StdIStream(__i_stream);

    /* set the partner stream */
    __value_stream->setIPartnerStream(__std_i_stream);
    __value_stream->readData();

    readEvents();
  }

  EventPlayer::~EventPlayer() {
    /* Begin> delete allocated objects */
    delete __value_stream;
    delete __std_i_stream;
    delete __i_stream;
    /* End< */

    /* Delete the events */
    for (std::list<CEvent*>::iterator it = __event_list.begin(); it != __event_list.end(); ++it)
      delete *it;
  }


  void EventPlayer::readEvents() {
    while (! __value_stream->isEmpty() ) {
      /* We currently only uses CEvents */
      CEvent* event = new CEvent();

      /* deserialize the events */
      event->createFromStream(*__value_stream);

      /* store the event */
      __event_list.push_back(event);
    }

    /* Start time of the first event */
    Time start_time;

    if (!__event_list.empty())
      start_time = __event_list.front()->getTimestamp().sync_time;

    /* Begin> Calculate the playback time for each event and add it to the */
    for (std::list<CEvent*>::iterator it = __event_list.begin(); it != __event_list.end(); ++it ) {
      /* Calculate the difference between the first timestamp and this one */
      Time time_to_wait = (*it)->getTimestamp().sync_time - Interval(start_time.sec, start_time.nsec);
      /* get the current time */
      time_t current_time = time(0);
      /* register the event */
      for (unsigned int i = 0; i < (*it)->size(); ++i) {
        __producer.registerTimer( current_time + time_to_wait.sec + __delay, &(*it)->getEvent(i) );
      }
    }
    /* End< */
  }
}
