#ifndef _NMM_ICLICVALUESTREAM_HPP_
#define _NMM_ICLICVALUESTREAM_HPP_

#include <string>
#include <list>
#include <vector>
#include "nmm/nmm_strstream.hpp"
#include "nmm/base/serialize/ValueStream.hpp"

namespace NMM {
class Event;
class IClicValueStream : public IValueStream {
public:
/** @name Constructor & Destructor
  */
//@{
/** Constructor
  */
  IClicValueStream(const string& event_key, const vector<string>& argument_stack );

/** Destructor
  */
  virtual ~IClicValueStream() {}

/** Virtual copy constructor
  */
  virtual IClicValueStream* copy() const ;
//@}

/** @name The operators
  */
//@{
/** This method reads an u_int16_t i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(u_int16_t&)       ;

/** This method reads an int16_t i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(int16_t&)         ;

/** This method reads an u_int32_t i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(u_int32_t&  )     ;

/** This method reads an int32_t i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(int32_t&  )       ;

/** This method reads an u_int64_t i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(u_int64_t&  )     ;

/** This method reads an int64_t i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(int64_t&  )       ;

/** This method reads a bool i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(bool&  )          ;

/** This method reads an unsigned long i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(unsigned long&)   ;

/** This method reads a long i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(long& )           ;

/** This method reads a double i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(double& )         ;

/** This method reads a float i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(float& )          ;

/** This method reads a char i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(char& )           ;

/** This method reads an unsigned char i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(unsigned char&)   ;

/** This method reads a string i from the stream and returns a reference to the stream.
  */
  virtual  IClicValueStream& operator>>(string&)          ;
//@}

/** @name Navigate through the attributes
  */
//@{
/** This method accesses the internal attributes of complex structure.
  */
  virtual bool goDown()    ;

/** This method leaves the internal attributes of complex structure.
  */
  virtual void goUp()      ;

/** This method returns true if no values are in the ValueStream.
  */
  virtual bool isEmpty()    const ;

/** This method reads all information from the next element
  */
  virtual bool nextElement() ;
//@}

/** @name Navigate through the attributes
  */
//@{
/** This method returns the current type information in a TypeInfo struct
  */
  virtual  const TypeInfo& getCurrentTypeInfo() const ;

/** This method returns the type name of the current element. The Stream must not be empty!
  * \sa isEmpty();
  */
  virtual const string& getCurrentType() const ;

/** This method returns the namespace of the current element. The Stream must not be empty!
  * \sa isEmpty();
  */
  virtual const string& getCurrentNamespace() const ;

/** This method returns the type name of this IClicValueStream.
  */
  virtual const string& getIValueStreamType() const ;
//@}
  virtual void readBuffer(char *, unsigned int);
  virtual void readBuffer(unsigned char *, unsigned int);
  inline  const string& getEventKey() const { return __event_key; }
  void readEvent(Event& event);
private:
  void cutBlanks(string& s );
private:
  unsigned int          __current_pos;
  string                __event_key;
  vector< string >      __argument_stack;
  static const TypeInfo __empty_type_info;
  static const string   __stream_type;
};


}

#endif // _NMM_IClicValueStream_HPP_


