/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *                    NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "CPUMonitor.hpp"
#include <cstdio>
#include <sstream>

namespace NMM {

  CPUMonitor::CPUMonitor(pid_t t_pid) : NamedObject("CPUMonitor") {
    mode = CPU;
    thread_pid = t_pid;
    sprintf(proc_cpu_file,"/proc/%i/cpu",t_pid);
    cpu_info = new ifstream(proc_cpu_file);
    if (! *cpu_info) { // if /proc/`pid`/cpu does not exist (kernel > 2.4.19) open  /proc/`pid`/stat
      sprintf(proc_cpu_file,"/proc/%i/stat",t_pid);
      cpu_info = new ifstream(proc_cpu_file);
      mode = STAT;
    }
  }

  CPUMonitor::~CPUMonitor() {
    cpu_info->close();
    delete cpu_info;
  }

  bool CPUMonitor::isThreadExisting() {
    // check if the thread exists
    stringstream name;
    name << "/proc/" << thread_pid;
    if (access(name.str().c_str(), R_OK) != 0)
      return false;

    return true;
  }

  const unsigned int CPUMonitor::getSystemTime() {
//  testing for existence takes too much time
//    if (!isThreadExisting())
//      return 0;

    unsigned int system_time   = 0;
    char tmp[16];

    cpu_info->seekg(0,ios::beg);
    cpu_info->sync();

    switch (mode) {
      case CPU:
        *cpu_info >> tmp; // accumulated time on all processors (entry cpu)
        *cpu_info >> tmp; // system time
        *cpu_info >> system_time; // user time
        break;
      case STAT:
        for (int i=0; i< 14; i++) { // skip the first 13 entrys
          *cpu_info >> tmp;
        }
        *cpu_info >> system_time;
        break;
    }
    //DEBUG_STREAM2("system-time of thread " << thread_pid << " was " << system_time << endl);

    return system_time;
  }

  const unsigned int CPUMonitor::getUserTime() {
//  testing for existence takes too much time
//    if (!isThreadExisting())
//      return 0;

    unsigned int user_time = 0;
    char tmp[16];
    cpu_info->seekg(0,ios::beg);
    cpu_info->sync();

    switch (mode) {
      case CPU:
        *cpu_info >> tmp; // accumulated time on all processors (entry cpu)
        *cpu_info >> user_time; // system time
        break;
      case STAT:
        for (int i=0; i< 13; i++) { // skip the first 12 entrys
          *cpu_info >> tmp;
        }
        *cpu_info >> user_time;
        break;
    }

    //DEBUG_STREAM2("user-time of thread " << thread_pid << " was " << user_time << endl);

    return user_time;
  }

}

