/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "ThreadPool.hpp"
#include <iostream>

namespace NMM {
  ThreadPool::ThreadPool() :
  __thread_condition(__thread_mutex), __allocated_threads(3), __notify(false)
  {
    for (unsigned int i = 0; i< __allocated_threads; ++i)
      __thread_pool.push_back(new WorkerThread());
  }

  ThreadPool::~ThreadPool() {
    __thread_mutex.lock();
    __notify = true;

    while ( __allocated_threads != __thread_pool.size() ) {
      GLOBAL_DEBUG_STREAM("Wait until all threads have been releeased" << endl);
      __thread_condition.wait();
    }
    GLOBAL_DEBUG_STREAM("All threads have been released" << endl);

    /* Delete all allocated threads */
    for (list<WorkerThread*>::iterator it = __thread_pool.begin(); it != __thread_pool.end(); ++it)
      delete *it;
  }

  WorkerThread* ThreadPool::requestThread() {
    MutexGuard mg(__thread_mutex);
    WorkerThread* free_thread = 0;

    if (__thread_pool.empty()) {
      /* increment the internal counter */
      ++__allocated_threads;
      GLOBAL_DEBUG_STREAM("Must spawn a new Thread" << endl);
      free_thread = new WorkerThread();
    } else {
      free_thread = __thread_pool.front();
      __thread_pool.erase(__thread_pool.begin());
    }
    /* enable automatic release */
    free_thread->setReleaseFlag(true);
    return free_thread;
  }

  void ThreadPool::releaseThread( WorkerThread* returning_thread) {
    MutexGuard mg(__thread_mutex);
    __thread_pool.push_back(returning_thread);
    if (__notify) {
      __thread_condition.notify();
    }
  }

}

